/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

/**
 * Frame view for the DatabaseChooserController to switch between databases
 */
public class DatabaseChooserView extends JFrame {
	private static final long serialVersionUID = -5398220319564659170L;

	private DatabaseChooserController controller;

	public DatabaseChooserView(DatabaseChooserController controller) {
		this.controller = controller;

		initComponents();

		initListeners();

		this.setTitle("Login Settings");
		this.setIconImage(new ImageIcon("resources/WPISuite-128x128.png")
				.getImage());

		// Get the current screen size to center the window
		int sizex = getSize().width;
		int sizey = getSize().height;
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension scrnsize = toolkit.getScreenSize();
		this.setLocation((scrnsize.width - sizex) / 2,
				(scrnsize.height - sizey) / 2);

		this.setVisible(true);
	}

	private void initListeners() {
		applyButton.addActionListener(applyListener);
		cancelButton.addActionListener(cancelListener);
		this.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosed(WindowEvent e) {
				controller.onClose();
			}

			@Override
			public void windowClosing(WindowEvent e) {
				controller.onClose();
			}
		});
	}

	private ActionListener applyListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			controller.applySettings();
		}
	};

	private ActionListener cancelListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			controller.cancel();
		}
	};

	public String getUrl() {
		return urlTextField.getText();
	}

	public void setUrl(String url) {
		urlTextField.setText(url);
	}

	public String getUsername() {
		return usernameTextField.getText();
	}

	public void setUsername(String username) {
		usernameTextField.setText(username);
	}

	public String getPassword() {
		return new String(passwordField.getPassword());
	}

	public void setPassword(String password) {
		passwordField.setText(password);
	}

	public void close() {
		this.dispose();
	}

	public void error(String string, Exception e) {
		JOptionPane.showMessageDialog(null, string, "Error!",
				JOptionPane.ERROR_MESSAGE);
	}

	public void showMessage(String string) {
		JOptionPane.showMessageDialog(null, string, "Information",
				JOptionPane.INFORMATION_MESSAGE);
	}

	public boolean askBoolean(String string) {
		return JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(null,
				string, "Confirm action", JOptionPane.YES_NO_OPTION);
	}

	private void initComponents() {

		databaseSettingsPanel = new javax.swing.JPanel();
		urlLabel = new javax.swing.JLabel();
		urlTextField = new javax.swing.JTextField();
		usernameLabel = new javax.swing.JLabel();
		usernameTextField = new javax.swing.JTextField();
		passwordLabel = new javax.swing.JLabel();
		passwordField = new javax.swing.JPasswordField();
		cancelButton = new javax.swing.JButton();
		applyButton = new javax.swing.JButton();

		setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
		setResizable(false);

		databaseSettingsPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Database Settings"));

		urlLabel.setText("URL");
		urlLabel
				.setToolTipText("For example, mysql://mysql.myserver.com/mydbname");
		urlTextField
				.setToolTipText("For example, mysql://mysql.myserver.com/mydbname");

		usernameLabel.setText("Username");

		passwordLabel.setText("Password");

		javax.swing.GroupLayout databaseSettingsPanelLayout = new javax.swing.GroupLayout(
				databaseSettingsPanel);
		databaseSettingsPanel.setLayout(databaseSettingsPanelLayout);
		databaseSettingsPanelLayout
				.setHorizontalGroup(databaseSettingsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								databaseSettingsPanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												databaseSettingsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING,
																false)
														.addComponent(
																urlLabel,
																javax.swing.GroupLayout.Alignment.TRAILING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																passwordLabel,
																javax.swing.GroupLayout.Alignment.TRAILING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																usernameLabel,
																javax.swing.GroupLayout.Alignment.TRAILING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												10,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addGroup(
												databaseSettingsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																urlTextField,
																javax.swing.GroupLayout.Alignment.TRAILING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																290,
																Short.MAX_VALUE)
														.addComponent(
																usernameTextField,
																javax.swing.GroupLayout.Alignment.TRAILING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																290,
																Short.MAX_VALUE)
														.addComponent(
																passwordField,
																javax.swing.GroupLayout.Alignment.TRAILING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																290,
																Short.MAX_VALUE))
										.addContainerGap()));
		databaseSettingsPanelLayout
				.setVerticalGroup(databaseSettingsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								databaseSettingsPanelLayout
										.createSequentialGroup()
										.addGroup(
												databaseSettingsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(urlLabel)
														.addComponent(
																urlTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												databaseSettingsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																usernameLabel)
														.addComponent(
																usernameTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												databaseSettingsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																passwordLabel)
														.addComponent(
																passwordField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addContainerGap(
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));

		cancelButton.setText("Cancel");

		applyButton.setText("Apply");

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		getContentPane().setLayout(layout);
		layout
				.setHorizontalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																databaseSettingsPanel,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addGroup(
																javax.swing.GroupLayout.Alignment.TRAILING,
																layout
																		.createSequentialGroup()
																		.addComponent(
																				applyButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				cancelButton)))
										.addContainerGap()));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												databaseSettingsPanel,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																cancelButton)
														.addComponent(
																applyButton))
										.addContainerGap()));

		pack();
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JButton applyButton;
	private javax.swing.JButton cancelButton;
	private javax.swing.JPanel databaseSettingsPanel;
	private javax.swing.JPasswordField passwordField;
	private javax.swing.JLabel passwordLabel;
	private javax.swing.JLabel urlLabel;
	private javax.swing.JTextField urlTextField;
	private javax.swing.JLabel usernameLabel;
	private javax.swing.JTextField usernameTextField;
	// End of variables declaration

}
