/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bryan Crabree - GUI Design
 *    Alex Kindle - GUI Design
 *    Ivo Dobrev - Implementation
 *    Ravi Singh - Implementation
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.awt.Dimension;
import java.awt.Toolkit;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.funcs.LoginFunctions;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.module.ActiveStateEvent.ActiveStateEventType;

/**
 * Class that deals with all details of a simple, standalone popup window to
 * edit one's user's properties
 */
public class EditProfileView extends javax.swing.JFrame {

	private static final long serialVersionUID = -5063205650060995336L;

	protected WPISuite wpiSuite;

	/** Creates new form Profile */
	public EditProfileView(WPISuite wpiSuite) {
		this.wpiSuite = wpiSuite;
		this.initComponents();
		this.initFields();

		this.setTitle("Edit Profile");
		this.setIconImage(new ImageIcon("resources/WPISuite-128x128.png")
				.getImage());

		// Get the current screen size to center the window
		int sizex = 330;
		int sizey = 280;
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension scrnsize = toolkit.getScreenSize();
		this.setSize(sizex, sizey);
		this.setLocation((scrnsize.width - sizex) / 2,
				(scrnsize.height - sizey) / 2);
	}

	// <editor-fold defaultstate="collapsed" desc="Generated Code">
	private void initComponents() {

		profilePanel = new javax.swing.JPanel();
		profileEmailLabel = new javax.swing.JLabel();
		profileEmailTextField = new javax.swing.JFormattedTextField();
		profileApplyChangesButton = new javax.swing.JButton();
		profileCancelChangesButton = new javax.swing.JButton();
		profileNewPasswordLabel = new javax.swing.JLabel();
		profileCurrentPasswordLabel = new javax.swing.JLabel();
		profileCurrentPasswordField = new javax.swing.JPasswordField();
		profileFirstNameLabel = new javax.swing.JLabel();
		profileFirstNameTextField = new javax.swing.JFormattedTextField();
		profileLastNameLabel = new javax.swing.JLabel();
		profileLastNameTextField = new javax.swing.JFormattedTextField();
		profileNewPasswordField = new javax.swing.JPasswordField();
		profileConfirmPasswordLabel = new javax.swing.JLabel();
		profileConfirmPasswordField = new javax.swing.JPasswordField();

		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		setTitle("Profile");
		setAlwaysOnTop(true);
		setResizable(false);

		profilePanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Profile"));

		profileEmailLabel.setText("E-Mail Address");

		profileApplyChangesButton.setText("Apply Changes");
		profileApplyChangesButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						profileApplyChangesButtonActionPerformed(evt);
					}
				});

		profileCancelChangesButton.setText("Cancel Changes");
		profileCancelChangesButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						profileCancelChangesButtonActionPerformed(evt);
					}
				});

		profileNewPasswordLabel.setText("New Password");

		profileCurrentPasswordLabel.setText("Current Password");

		profileFirstNameLabel.setText("First Name");

		profileLastNameLabel.setText("Last Name");

		profileConfirmPasswordLabel.setText("Confirm Password");

		javax.swing.GroupLayout profilePanelLayout = new javax.swing.GroupLayout(
				profilePanel);
		profilePanel.setLayout(profilePanelLayout);
		profilePanelLayout
				.setHorizontalGroup(profilePanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								profilePanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												profilePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																profilePanelLayout
																		.createSequentialGroup()
																		.addGroup(
																				profilePanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.TRAILING)
																						.addComponent(
																								profileNewPasswordLabel)
																						.addComponent(
																								profileLastNameLabel)
																						.addComponent(
																								profileFirstNameLabel)
																						.addComponent(
																								profileEmailLabel)
																						.addComponent(
																								profileConfirmPasswordLabel))
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				profilePanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addComponent(
																								profileEmailTextField,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								184,
																								Short.MAX_VALUE)
																						.addComponent(
																								profileFirstNameTextField,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								184,
																								Short.MAX_VALUE)
																						.addComponent(
																								profileLastNameTextField,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								184,
																								Short.MAX_VALUE)
																						.addComponent(
																								profileNewPasswordField,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								184,
																								Short.MAX_VALUE)
																						.addComponent(
																								profileConfirmPasswordField,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								184,
																								Short.MAX_VALUE)))
														.addGroup(
																profilePanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				profileCurrentPasswordLabel)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				profileCurrentPasswordField,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				184,
																				Short.MAX_VALUE))
														.addGroup(
																profilePanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				profileApplyChangesButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				profileCancelChangesButton)))
										.addContainerGap()));
		profilePanelLayout
				.setVerticalGroup(profilePanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								profilePanelLayout
										.createSequentialGroup()
										.addGroup(
												profilePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																profileEmailTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																profileEmailLabel))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												profilePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																profileFirstNameLabel)
														.addComponent(
																profileFirstNameTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												profilePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																profileLastNameTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																20,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																profileLastNameLabel))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												profilePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																profileNewPasswordField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																profileNewPasswordLabel))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												profilePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																profileConfirmPasswordLabel)
														.addComponent(
																profileConfirmPasswordField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												profilePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																profileCurrentPasswordLabel)
														.addComponent(
																profileCurrentPasswordField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.addGroup(
												profilePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																profileApplyChangesButton)
														.addComponent(
																profileCancelChangesButton))
										.addContainerGap()));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				profilePanel, javax.swing.GroupLayout.DEFAULT_SIZE,
				javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				profilePanel, javax.swing.GroupLayout.DEFAULT_SIZE,
				javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE));

		pack();
	}// </editor-fold>

	private void initFields() {
		User currUser = this.wpiSuite.getState().getUser();
		this.profileFirstNameTextField.setText(currUser.getFirstName());
		this.profileLastNameTextField.setText(currUser.getLastName());
		this.profileEmailTextField.setText(currUser.getEmailAddress());
	}

	/**
	 * Function to deal with changes in the user's profile. Also does validation
	 * of the new password using LoginFunctions.validatePassword(String)
	 * 
	 * @param evt
	 */
	private void profileApplyChangesButtonActionPerformed(
			java.awt.event.ActionEvent evt) {
		// TODO add your handling code here:

		String password = String.copyValueOf(this.profileCurrentPasswordField
				.getPassword());
		String firstName = this.profileFirstNameTextField.getText();
		String lastName = this.profileLastNameTextField.getText();
		String email = this.profileEmailTextField.getText();
		String newPassword = String.copyValueOf(this.profileNewPasswordField
				.getPassword());
		String confirmPassword = String
				.copyValueOf(this.profileConfirmPasswordField.getPassword());

		User currUser = this.wpiSuite.getState().getUser();

		boolean editValue = true;

		if (!newPassword.isEmpty())
			if (password.isEmpty()) {
				JOptionPane.showMessageDialog(this,
						"Please provide current password.");
				editValue = false;
			} else if (currUser.getPasswordHash().equals(
					Database.hashPassword(password))) {
				System.out.println("match");
				if (!newPassword.equals(confirmPassword)) {
					JOptionPane
							.showMessageDialog(this,
									"The new and confirm password fields do not match.");
					editValue = false;
				}
				try {
					LoginFunctions.validatePassword(newPassword);
				} catch (Exception e) {
					JOptionPane.showMessageDialog(this, e.getMessage());
					editValue = false;
				}
			} else {
				JOptionPane.showMessageDialog(this, "Incorrect password");
				editValue = false;
			}

		if (editValue) {
			currUser.setFirstName(firstName);
			currUser.setLastName(lastName);
			currUser.setEmailAddress(email);
			currUser.setPasswordHash(Database.hashPassword(newPassword));
			currUser.save();
			this.wpiSuite.getState().fireStateChange(ActiveStateEventType.USER);
			this.dispose();
		}

	}

	private void profileCancelChangesButtonActionPerformed(
			java.awt.event.ActionEvent evt) {
		// TODO add your handling code here:
		this.dispose();
	}

	// Variables declaration - do not modify
	private javax.swing.JButton profileApplyChangesButton;
	private javax.swing.JButton profileCancelChangesButton;
	private javax.swing.JPasswordField profileConfirmPasswordField;
	private javax.swing.JLabel profileConfirmPasswordLabel;
	private javax.swing.JPasswordField profileCurrentPasswordField;
	private javax.swing.JLabel profileCurrentPasswordLabel;
	private javax.swing.JLabel profileEmailLabel;
	private javax.swing.JFormattedTextField profileEmailTextField;
	private javax.swing.JLabel profileFirstNameLabel;
	private javax.swing.JFormattedTextField profileFirstNameTextField;
	private javax.swing.JLabel profileLastNameLabel;
	private javax.swing.JFormattedTextField profileLastNameTextField;
	private javax.swing.JPasswordField profileNewPasswordField;
	private javax.swing.JLabel profileNewPasswordLabel;
	private javax.swing.JPanel profilePanel;
	// End of variables declaration

}
