/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Scanner;


import org.apache.log4j.Logger;

import edu.wpi.wpisuite.module.ActiveState;
import edu.wpi.wpisuite.module.Module;

/**
 * Dynamically finds modules on the classpath, loads them, and provides a
 * Constructor to instantiate an instance of them
 */
public class ModuleLoader {
	Logger logger = Logger.getLogger(this.getClass());

	/**
	 * Instantiates one of each known module type and creates a mapping between
	 * that and the class of the module.
	 * 
	 * @param state
	 *            The state to instantiate the modules with
	 * @return A map from all found module classes to a new instance of that
	 *         class
	 */
	public Map<Class<? extends Module>, Module> initModuleMap(ActiveState state) {
		Map<Class<? extends Module>, Module> modules = new HashMap<Class<? extends Module>, Module>();
		List<Class<? extends Module>> modclasses = getModuleClasses();
		System.out.println("in initmodulemap. loopsize = " + modclasses.size());
		for (Class<? extends Module> clazz : modclasses) {
			Module module;
			
			try {
			    module = clazz.getConstructor(state.getClass()).newInstance(
						state);
				System.out.println(module.getModuleName() + " module initialized.");
				logger.info(module.getModuleName() + " module initialized.");
				modules.put(clazz, module);
			} catch (Exception e) {
			    System.out.println("Construction failed for module with class "
						+ clazz.getSimpleName());
				logger.error("Construction failed for module with class "
						+ clazz.getSimpleName());
				logger.debug(Arrays.toString(e.getCause().getStackTrace()));
			}
		}
		if (modules != null) {
		    System.out.println("modules is NOT null after initModuleMap()");
		    return modules;
		} else {
		    System.out.println("why is modules null?");
		    return modules;
		}
	}

	/**
	 * @return A list of all findable classes that can act as modules. Currently
	 *         just uses a list of modules hard-coded in. In the future, this
	 *         should dynamically load them at run-time.
	 */
	private List<String> getModuleClassNames() {
		List<String> moduleClasses = new ArrayList<String>();

		File dir = new File("./");
		System.out.println("looking in" + dir.getAbsolutePath());
		String[] fileNames = dir.list(new ModuleFilenameFilter());
		Scanner in;
		
		for (String fileName : fileNames) {
			try {
				in = new Scanner(new File(dir+"/"+fileName));
				while (in.hasNextLine())
					moduleClasses.add(in.nextLine());
			} catch (FileNotFoundException e) {
				// This can't happen, unless someone is trying
				e.printStackTrace();
			}
		}

		return moduleClasses;
	}

	/**
	 * @return A list of all findable classes that can act as modules. Currently
	 *         just uses a list of modules hard-coded in. In the future, this
	 *         should dynamically load them at run-time.
	 */
	@SuppressWarnings("unchecked")
	private List<Class<? extends Module>> getModuleClasses() {
		List<Class<? extends Module>> moduleClasses = new ArrayList<Class<? extends Module>>();

		for (String name : getModuleClassNames())
			try {
				Class<?> clazz = getClass().getClassLoader().loadClass(name);
				moduleClasses.add((Class<? extends Module>) clazz);
			} catch (ClassNotFoundException e) {
				logger.error("Could not find class with name " + name);
			}

		return moduleClasses;
	}

}
