/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Set;


import edu.wpi.wpisuite.components.editor.InvalidInputException;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.funcs.LoginFunctions;
import edu.wpi.wpisuite.database.Database;

/**
 * Deals with the logic of forcing users to update expired passwords
 */
public class PasswordExpiredController {
	private User user;
	private PasswordExpiredView view;

	public PasswordExpiredController(User user) {
		this.view = new PasswordExpiredView(this);
		this.user = user;

	}

	public void validateConfirmation(String newPass, String confirmPass)
			throws InvalidInputException {
		if (!newPass.equals(confirmPass))
			throw new InvalidInputException("Passwords do not match");
	}

	public void validatePassword(String newPass) throws Exception {
		LoginFunctions.validatePassword(newPass);
	}

	public void changePassword() {
		this.user.setPasswordHash(Database.hashPassword(view.getPassword()));
		this.user.setPasswordExpired(false);
		this.user.save();

		this.close();
	}

	private void close() {
		view.close();
	}

	private Set<ActionListener> closeListeners = new HashSet<ActionListener>();

	public void addCloseListener(ActionListener actionListener) {
		closeListeners.add(actionListener);
	}

	public void removeCloseListener(ActionListener actionListener) {
		closeListeners.remove(actionListener);
	}

	public void onClose() {
		for (ActionListener l : closeListeners)
			l.actionPerformed(null);
	}
}
