/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bryan Crabree - initial design, API, and implementation
 *    Dan Bjorge - additional implementation (password validation)
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 * A simple, self-contained popup window to deal with forcing a user to change
 * their password if theirs has expired
 */
@SuppressWarnings("serial")
public class PasswordExpiredView extends JFrame {
	private PasswordExpiredController controller;

	public PasswordExpiredView(PasswordExpiredController controller) {
		this.controller = controller;

		this.initComponents();
		this.initListeners();
		this.initPosition();

		this.setTitle("Password Expired");
		this.setIconImage(new ImageIcon("resources/WPISuite-128x128.png")
				.getImage());

		this.setVisible(true);
		this.newPasswordField.setText("");
		this.confirmPasswordField.setText("");
	}

	private void initPosition() {
		// Get the current screen size to center the window
		int sizex = this.getSize().width;
		int sizey = this.getSize().height;
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension scrnsize = toolkit.getScreenSize();
		this.setLocation((scrnsize.width - sizex) / 2,
				(scrnsize.height - sizey) / 2);
	}

	private void initListeners() {
		this.newPasswordField.getDocument().addDocumentListener(fieldListener);

		this.confirmPasswordField.getDocument().addDocumentListener(
				fieldListener);

		this.changePasswordButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				controller.changePassword();
			}
		});
	}

	private DocumentListener fieldListener = new DocumentListener() {
		@Override
		public void changedUpdate(DocumentEvent arg0) {
			validateFields();
		}

		@Override
		public void insertUpdate(DocumentEvent arg0) {
			validateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent arg0) {
			validateFields();
		}
	};

	public void close() {
		this.dispose();
		controller.onClose();
	}

	private void validateFields() {
		this.changePasswordButton.setEnabled(true);
		this.newPasswordErrorLabel.setText("");
		this.confirmPasswordErrorLabel.setText("");

		try {
			controller.validatePassword(new String(newPasswordField
					.getPassword()));
		} catch (Exception e) {
			newPasswordErrorLabel.setText(e.getMessage());
			changePasswordButton.setEnabled(false);
		}

		try {
			controller.validateConfirmation(new String(newPasswordField
					.getPassword()), new String(confirmPasswordField
					.getPassword()));
		} catch (Exception e) {
			confirmPasswordErrorLabel.setText(e.getMessage());
			changePasswordButton.setEnabled(false);
		}
	}

	public String getPassword() {
		return new String(newPasswordField.getPassword());
	}

	// /////////////////////////////////////////////////////
	// //
	// Avoid editing below here - netbeans'd //
	// //
	// /////////////////////////////////////////////////////

	private void initComponents() {

		headerLabel = new javax.swing.JLabel();
		headerSeparator = new javax.swing.JSeparator();
		newPasswordLabel = new javax.swing.JLabel();
		newPasswordField = new javax.swing.JPasswordField();
		confirmPasswordLabel = new javax.swing.JLabel();
		confirmPasswordField = new javax.swing.JPasswordField();
		changePasswordButton = new javax.swing.JButton();
		newPasswordErrorLabel = new javax.swing.JLabel();
		confirmPasswordErrorLabel = new javax.swing.JLabel();

		setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
		setTitle("Password Expired");
		setResizable(false);

		headerLabel.setFont(new java.awt.Font("Tahoma", 0, 16)); // NOI18N
		headerLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		headerLabel.setText("Password Expired");

		newPasswordLabel.setText("New Password:");

		newPasswordField.setText("jPasswordField1");

		confirmPasswordLabel.setText("Confirm Password:");

		confirmPasswordField.setText("jPasswordField2");

		changePasswordButton.setText("Change Password");

		newPasswordErrorLabel.setForeground(new java.awt.Color(255, 0, 0));
		newPasswordErrorLabel.setText("Some error");

		confirmPasswordErrorLabel.setForeground(new java.awt.Color(255, 0, 0));
		confirmPasswordErrorLabel.setText("Another errror!");

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		getContentPane().setLayout(layout);
		layout
				.setHorizontalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addComponent(headerSeparator,
								javax.swing.GroupLayout.DEFAULT_SIZE, 310,
								Short.MAX_VALUE)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												headerLabel,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												290, Short.MAX_VALUE)
										.addContainerGap())
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																changePasswordButton,
																javax.swing.GroupLayout.Alignment.TRAILING)
														.addComponent(
																confirmPasswordErrorLabel,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																290,
																Short.MAX_VALUE))
										.addContainerGap())
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.TRAILING)
														.addComponent(
																newPasswordLabel,
																javax.swing.GroupLayout.Alignment.LEADING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																90,
																Short.MAX_VALUE)
														.addComponent(
																confirmPasswordLabel,
																javax.swing.GroupLayout.Alignment.LEADING))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.TRAILING)
														.addComponent(
																confirmPasswordField,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																196,
																Short.MAX_VALUE)
														.addComponent(
																newPasswordField,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																196,
																Short.MAX_VALUE))
										.addContainerGap())
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												newPasswordErrorLabel,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												290, Short.MAX_VALUE)
										.addContainerGap()));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(headerLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												headerSeparator,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																newPasswordLabel)
														.addComponent(
																newPasswordField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(newPasswordErrorLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																confirmPasswordLabel)
														.addComponent(
																confirmPasswordField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(confirmPasswordErrorLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE).addComponent(
												changePasswordButton).addGap(
												11, 11, 11)));

		pack();
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JButton changePasswordButton;
	private javax.swing.JLabel confirmPasswordErrorLabel;
	private javax.swing.JPasswordField confirmPasswordField;
	private javax.swing.JLabel confirmPasswordLabel;
	private javax.swing.JLabel headerLabel;
	private javax.swing.JSeparator headerSeparator;
	private javax.swing.JLabel newPasswordErrorLabel;
	private javax.swing.JPasswordField newPasswordField;
	private javax.swing.JLabel newPasswordLabel;
	// End of variables declaration

}
