/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ravi Singh - initial design and implementation
 *    Ivo Dobrev - initial design and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import edu.wpi.wpisuite.module.Status;
import edu.wpi.wpisuite.module.StatusListener;
import edu.wpi.wpisuite.module.Status.StatusType;


/**
 * StatusBar for the WPISuite. Contains a statusIndicator icon and a statusMsg
 * label. This status bar is a listener for a StatusState, such as an
 * ActiveState.
 */
@SuppressWarnings("serial")
public class StatusBar extends JToolBar implements StatusListener {
	private JPanel statusPanel;
	private JLabel statusMsg;
	private JLabel statusIndicator;

	private Map<StatusType, ImageIcon> statusIcons = new HashMap<StatusType, ImageIcon>();

	/**
	 * Constructor - sets up default look and positioning
	 */
	private StatusBar() {
		setLayout(new BorderLayout());
		setFloatable(false);
		setPreferredSize(new Dimension(10, 23));
		initIcons();
		initComponents();
	}

	private static StatusBar bar = new StatusBar();

	public static StatusBar getBar() {
		return bar;
	}

	/**
	 * Constructor which also sets a listener on the given state
	 * 
	 * @param state
	 *            The state to set a listener over
	 */
	/*
	 * public StatusBar(StatusState state) { this();
	 * state.registerStatusListener(this); }
	 */

	/**
	 * Initializes the map representing status icons
	 */
	private void initIcons() {
		statusIcons.put(StatusType.NONE, null);
		statusIcons.put(StatusType.ERROR, new ImageIcon(
				"resources/StatusBarIcons/error.png"));
		statusIcons.put(StatusType.SUCCESS, new ImageIcon(
				"resources/StatusBarIcons/checkmark.png"));
		statusIcons.put(StatusType.FAILURE, new ImageIcon(
				"resources/StatusBarIcons/crossmark.png"));
	}

	/**
	 * Initializes components for the status bar
	 */
	private void initComponents() {
		this.statusPanel = new JPanel();
		this.statusPanel.setLayout(new FlowLayout());

		this.statusIndicator = new JLabel();
		statusPanel.add(statusIndicator);

		this.statusMsg = new JLabel();
		statusPanel.add(statusMsg);

		add(statusPanel, BorderLayout.WEST);
	}

	/**
	 * Function to set the status string in the status bar
	 * 
	 * @param msg
	 */
	public void setStatusMessage(String msg) {
		statusMsg.setText(msg);
	}

	/**
	 * Function to set the status indicator in the status bar
	 * 
	 * @param msgType
	 */
	public void setStatusIndicator(StatusType statusType) {
		statusIndicator.setIcon(statusIcons.get(statusType));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.StatusListener#statusChanged(module.Status)
	 */
	@Override
	public void statusChanged(Status newStatus) {
		setStatusIndicator(newStatus.getType());
		setStatusMessage(newStatus.getMessage());
	}
}