/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components;

import javax.swing.ComboBoxModel;
import javax.swing.JComboBox;

import net.java.ao.Entity;

/**
 * A combo box which is specialized for the selection of entities. With the aid
 * of a specialized combo box model, it allows for the persistence of entities
 * that display properly, despite that entities do not have useful toString
 * methods for this purpose. It also allows for type-safety through
 * parameterization.
 * 
 * @param <T>
 *            The type of entity which this combo box is to select from
 */
public class EntityComboBox<T extends Entity> extends JComboBox {
	private static final long serialVersionUID = 5670854738374013753L;

	/**
	 * @param model
	 *            The new combo box model to use as data for this combo box
	 */
	public void setModel(EntityComboBoxModel<T> model) {
		super.setModel(model);
	}

	/**
	 * This function should not be used - only EntityComboBoxModels are
	 * appropriate for use with this class. This function is here purely to stop
	 * the superclass's version from allowing arbitrary model specification - if
	 * it is used, it will IGNORE the given model and instead use a newly
	 * instantiated EntityComboBoxModel of the appropriate type.
	 * 
	 * @deprecated Use {@link #setModel(EntityComboBoxModel)}
	 * @param model
	 *            Ignored
	 */
	@Override
	@Deprecated
	public void setModel(ComboBoxModel model) {
		this.setModel(new EntityComboBoxModel<T>());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComboBox#getModel()
	 */
	@Override
	@SuppressWarnings("unchecked")
	public EntityComboBoxModel<T> getModel() {
		try {
			return (EntityComboBoxModel<T>) super.getModel();
		} catch (Throwable t) {
			return null;
		}
	}

	/**
	 * @return The currently selected, appropriately typed entity
	 */
	public T getSelectedEntity() {
		int index = this.getSelectedIndex();
		return this.getModel().getEntity(index);
	}

	/**
	 * Sets the selection to the given entity, or unselects everything if the
	 * given entity is not in the model.
	 * 
	 * @param ent
	 *            The entity to select
	 */
	public void setSelectedEntity(T ent) {
		int index = this.getModel().getEntityIndex(ent);
		this.setSelectedIndex(index);
	}
}
