/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design, API, and implementation
 *    
 *******************************************************************************/
package edu.wpi.wpisuite.components;

import java.awt.Component;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;
import javax.swing.ListModel;

import net.java.ao.Entity;


import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.module.ActiveState;

/**
 * List cell renderer to set the appropriate icon for any type of entity
 * 
 * @author Lucas Scotta
 * @see http://www.java2s.com/Tutorial/Java/0240__Swing/
 *      CreatingaComplexListCellRendererfonticonandcolor.htm
 * @see http://www.famfamfam.com/lab/icons/silk/ for icons
 */
@SuppressWarnings("serial")
public class EntityListCellRenderer extends DefaultListCellRenderer {
	EntityIconSet entityIcons;

	public EntityListCellRenderer() {
		entityIcons = new EntityIconSet();
	}

	@Override
	@SuppressWarnings("unchecked")
	public Component getListCellRendererComponent(JList list, Object value,
			int index, boolean isSelected, boolean cellHasFocus) {

		super.getListCellRendererComponent(list, value, index, isSelected,
				cellHasFocus);

		ListModel m = list.getModel();

		Entity ent = null;
		if (m instanceof EntityListModel)
			// EntityListModel model = ((EntityList) list).getModel();
			ent = ((EntityListModel) m).getEntity(index);
		else if (m instanceof EntityComboBoxModel)
			ent = ((EntityComboBoxModel) m).getEntity(index);

		if (ent != null)
			if (ent instanceof User) {
				if (((User) ent).isAdmin())
					setIcon(entityIcons.getUserAdminIcon());
				else
					setIcon(entityIcons.getUserIcon(IconType.DEFAULT));
			} else if (ent instanceof Project) {
				if (((Project) ent).equals(ActiveState.getInstance()
						.getProject()))
					setIcon(entityIcons.getProjectActiveIcon());
				else
					setIcon(entityIcons.getProjectIcon(IconType.DEFAULT));
			} else if (ent instanceof Release)
				setIcon(entityIcons.getReleaseIcon(IconType.DEFAULT));
			else if (ent instanceof Iteration)
				setIcon(entityIcons.getIterationIcon(IconType.DEFAULT));
			else if (ent instanceof UserStory)
				setIcon(entityIcons.getUserStoryIcon(IconType.DEFAULT));
			else if (ent instanceof Feature)
				setIcon(entityIcons.getFeatureIcon(IconType.DEFAULT));
			else {
				System.out.println(ent.getClass().getCanonicalName());
				setIcon(entityIcons.getDefaultIcon());
			}

		return this;
	}

}
