/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components;

import edu.wpi.wpisuite.data.Iteration;

/**
 * A specialized EntityComboBoxModel to deal with the fact that in an iteration,
 * null represents the backlog
 */
@SuppressWarnings("serial")
public class IterationComboBoxModel extends EntityComboBoxModel<Iteration> {
	/**
	 * Initializes a new list model based on the given iterations
	 * 
	 * @param iterationList
	 *            The iterations to initialize the model with
	 */
	public IterationComboBoxModel(Iteration[] iterationList) {
		super(iterationList);
	}

	/**
	 * Initializes a new, blank list model
	 */
	public IterationComboBoxModel() {
		super();
	}

	/**
	 * Empties the model, then adds the given list along with a null for the
	 * backlog
	 * 
	 * @param iterationList
	 *            The list of iterations to reinitialize with
	 */
	@Override
	public void reset(Iteration[] iterationList) {
		super.reset(iterationList);
		addNullElement(); // backlog always included
	}

	/**
	 * Adds a null to the underlying models, but names it backlog
	 */
	@Override
	protected void addNullElement() {
		this.ents.add(null);
		this.addStringElement("-Backlog-");
	}
}