/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components;

import javax.swing.ComboBoxModel;
import javax.swing.JComboBox;

/**
 * A combo box which stores its contents and their string representations
 * separately, so that an object's display value doesn't necessarily have to be
 * its toString() value.
 * 
 * @param <T>
 *            The type of object to store in the combo box
 */
@SuppressWarnings("serial")
public class MappedComboBox<T> extends JComboBox {
	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComboBox#getModel()
	 */
	@Override
	@SuppressWarnings("unchecked")
	public MappedComboBoxModel<T> getModel() {
		try {
			return (MappedComboBoxModel<T>) super.getModel();
		} catch (Throwable t) {
			return null;
		}
	}

	/**
	 * Sets a new data model for this combo box
	 * 
	 * @param model
	 *            The new data model to use for this combo box
	 */
	public void setModel(MappedComboBoxModel<T> model) {
		super.setModel(model);
	}

	/**
	 * This function should not be used - only MappedComboBoxModels are
	 * appropriate for use with this class. This function is here purely to stop
	 * the superclass's version from allowing arbitrary model specification - if
	 * it is used, it will IGNORE the given model and instead use a newly
	 * instantiated MappedComboBoxModel of the appropriate type.
	 * 
	 * @deprecated Use {@link #setModel(MappedComboBoxModel)}
	 * @param model
	 *            Ignored
	 */
	@Override
	@Deprecated
	public void setModel(ComboBoxModel model) {
		this.setModel(new MappedComboBoxModel<T>());
	}

	/**
	 * @return The selected item
	 */
	public T getSelected() {
		int index = this.getSelectedIndex();
		return this.getModel().get(index);
	}

	/**
	 * Sets the given item to be selected
	 * 
	 * @param key
	 *            The item to select
	 */
	public void setSelected(T key) {
		int index = this.getModel().getIndex(key);
		this.setSelectedIndex(index);
	}
}
