/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components;

import javax.swing.ComboBoxModel;

/**
 * A MappedComboBox which is allowed to represent any Object
 */
@SuppressWarnings("serial")
public class ObjectComboBox extends MappedComboBox<Object> {
	/**
	 * Sets the data model to the given one
	 * 
	 * @param model
	 *            The data model to set
	 */
	public void setModel(ObjectComboBoxModel model) {
		super.setModel(model);
	}

	/**
	 * This function should not be used - only ObjectComboBoxModels are
	 * appropriate for use with this class. This function is here purely to stop
	 * the superclass's version from allowing arbitrary model specification - if
	 * it is used, it will IGNORE the given model and instead use a newly
	 * instantiated ObjectComboBoxModel of the appropriate type.
	 * 
	 * @deprecated Use {@link #setModel(ObjectComboBoxModel)}
	 * @param model
	 *            Ignored
	 */
	@Override
	@Deprecated
	public void setModel(ComboBoxModel model) {
		this.setModel(new ObjectComboBoxModel());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see components.MappedComboBox#getModel()
	 */
	@Override
	public ObjectComboBoxModel getModel() {
		try {
			return (ObjectComboBoxModel) super.getModel();
		} catch (Throwable t) {
			return null;
		}
	}
}
