/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components;

/**
 * A MappedComboBoxModel that can represent any Object.
 */
@SuppressWarnings("serial")
public class ObjectComboBoxModel extends MappedComboBoxModel<Object> {

	/**
	 * Constructs a new model based on the given group of objects
	 * 
	 * @param objs
	 *            The objects to initialze the model with
	 */
	public ObjectComboBoxModel(Object[] objs) {
		this();
		for (Object obj : objs)
			this.addElement(obj);
	}

	/**
	 * Constructs a new, blank model
	 */
	public ObjectComboBoxModel() {
		super();
	}

	/**
	 * Adds a mapping from the given object to its toString() value
	 */
	@Override
	public void addElement(Object obj) {
		super.addElement(new Mapping<Object>(obj, (obj == null ? null : obj
				.toString())));
	}

}
