/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor;

import java.awt.event.ActionEvent;
import java.io.IOException;

/**
 * A CommandListener is functionally almost equivalent to an ActionListener -
 * the only difference is that CommandListeners may throw IOExceptions back at
 * the listenee and expect to have them reasonably handled.
 * 
 * This will typically be used as an interface from a GUI element to a lower-
 * level handler, ie, as a way for model code to be informed when a user hits a
 * button. The ability for the handler to throw an exception means that the
 * exact mechanics of why the exception was thrown can be hidden from the GUI,
 * instead leaving the GUI merely to display the error described by the handler.
 */
public interface CommandListener {
	/**
	 * Called by the listenee whenever a Command is executed. Typical usage
	 * might involve a GUI element activating a call to this function over any
	 * listeners of that GUI element.
	 * 
	 * @param evt
	 *            The ActionEvent defining this command
	 * @throws IOException
	 *             If anything goes wrong in executing the command
	 */
	public void commandPerformed(ActionEvent evt) throws IOException;
}
