/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *    Ian Williams - Initial design
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor;

import edu.wpi.wpisuite.module.ActiveState;

/**
 * This interface encompasses all the elements which are required of any
 * controller class for any editor panel. This encompasses all methods that the
 * editor will use to inform its container controller of changes, and all
 * methods which the editor requires for the sake of context.
 * 
 * It is suggested though not necessary that modules which wish to contain
 * editors implement this interface directly
 */
public interface EditorContainer {
	/**
	 * @return The current state of the application as a whole
	 */
	public ActiveState getState();

	/**
	 * Informs the container that this editor has completed its function and
	 * should disappear from view, for example if the object it is editing has
	 * been deleted.
	 */
	public void closeEditor();

	/**
	 * Informs the container that the object this editor is editing has been
	 * saved.
	 * 
	 * @deprecated The soon-to-be model wrappers should handle these changes
	 *             directly, instead of passing this around between controllers
	 */
	@Deprecated
	public void onEditorSave();
}
