/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor;

import edu.wpi.wpisuite.components.editor.Editor.Mode;

/**
 * Objects implementing this class are listening on changes to the Mode of a
 * given Editor panel. This interface defines a way for the implementing
 * listener to update itself accordingly whenever the listenee changes its
 * editing mode.
 */
public interface ModeListener {
	/**
	 * Called on changes to the Mode of the listenee. This method will typically
	 * update the actions the user could take on an EditorView, for example by
	 * changing the actions represented by certain buttons or enabling/disabling
	 * components.
	 * 
	 * @param mode
	 *            The new Mode the listenee has entered
	 */
	public void changeMode(Mode mode);
}
