/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor;

/**
 * A PropertyValidator is an object which checks "properties", defined as any
 * mapping from a String propertyName to an Object value, to ensure that they
 * are valid. This check typically occurs whenever the property changes.
 */
public interface PropertyValidator {
	/**
	 * Asserts that the given property is valid in its current state. If it is,
	 * this method does nothing. Otherwise, it throws an InvalidInputException
	 * whose message generally contains more specific details of why the input
	 * is invalid, such that the object displaying this property can properly
	 * display the error message without actually needing to know the specific
	 * error.
	 * 
	 * @param propertyName
	 *            The name of the property to validate
	 * @param newValue
	 *            The current value of the property to validate
	 * @throws InvalidInputException
	 *             If the property is somehow invalid. Will generally contain
	 *             more specific information as to why the property is invalid
	 *             in the message.
	 */
	public void validateProperty(String propertyName, Object newValue)
			throws InvalidInputException;
}
