/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor;

/**
 * Objects implementing this class are listening on changes to the validation
 * status of another object. This interface defines a way for the implementer to
 * update itself accordingly whenever the listenee says that the validation
 * state has changed.
 */
public interface ValidListener {
	/**
	 * The action to perform when the listenee informs this implementer that its
	 * validation state. This could include such things as enabling saving.
	 * 
	 * @param newValid
	 *            Whether or not the listenee currently represents a valid
	 *            object
	 */
	public void changeValid(boolean newValid);
}
