/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor.entity;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.LayoutStyle.ComponentPlacement;


import edu.wpi.wpisuite.components.EntityComboBox;
import edu.wpi.wpisuite.components.EntityIconSet;
import edu.wpi.wpisuite.components.EntityListCellRenderer;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.DefaultEntityEditorPanelView;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.User;

/**
 * A swing panel view for a Feature Editor
 */
@SuppressWarnings("serial")
public class FeatureEditorPanelView extends DefaultEntityEditorPanelView {

	EntityIconSet icons = new EntityIconSet();

	/**
	 * Calls super
	 * 
	 * @param p
	 *            The editor controller for this view
	 */
	public FeatureEditorPanelView(FeatureEditor ed) {
		super(ed);

		this.ownerComboBox.setRenderer(new EntityListCellRenderer());
		this.releaseComboBox.setRenderer(new EntityListCellRenderer());
		this.releaseLabel.setIcon(icons.getReleaseIcon(IconType.DEFAULT));
		this.ownerLabel.setIcon(icons.getUserIcon(IconType.DEFAULT));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see components.editor.DefaultEntityEditorPanelView#registerComponents()
	 */
	@Override
	protected void registerComponents() {
		registerComponent("header", this.featureHeaderLabel);
		registerComponent("title", this.titleLabel, this.titleTextField);
		registerComponent("description", this.descriptionLabel,
				this.descriptionTextArea);
		registerComponent("owner", this.ownerLabel, this.ownerComboBox);
		registerComponent("release", this.releaseLabel, this.releaseComboBox);

		registerComponent("claimOwnership", this.ownerClaimButton);
	}

	// /////////////////////////////////////////////////////////////////////////
	// //
	// Code below this point was created with NetBeans form builder. //
	// Avoid editing manually. If necessary, do so with caution. //
	// //
	// /////////////////////////////////////////////////////////////////////////

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed" desc="Generated Code">
	@Override
	protected void initComponents() {

		this.featureHeaderLabel = new javax.swing.JLabel();
		this.titleLabel = new javax.swing.JLabel();
		this.titleTextField = new javax.swing.JTextField();
		this.descriptionLabel = new javax.swing.JLabel();
		this.titleScrollPane = new javax.swing.JScrollPane();
		this.descriptionTextArea = new javax.swing.JTextArea();
		this.cancelButton = new javax.swing.JButton();
		this.actionButton = new javax.swing.JButton();
		this.headerSeparator = new javax.swing.JSeparator();
		this.ownerComboBox = new EntityComboBox<User>();
		this.ownerClaimButton = new javax.swing.JButton();
		this.ownerLabel = new javax.swing.JLabel();

		this.releaseComboBox = new EntityComboBox<Release>();
		this.releaseLabel = new javax.swing.JLabel();

		this.deleteButton = new javax.swing.JButton();

		this.featureHeaderLabel.setFont(new java.awt.Font("Tahoma", 0, 16));
		this.featureHeaderLabel.setText("Feature");

		this.titleLabel.setText("Title:");

		this.descriptionLabel.setText("Description:");

		this.releaseLabel.setText("Release:");

		this.descriptionTextArea.setColumns(1);
		this.descriptionTextArea.setLineWrap(true);
		this.descriptionTextArea.setRows(1);
		this.titleScrollPane.setViewportView(this.descriptionTextArea);

		this.cancelButton.setText("Cancel");

		this.actionButton.setText("Action");

		// ownerComboBox.setEnabled(false);
		this.ownerComboBox.setMinimumSize(new java.awt.Dimension(100, 20));

		this.ownerClaimButton.setText("Claim");

		this.ownerLabel.setText("Owner:");

		this.deleteButton.setText("Delete");

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout
				.setHorizontalGroup(layout
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								Alignment.TRAILING,
								layout.createSequentialGroup().addContainerGap(
										372, Short.MAX_VALUE).addComponent(
										this.actionButton).addPreferredGap(
										ComponentPlacement.UNRELATED)
										.addComponent(this.deleteButton)
										.addPreferredGap(
												ComponentPlacement.UNRELATED)
										.addComponent(this.cancelButton)
										.addContainerGap())
						.addGroup(
								layout.createSequentialGroup()
										.addContainerGap().addComponent(
												this.featureHeaderLabel,
												GroupLayout.DEFAULT_SIZE, 573,
												Short.MAX_VALUE)
										.addContainerGap())
						.addComponent(this.headerSeparator,
								GroupLayout.DEFAULT_SIZE, 593, Short.MAX_VALUE)
						.addGroup(
								layout.createSequentialGroup()
										.addContainerGap().addComponent(
												this.descriptionLabel)
										.addContainerGap(526, Short.MAX_VALUE))
						.addGroup(
								layout.createSequentialGroup()
										.addContainerGap().addComponent(
												this.titleScrollPane,
												GroupLayout.DEFAULT_SIZE, 573,
												Short.MAX_VALUE)
										.addContainerGap())
						.addGroup(
								layout.createSequentialGroup()
										.addContainerGap().addComponent(
												this.titleLabel)
										.addContainerGap(526, Short.MAX_VALUE))
						.addGroup(
								layout.createSequentialGroup()
										.addContainerGap().addComponent(
												this.titleTextField,
												GroupLayout.DEFAULT_SIZE, 573,
												Short.MAX_VALUE)
										.addContainerGap())
						.addGroup(
								Alignment.TRAILING,
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																layout
																		.createSequentialGroup()
																		.addComponent(
																				this.releaseComboBox,
																				GroupLayout.DEFAULT_SIZE,
																				392,
																				Short.MAX_VALUE)
																		.addPreferredGap(
																				ComponentPlacement.RELATED))
														.addComponent(
																this.releaseLabel,
																GroupLayout.DEFAULT_SIZE,
																24,
																Short.MAX_VALUE))
										.addGroup(
												layout
														.createParallelGroup(
																Alignment.TRAILING)
														.addComponent(
																this.ownerComboBox,
																Alignment.LEADING,
																GroupLayout.PREFERRED_SIZE,
																112,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.ownerLabel,
																Alignment.LEADING))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addComponent(this.ownerClaimButton)
										.addContainerGap()));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								Alignment.TRAILING,
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(this.featureHeaderLabel)
										.addPreferredGap(
												ComponentPlacement.UNRELATED)
										.addComponent(this.headerSeparator,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addComponent(this.titleLabel)
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addComponent(this.titleTextField,
												GroupLayout.PREFERRED_SIZE, 20,
												GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												ComponentPlacement.UNRELATED)
										.addGroup(
												layout
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(
																this.releaseLabel)
														.addComponent(
																this.ownerLabel))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(
																this.ownerClaimButton)
														.addComponent(
																this.ownerComboBox,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.releaseComboBox,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												ComponentPlacement.UNRELATED)
										.addComponent(this.descriptionLabel,
												GroupLayout.PREFERRED_SIZE, 14,
												GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addComponent(this.titleScrollPane,
												GroupLayout.DEFAULT_SIZE, 141,
												Short.MAX_VALUE)
										.addPreferredGap(
												ComponentPlacement.UNRELATED)
										.addGroup(
												layout
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(
																this.cancelButton,
																GroupLayout.PREFERRED_SIZE,
																23,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.deleteButton)
														.addComponent(
																this.actionButton))
										.addContainerGap()));
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JLabel descriptionLabel;
	private javax.swing.JTextArea descriptionTextArea;
	private javax.swing.JLabel featureHeaderLabel;
	private javax.swing.JSeparator headerSeparator;
	private javax.swing.JButton ownerClaimButton;
	private javax.swing.JLabel ownerLabel;
	private EntityComboBox<User> ownerComboBox;
	private javax.swing.JLabel releaseLabel;
	private EntityComboBox<Release> releaseComboBox;
	private javax.swing.JLabel titleLabel;
	private javax.swing.JScrollPane titleScrollPane;
	private javax.swing.JTextField titleTextField;
	// End of variables declaration

}
