/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor.entity;

import com.toedter.calendar.JDateChooser;

import edu.wpi.wpisuite.components.EntityComboBox;
import edu.wpi.wpisuite.components.EntityIconSet;
import edu.wpi.wpisuite.components.EntityListCellRenderer;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.DefaultEntityEditorPanelView;
import edu.wpi.wpisuite.data.Release;

/**
 * A swing panel view for the IterationEditor
 */
@SuppressWarnings("serial")
public class IterationEditorPanelView extends DefaultEntityEditorPanelView {

	EntityIconSet icons = new EntityIconSet();

	/**
	 * Calls super
	 * 
	 * @param p
	 *            The editor controller for this view
	 */
	public IterationEditorPanelView(IterationEditor p) {
		super(p);
		this.releaseComboBox.setRenderer(new EntityListCellRenderer());
		this.releaseLabel.setIcon(icons.getReleaseIcon(IconType.DEFAULT));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see components.editor.DefaultEntityEditorPanelView#registerComponents()
	 */
	@Override
	public void registerComponents() {
		registerComponent("header", this.headerLabel);

		registerComponent("name", this.nameLabel, this.nameTextField);
		registerComponent("startDate", this.startDateLabel,
				this.startDateChooser);
		registerComponent("endDate", this.endDateLabel, this.endDateChooser);
		registerComponent("release", this.releaseLabel, this.releaseComboBox);

		registerComponent("totalUnits", this.totalUnitsLabel,
				this.totalUnitsFieldLabel);
	}

	// /////////////////////////////////////////////////////////////////////////
	// //
	// Code below this point was created with NetBeans form builder. //
	// Avoid editing manually. If necessary, do so with caution. //
	// //
	// /////////////////////////////////////////////////////////////////////////

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed" desc="Generated Code">
	@Override
	protected void initComponents() {

		this.headerLabel = new javax.swing.JLabel();
		this.headerSeparator = new javax.swing.JSeparator();
		this.nameLabel = new javax.swing.JLabel();
		this.nameTextField = new javax.swing.JTextField();
		this.startDateLabel = new javax.swing.JLabel();
		this.endDateLabel = new javax.swing.JLabel();
		this.startDateChooser = new JDateChooser();
		this.endDateChooser = new JDateChooser();
		this.cancelButton = new javax.swing.JButton();
		this.actionButton = new javax.swing.JButton();
		this.deleteButton = new javax.swing.JButton();
		this.totalUnitsLabel = new javax.swing.JLabel();
		this.totalUnitsFieldLabel = new javax.swing.JLabel();

		this.releaseLabel = new javax.swing.JLabel();
		this.releaseComboBox = new EntityComboBox<Release>();

		this.releaseLabel.setText("Release:");

		this.headerLabel.setFont(new java.awt.Font("Tahoma", 0, 16)); // NOI18N
		this.headerLabel.setText("Iteration");

		this.nameLabel.setText("Name:");

		this.startDateLabel.setText("Start Date:");

		this.endDateLabel.setText("End Date:");

		this.totalUnitsFieldLabel.setFont(new java.awt.Font("Tahoma", 0, 16)); // NOI18N
		this.totalUnitsFieldLabel
				.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);

		this.totalUnitsLabel.setText("Units Total");
		// totalUnitsLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT
		// );

		this.cancelButton.setText("Cancel");

		this.actionButton.setText("Action");

		this.deleteButton.setText("Delete");

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout
				.setHorizontalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(this.headerLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(
												this.totalUnitsFieldLabel,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												452, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(this.totalUnitsLabel)
										.addContainerGap())
						.addComponent(this.headerSeparator,
								javax.swing.GroupLayout.DEFAULT_SIZE, 452,
								Short.MAX_VALUE)
						.addGroup(
								layout.createSequentialGroup()
										.addContainerGap().addComponent(
												this.nameLabel)
										.addContainerGap(411, Short.MAX_VALUE))
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												this.nameTextField,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												432, Short.MAX_VALUE)
										.addContainerGap())
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.startDateChooser,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																110,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.startDateLabel))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.endDateLabel)
														.addComponent(
																this.endDateChooser,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																110,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.releaseLabel)
														.addComponent(
																this.releaseComboBox,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																110,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addContainerGap(216, Short.MAX_VALUE))
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								layout
										.createSequentialGroup()
										.addContainerGap(304, Short.MAX_VALUE)
										.addComponent(this.actionButton)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(this.cancelButton)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(this.deleteButton)
										.addContainerGap()));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.headerLabel)
														.addComponent(
																this.totalUnitsFieldLabel)
														.addComponent(
																this.totalUnitsLabel))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.headerSeparator,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(this.nameLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.nameTextField,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.startDateLabel)
														.addComponent(
																this.endDateLabel)
														.addComponent(
																this.releaseLabel))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.startDateChooser)
														.addComponent(
																this.endDateChooser)
														.addComponent(
																this.releaseComboBox))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												64, Short.MAX_VALUE)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.cancelButton)
														.addComponent(
																this.actionButton)
														.addComponent(
																this.deleteButton))
										.addContainerGap()));
	}// </editor-fold>

	// Variables declaration - do not modify
	private JDateChooser endDateChooser;
	private javax.swing.JLabel endDateLabel;
	private javax.swing.JLabel headerLabel;
	private javax.swing.JSeparator headerSeparator;
	private javax.swing.JLabel nameLabel;
	private javax.swing.JTextField nameTextField;
	private JDateChooser startDateChooser;
	private javax.swing.JLabel startDateLabel;
	private javax.swing.JLabel releaseLabel;
	private EntityComboBox<Release> releaseComboBox;
	private javax.swing.JLabel totalUnitsFieldLabel;
	private javax.swing.JLabel totalUnitsLabel;
	// End of variables declaration

}
