/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor.entity;

import java.util.ArrayList;
import java.util.List;

import edu.wpi.wpisuite.components.MappedComboBox;
import edu.wpi.wpisuite.components.MappedComboBoxModel;
import edu.wpi.wpisuite.components.MappedComboBoxModel.Mapping;
import edu.wpi.wpisuite.components.editor.DefaultEntityEditorPanelView;

/**
 * A swing panel view for the UserEditor
 */
@SuppressWarnings("serial")
public class UserEditorPanelView extends DefaultEntityEditorPanelView {

	/**
	 * Calls super
	 * 
	 * @param p
	 *            The editor controller for this view
	 */
	public UserEditorPanelView(UserEditor p) {
		super(p);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see components.editor.DefaultEntityEditorPanelView#registerComponents()
	 */
	@Override
	public void registerComponents() {
		this.setPrivilegesModel();
		registerComponent("username", this.usernameLabel,
				this.usernameTextField);
		registerComponent("firstName", this.firstNameLabel,
				this.firstNameTextField);
		registerComponent("lastName", this.lastNameLabel,
				this.lastNameTextField);
		registerComponent("emailAddress", this.emailLabel,
				this.emailAddressTextField);
		registerComponent("isLocked", this.accountLockedLabel,
				this.accountLockedCheckBox);
		registerComponent("isAdmin", this.privilegesLabel,
				this.privilegesComboBox);
	}

	/**
	 * Sets up the privileges combo box with the appropriate options
	 */
	private void setPrivilegesModel() {
		List<Mapping<Boolean>> mappings = new ArrayList<Mapping<Boolean>>();
		mappings.add(new Mapping<Boolean>(false, "Default User"));
		mappings.add(new Mapping<Boolean>(true, "Administrator"));
		privilegesComboBox.setModel(new MappedComboBoxModel<Boolean>(mappings));
	}

	// /////////////////////////////////////////////////////////////////////////
	// //
	// Code below this point was created with NetBeans form builder. //
	// Avoid editing manually. If necessary, do so with caution. //
	// //
	// /////////////////////////////////////////////////////////////////////////

	@Override
	protected void initComponents() {

		infoPanel = new javax.swing.JPanel();
		usernameLabel = new javax.swing.JLabel();
		usernameTextField = new javax.swing.JFormattedTextField();
		emailLabel = new javax.swing.JLabel();
		firstNameTextField = new javax.swing.JFormattedTextField();
		lastNameLabel = new javax.swing.JLabel();
		firstNameLabel = new javax.swing.JLabel();
		privilegesLabel = new javax.swing.JLabel();
		privilegesComboBox = new MappedComboBox<Boolean>();
		emailAddressTextField = new javax.swing.JFormattedTextField();
		lastNameTextField = new javax.swing.JFormattedTextField();
		actionButton = new javax.swing.JButton();
		cancelButton = new javax.swing.JButton();
		deleteButton = new javax.swing.JButton();
		accountLockedCheckBox = new javax.swing.JCheckBox();
		accountLockedLabel = new javax.swing.JLabel();

		infoPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("User"));

		usernameLabel.setText("Username");

		emailLabel.setText("First Name");

		lastNameLabel.setText("Last Name");

		firstNameLabel.setText("E-Mail Address");

		privilegesLabel.setText("Account Privileges");

		actionButton.setText("Apply");

		cancelButton.setText("Cancel");

		deleteButton.setText("Delete");

		accountLockedLabel.setText("Account Locked");

		javax.swing.GroupLayout infoPanelLayout = new javax.swing.GroupLayout(
				infoPanel);
		infoPanel.setLayout(infoPanelLayout);
		infoPanelLayout
				.setHorizontalGroup(infoPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								infoPanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												infoPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																infoPanelLayout
																		.createSequentialGroup()
																		.addGroup(
																				infoPanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING,
																								false)
																						.addComponent(
																								usernameLabel)
																						.addComponent(
																								emailLabel)
																						.addComponent(
																								usernameTextField,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								213,
																								Short.MAX_VALUE)
																						.addComponent(
																								firstNameTextField))
																		.addGap(
																				18,
																				18,
																				18)
																		.addGroup(
																				infoPanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addGroup(
																								infoPanelLayout
																										.createSequentialGroup()
																										.addGroup(
																												infoPanelLayout
																														.createParallelGroup(
																																javax.swing.GroupLayout.Alignment.LEADING)
																														.addComponent(
																																lastNameLabel)
																														.addComponent(
																																lastNameTextField,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																225,
																																Short.MAX_VALUE)
																														.addComponent(
																																emailAddressTextField,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																225,
																																Short.MAX_VALUE))
																										.addPreferredGap(
																												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																												18,
																												javax.swing.GroupLayout.PREFERRED_SIZE)
																										.addGroup(
																												infoPanelLayout
																														.createParallelGroup(
																																javax.swing.GroupLayout.Alignment.TRAILING)
																														.addGroup(
																																infoPanelLayout
																																		.createParallelGroup(
																																				javax.swing.GroupLayout.Alignment.LEADING)
																																		.addComponent(
																																				privilegesLabel)
																																		.addComponent(
																																				privilegesComboBox,
																																				javax.swing.GroupLayout.PREFERRED_SIZE,
																																				javax.swing.GroupLayout.DEFAULT_SIZE,
																																				javax.swing.GroupLayout.PREFERRED_SIZE))
																														.addGroup(
																																infoPanelLayout
																																		.createSequentialGroup()
																																		.addComponent(
																																				accountLockedLabel)
																																		.addPreferredGap(
																																				javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
																																		.addComponent(
																																				accountLockedCheckBox))))
																						.addComponent(
																								firstNameLabel)))
														.addGroup(
																javax.swing.GroupLayout.Alignment.TRAILING,
																infoPanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				actionButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				deleteButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				cancelButton)))
										.addContainerGap()));
		infoPanelLayout
				.setVerticalGroup(infoPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								infoPanelLayout
										.createSequentialGroup()
										.addGroup(
												infoPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.TRAILING)
														.addGroup(
																infoPanelLayout
																		.createSequentialGroup()
																		.addGroup(
																				infoPanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.BASELINE)
																						.addComponent(
																								usernameLabel)
																						.addComponent(
																								firstNameLabel))
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				infoPanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.BASELINE)
																						.addComponent(
																								usernameTextField,
																								javax.swing.GroupLayout.PREFERRED_SIZE,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								javax.swing.GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								emailAddressTextField,
																								javax.swing.GroupLayout.PREFERRED_SIZE,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								javax.swing.GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								accountLockedLabel)))
														.addComponent(
																accountLockedCheckBox))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												infoPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																infoPanelLayout
																		.createSequentialGroup()
																		.addGroup(
																				infoPanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.BASELINE)
																						.addComponent(
																								emailLabel)
																						.addComponent(
																								lastNameLabel)
																						.addComponent(
																								privilegesLabel))
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				infoPanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.BASELINE)
																						.addComponent(
																								firstNameTextField,
																								javax.swing.GroupLayout.PREFERRED_SIZE,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								javax.swing.GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								lastNameTextField,
																								javax.swing.GroupLayout.PREFERRED_SIZE,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								javax.swing.GroupLayout.PREFERRED_SIZE)))
														.addGroup(
																infoPanelLayout
																		.createSequentialGroup()
																		.addGap(
																				20,
																				20,
																				20)
																		.addComponent(
																				privilegesComboBox,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				javax.swing.GroupLayout.PREFERRED_SIZE)))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.addGroup(
												infoPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																deleteButton)
														.addComponent(
																cancelButton)
														.addComponent(
																actionButton))
										.addContainerGap()));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				infoPanel, javax.swing.GroupLayout.DEFAULT_SIZE,
				javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				infoPanel, javax.swing.GroupLayout.DEFAULT_SIZE,
				javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE));
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JCheckBox accountLockedCheckBox;
	private javax.swing.JLabel accountLockedLabel;
	private javax.swing.JFormattedTextField emailAddressTextField;
	private javax.swing.JLabel emailLabel;
	private javax.swing.JLabel firstNameLabel;
	private javax.swing.JFormattedTextField firstNameTextField;
	private javax.swing.JPanel infoPanel;
	private javax.swing.JLabel lastNameLabel;
	private javax.swing.JFormattedTextField lastNameTextField;
	private MappedComboBox<Boolean> privilegesComboBox;
	private javax.swing.JLabel privilegesLabel;
	private javax.swing.JLabel usernameLabel;
	private javax.swing.JFormattedTextField usernameTextField;
	// End of variables declaration

}
