/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor.entity;


import edu.wpi.wpisuite.components.EntityComboBox;
import edu.wpi.wpisuite.components.EntityIconSet;
import edu.wpi.wpisuite.components.EntityListCellRenderer;
import edu.wpi.wpisuite.components.IterationComboBoxModel;
import edu.wpi.wpisuite.components.ObjectComboBox;
import edu.wpi.wpisuite.components.ObjectComboBoxModel;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.DefaultEntityEditorPanelView;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;

/**
 * A swing panel view for the UserStoryEditor
 */
@SuppressWarnings("serial")
public class UserStoryEditorPanelView extends DefaultEntityEditorPanelView {

	EntityIconSet icons = new EntityIconSet();

	/**
	 * Calls super and sets up some minimal component properties
	 * 
	 * @param p
	 *            The editor controller for this view
	 */
	public UserStoryEditorPanelView(UserStoryEditor p) {
		super(p);
		descriptionTextArea.setLineWrap(true);
		descriptionTextArea.setWrapStyleWord(true);
		ownerComboBox.setRenderer(new EntityListCellRenderer());
		iterationComboBox.setRenderer(new EntityListCellRenderer());
		this.ownerLabel.setIcon(icons.getUserIcon(IconType.DEFAULT));
		this.iterationLabel.setIcon(icons.getIterationIcon(IconType.DEFAULT));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see components.editor.DefaultEntityEditorPanelView#registerComponents()
	 */
	@Override
	public void registerComponents() {
		registerComponent("header", this.userStoryHeaderLabel);
		registerComponent("title", this.titleLabel, this.titleTextField);
		registerComponent("description", this.descriptionLabel,
				this.descriptionTextArea);
		registerComponent("status", this.statusLabel, this.statusComboBox);
		registerComponent("actualUnits", this.actualUnitsLabel,
				this.actualUnitsTextField);
		registerComponent("estimatedUnits", this.estimatedUnitsLabel,
				this.estimatedUnitsTextField);
		registerComponent("owner", this.ownerLabel, this.ownerComboBox);
		registerComponent("iteration", this.iterationLabel,
				this.iterationComboBox);
		registerComponent("claimOwnership", this.ownerClaimButton);
	}

	// /////////////////////////////////////////////////////////////////////////
	// //
	// Code below this point was created with NetBeans form builder. //
	// Avoid editing manually. If necessary, do so with caution. //
	// //
	// /////////////////////////////////////////////////////////////////////////

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@Override
	protected void initComponents() {

		this.userStoryHeaderLabel = new javax.swing.JLabel();
		this.headerSeparator = new javax.swing.JSeparator();
		this.titleLabel = new javax.swing.JLabel();
		this.titleTextField = new javax.swing.JTextField();
		this.statusLabel = new javax.swing.JLabel();
		this.statusComboBox = new ObjectComboBox();
		this.cancelButton = new javax.swing.JButton();
		this.actionButton = new javax.swing.JButton();
		this.descriptionLabel = new javax.swing.JLabel();
		this.descriptionScrollPane = new javax.swing.JScrollPane();
		this.descriptionTextArea = new javax.swing.JTextArea();
		this.actualUnitsTextField = new javax.swing.JTextField();
		this.iterationComboBox = new EntityComboBox<Iteration>();
		this.estimatedUnitsTextField = new javax.swing.JTextField();
		this.iterationLabel = new javax.swing.JLabel();
		this.ownerLabel = new javax.swing.JLabel();
		this.estimatedUnitsLabel = new javax.swing.JLabel();
		this.actualUnitsLabel = new javax.swing.JLabel();
		this.ownerComboBox = new EntityComboBox<User>();
		this.ownerClaimButton = new javax.swing.JButton();
		this.deleteButton = new javax.swing.JButton();

		this.userStoryHeaderLabel.setFont(new java.awt.Font("Tahoma", 0, 16));
		this.userStoryHeaderLabel.setText("User Story");

		this.titleLabel.setText("Title:");

		this.statusLabel.setText("Status:");
		this.statusComboBox.setModel(new ObjectComboBoxModel(UserStory.Status
				.values()));

		this.cancelButton.setText("Cancel");

		this.actionButton.setText("Action");

		this.descriptionLabel.setText("Description:");

		this.descriptionTextArea.setColumns(20);
		this.descriptionTextArea.setRows(2);
		this.descriptionScrollPane.setViewportView(this.descriptionTextArea);

		this.iterationLabel.setText("Iteration:");
		this.iterationComboBox.setModel(new IterationComboBoxModel());

		this.ownerLabel.setText("Owner:");

		this.estimatedUnitsLabel.setText("Est. Units:");

		this.actualUnitsLabel.setText("Actual Units:");

		// ownerComboBox.setEnabled(false);
		this.ownerComboBox.setMinimumSize(new java.awt.Dimension(100, 20));

		this.ownerClaimButton.setText("Claim");

		this.deleteButton.setText("Delete");

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout
				.setHorizontalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												this.userStoryHeaderLabel,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												668, Short.MAX_VALUE)
										.addContainerGap())
						.addComponent(this.headerSeparator,
								javax.swing.GroupLayout.DEFAULT_SIZE, 688,
								Short.MAX_VALUE)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								layout
										.createSequentialGroup()
										.addContainerGap(467, Short.MAX_VALUE)
										.addComponent(this.actionButton)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(this.deleteButton)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(this.cancelButton)
										.addContainerGap())
						.addGroup(
								layout.createSequentialGroup()
										.addContainerGap().addComponent(
												this.descriptionLabel)
										.addContainerGap(621, Short.MAX_VALUE))
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												this.descriptionScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												668, Short.MAX_VALUE)
										.addContainerGap())
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																layout
																		.createSequentialGroup()
																		.addGroup(
																				layout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addGroup(
																								javax.swing.GroupLayout.Alignment.TRAILING,
																								layout
																										.createSequentialGroup()
																										.addGroup(
																												layout
																														.createParallelGroup(
																																javax.swing.GroupLayout.Alignment.LEADING)
																														.addComponent(
																																this.statusComboBox,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																363,
																																Short.MAX_VALUE)
																														.addComponent(
																																this.statusLabel))
																										.addPreferredGap(
																												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																										.addGroup(
																												layout
																														.createParallelGroup(
																																javax.swing.GroupLayout.Alignment.LEADING)
																														.addComponent(
																																this.estimatedUnitsTextField,
																																javax.swing.GroupLayout.PREFERRED_SIZE,
																																60,
																																javax.swing.GroupLayout.PREFERRED_SIZE)
																														.addComponent(
																																this.estimatedUnitsLabel))
																										.addPreferredGap(
																												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																										.addGroup(
																												layout
																														.createParallelGroup(
																																javax.swing.GroupLayout.Alignment.LEADING)
																														.addComponent(
																																this.actualUnitsLabel)
																														.addComponent(
																																this.actualUnitsTextField,
																																javax.swing.GroupLayout.PREFERRED_SIZE,
																																61,
																																javax.swing.GroupLayout.PREFERRED_SIZE)))
																						.addComponent(
																								this.titleLabel))
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				layout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addComponent(
																								this.ownerLabel)
																						.addGroup(
																								javax.swing.GroupLayout.Alignment.TRAILING,
																								layout
																										.createSequentialGroup()
																										.addComponent(
																												this.ownerComboBox,
																												javax.swing.GroupLayout.DEFAULT_SIZE,
																												103,
																												Short.MAX_VALUE)
																										.addPreferredGap(
																												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																										.addComponent(
																												this.ownerClaimButton)))
																		.addContainerGap())
														.addGroup(
																layout
																		.createSequentialGroup()
																		.addComponent(
																				this.titleTextField,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				532,
																				Short.MAX_VALUE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				layout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING,
																								false)
																						.addGroup(
																								layout
																										.createSequentialGroup()
																										.addComponent(
																												this.iterationLabel)
																										.addPreferredGap(
																												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																												javax.swing.GroupLayout.DEFAULT_SIZE,
																												Short.MAX_VALUE))
																						.addGroup(
																								javax.swing.GroupLayout.Alignment.TRAILING,
																								layout
																										.createSequentialGroup()
																										.addComponent(
																												this.iterationComboBox,
																												javax.swing.GroupLayout.PREFERRED_SIZE,
																												130,
																												javax.swing.GroupLayout.PREFERRED_SIZE)
																										.addPreferredGap(
																												javax.swing.LayoutStyle.ComponentPlacement.RELATED)))
																		.addGap(
																				10,
																				10,
																				10)))));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(this.userStoryHeaderLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.headerSeparator,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.iterationLabel)
														.addComponent(
																this.titleLabel))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.titleTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.iterationComboBox,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.estimatedUnitsLabel)
														.addComponent(
																this.actualUnitsLabel)
														.addComponent(
																this.statusLabel)
														.addComponent(
																this.ownerLabel))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.statusComboBox,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.actualUnitsTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.estimatedUnitsTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.ownerComboBox,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.ownerClaimButton))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(this.descriptionLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.descriptionScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												108, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.cancelButton)
														.addComponent(
																this.deleteButton)
														.addComponent(
																this.actionButton))
										.addContainerGap()));
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JLabel actualUnitsLabel;
	private javax.swing.JTextField actualUnitsTextField;
	private javax.swing.JLabel descriptionLabel;
	private javax.swing.JScrollPane descriptionScrollPane;
	private javax.swing.JTextArea descriptionTextArea;
	private javax.swing.JLabel estimatedUnitsLabel;
	private javax.swing.JTextField estimatedUnitsTextField;
	private javax.swing.JSeparator headerSeparator;
	private EntityComboBox<Iteration> iterationComboBox;
	private javax.swing.JLabel iterationLabel;
	private javax.swing.JButton ownerClaimButton;
	private javax.swing.JLabel ownerLabel;
	private EntityComboBox<User> ownerComboBox;
	private javax.swing.JLabel statusLabel;
	private ObjectComboBox statusComboBox;
	private javax.swing.JLabel titleLabel;
	private javax.swing.JTextField titleTextField;
	private javax.swing.JLabel userStoryHeaderLabel;
	// End of variables declaration

}
