/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.transfer;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.List;

import net.java.ao.Entity;
import edu.wpi.wpisuite.data.funcs.Artifact;

/**
 * A transferable class that holds a list of objects
 * 
 * @see http://www.java2s.com/Tutorial/Java/0240__Swing/
 *      TransferbothTextandColorbetweenJTextFieldandJTextArea.htm
 * 
 */
public class DataTransferable<T> implements Transferable {
	private List<T> data;

	private DataFlavor[] flavors;

	String dataMimeType;

	public DataTransferable(String dataMimeType) {
		this.dataMimeType = dataMimeType;
	}

	/**
	 * 
	 * @param data
	 *            A list of objects to transfer
	 * @param dataClass
	 *            The class of the data objects
	 */
	public DataTransferable(List<T> data, Class<T> dataClass) {
		this.dataMimeType = DataFlavor.javaJVMLocalObjectMimeType + ";class="
				+ dataClass.getName();

		// String dataMimeType = DataFlavor.javaJVMLocalObjectMimeType
		// + ";class=" +
		// ReflectingGenerics.getTypeArguments(DataTransferable.class,
		// getClass()).get(0).getName();

		if (data.get(0) instanceof String)
			System.out.println("String!!");

		DataFlavor dataFlavor = null;
		try {
			dataFlavor = new DataFlavor(dataMimeType);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		this.flavors = new DataFlavor[2];
		this.flavors[0] = DataFlavor.stringFlavor;
		this.flavors[1] = dataFlavor;
		this.data = data;
	}

	public DataFlavor[] getTransferDataFlavors() {
		return this.flavors.clone();
	}

	public boolean isDataFlavorSupported(DataFlavor flavor) {
		for (int i = 0; i < this.flavors.length; i++)
			if (flavor.equals(this.flavors[i]))
				return true;
		return false;
	}

	/**
	 * @return The data for the requested DataFlavor, if it is supported
	 */
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (flavor.equals(this.flavors[0])) {

			String asString = "";
			for (int i = 0; i < data.size(); i++) {
				T item = data.get(i);

				if (item instanceof Entity)
					asString += new Artifact((Entity) item).getSimpleName();
				else
					asString += item.toString();

				if (i < data.size() - 1)
					asString += "\n";
			}
			return asString;

		} else if (flavor.equals(this.flavors[1]))
			return this.data;
		else
			throw new UnsupportedFlavorException(flavor);
	}
}
