/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design, API, and implementation
 *    Dan Bjorge - Cleanup, parameterization
 *******************************************************************************/

package edu.wpi.wpisuite.components.transfer;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.TransferHandler;

import edu.wpi.wpisuite.components.EntityList;

/**
 * 
 * @see http://www.java2s.com/Tutorial/Java/0240__Swing/
 *      TransferbothTextandColorbetweenJTextFieldandJTextArea.htm
 * 
 */
@SuppressWarnings("serial")
public class EntityListTransferHandler<T> extends TransferHandler {

	Class<T> dataClass;
	String dataMimeType;
	public DataFlavor supportedDataFlavor;

	public EntityListTransferHandler(Class<T> dataClass) {
		this.dataClass = dataClass;
		this.dataMimeType = DataFlavor.javaJVMLocalObjectMimeType + ";class="
				+ dataClass.getName();
		try {
			this.supportedDataFlavor = new DataFlavor(dataMimeType);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	@Override
	public int getSourceActions(JComponent c) {
		return TransferHandler.COPY_OR_MOVE;
	}

	@SuppressWarnings("unchecked")
	@Override
	/*
	 * * Note: the DefaultListModel .get(int index) should be overridden to
	 * return the object at the specified row By default, it will simply return
	 * a string value
	 */
	protected Transferable createTransferable(JComponent component) {
		List<T> selectedObjects = null;

		if (component instanceof EntityList)
			selectedObjects = ((EntityList) component).getSelectedEntities();
		else if (component instanceof JList) {
			int[] selectedIndices = ((JList) component).getSelectedIndices();
			DefaultListModel listModel = (DefaultListModel) ((JList) component)
					.getModel();
			selectedObjects = new ArrayList<T>();
			for (int i = 0; i < selectedIndices.length; i++) {
				Object obj = listModel.get(selectedIndices[i]);
				if (dataClass.isInstance(obj))
					selectedObjects.add((T) obj);
				else
					throw new IllegalArgumentException(
							"The list an EntityListTransferHandler is applied to must provide a means of getting an Entity, not a String");
			}
		} else
			throw new IllegalArgumentException(
					"EntityListTransferHandlers cannot be applied to non-lists");

		Transferable transferable = new DataTransferable<T>(selectedObjects,
				dataClass);
		return transferable;
	}

	@Override
	public boolean canImport(TransferHandler.TransferSupport support) {

		// TODO This makes it so we cannot paste in user stories this way
		if (!support.isDataFlavorSupported(supportedDataFlavor)
				|| !support.isDrop())
			return false;

		return true;
	}

	@SuppressWarnings("unchecked")
	@Override
	public boolean importData(TransferHandler.TransferSupport support) {
		// TODO: use generic type -> class -> name
		// String dataMimeType = DataFlavor.javaJVMLocalObjectMimeType
		// + ";class=" +
		// ReflectingGenerics.getTypeArguments(ListTransferHandler.class,
		// getClass()).get(0).getName();

		if (!canImport(support))
			return false;

		List<T> data = null;
		try {
			data = (List<T>) support.getTransferable().getTransferData(
					supportedDataFlavor);

			postImport(support, data);

		} catch (Exception e) {
			e.printStackTrace();
		}
		return true;
	}

	public void postImport(TransferHandler.TransferSupport support, List<T> data) {
		JList listComponent = (JList) support.getComponent();
		DefaultListModel listModel = (DefaultListModel) listComponent
				.getModel();
		for (T item : data)
			listModel.addElement(item);
	}
}
