/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.transfer;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.TransferHandler;

import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.module.rm.UserStoryTableModel;

// TODO: Finish generalizing this to any table model, right now it depends on UserStoryTableModel
// Create a RowTableModel class that will work with this.
/**
 * 
 * @see http://www.java2s.com/Tutorial/Java/0240__Swing/
 *      TransferbothTextandColorbetweenJTextFieldandJTextArea.htm
 */
@SuppressWarnings("serial")
public class UserStoryTableTransferHandler<T> extends TransferHandler {

	Class<T> dataClass;
	String dataMimeType;
	public DataFlavor supportedDataFlavor;

	public UserStoryTableTransferHandler(Class<T> dataClass) {
		this.dataClass = dataClass;
		this.dataMimeType = DataFlavor.javaJVMLocalObjectMimeType + ";class="
				+ dataClass.getName();
		try {
			this.supportedDataFlavor = new DataFlavor(dataMimeType);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	@Override
	public int getSourceActions(JComponent c) {
		return TransferHandler.COPY_OR_MOVE;
	}

	@SuppressWarnings("unchecked")
	@Override
	protected Transferable createTransferable(JComponent component) {
		JTable table = (JTable) component;
		int[] selectedRows = table.getSelectedRows();
		for (int i = 0; i < selectedRows.length; i++)
			selectedRows[i] = table.convertRowIndexToModel(selectedRows[i]);

		// TODO: Generalize to DefaultTableModel
		UserStoryTableModel tableModel = (UserStoryTableModel) table.getModel();
		List<T> selectedObjects = new ArrayList<T>();

		for (int i = 0; i < selectedRows.length; i++)
			selectedObjects.add((T) tableModel.getUserStoryAt(selectedRows[i]));

		DataTransferable<T> transferable = new DataTransferable<T>(
				selectedObjects, dataClass);
		return transferable;
	}

	@Override
	public boolean canImport(TransferHandler.TransferSupport support) {

		for (DataFlavor f : support.getDataFlavors())
			if (f.equals(supportedDataFlavor))
				return true;

		return false;
	}

	@SuppressWarnings("unchecked")
	@Override
	public boolean importData(TransferSupport support) {
		// TODO: use generic type -> class -> name
		// String userStoryMimeType = DataFlavor.javaJVMLocalObjectMimeType
		// + ";class=data.UserStory";// + UserStory.class.getName();

		if (!canImport(support))
			return false;

		System.out.println("data mime type:" + dataMimeType);

		List<T> data = null;
		try {
			data = (List<T>) support.getTransferable().getTransferData(
					supportedDataFlavor);
			for (T item : data)
				// TODO: Delete
				System.out.println("Incoming item: " + item.toString());
			postImport((JComponent) support.getComponent(), data);

		} catch (Exception e) {
			e.printStackTrace();
		}
		return true;
	}

	public void postImport(JComponent component, List<T> data) {
		JTable tableComponent = (JTable) component;
		UserStoryTableModel tableModel = (UserStoryTableModel) tableComponent
				.getModel();

		for (T item : data)
			tableModel.addUserStory((UserStory) item);
	}
}
