/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *    Sabrina Varanelli - Initial design
 *    Ivo Dobrev - Additional design and implementation
 *    Ravi Singh - Additional design and implementation
 *    Lucas Scotta - Additional design and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data;

import net.java.ao.Entity;
import net.java.ao.ManyToMany;
import net.java.ao.OneToMany;

/**
 * A Project is the overarching container in WPISuite. It contains all of the
 * information pertaining to a single development project, most notably
 * including references to its members and its requirements.
 */
public interface Project extends Entity {

	public String getName();

	public void setName(String name);

	public String getDescription();

	public void setDescription(String description);

	@ManyToMany(ProjectMember.class)
	public User[] getMembers();

	@OneToMany
	public Iteration[] getIterations();

	@OneToMany
	public Release[] getReleases();

	// **This is what I WANT, but I don't think there's a pleasant way to do it.
	// @OneToMany
	// public Requirement[] getRequirements();
	// **So instead, these...
	@OneToMany
	public Feature[] getFeatures();

	@OneToMany
	public UserStory[] getUserStories();

	@OneToMany
	public TraceabilityMatrix[] getTraceabilityMatrices();
}