/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data;

import net.java.ao.Entity;

/**
 * Defines a single criterion helping to define the set of requirements that
 * make up an axis of a traceability matrix. The typeData field contains an
 * array of ints which will, in most cases, represent the id numbers of a group
 * of objects (often entities or enums) - the specific interpretation is
 * dependent on the CriterionType.
 */
public interface TraceabilityMatrixCriterion extends Entity {
	public enum CriterionType {
		REQUIREMENT("is the requirement..."), ITERATION(
				"belongs to iteration..."), RELEASE("belongs to release..."), STATUS(
				"has status..."), AND("satisfies all of...", true), OR(
				"satisfies at least one of...", true);

		private String str;
		private boolean isTree;

		private CriterionType(String str) {
			this(str, false);
		}

		private CriterionType(String str, boolean isTree) {
			this.str = str;
			this.isTree = isTree;
		}

		@Override
		public String toString() {
			return str;
		}

		public boolean isTree() {
			return isTree;
		}
	}

	public CriterionType getType();

	public void setType(CriterionType type);

	public int[] getTypeData();

	public void setTypeData(int[] data);
}
