/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Daniel Bjorge - Initial design, API, and implementation
 *    Lucas Scotta - Initial design
 *    Han Li - Initial design
 *    Sabrina Varanelli - Initial design
 *******************************************************************************/

package edu.wpi.wpisuite.data;

import net.java.ao.Entity;
import net.java.ao.ManyToMany;
import net.java.ao.schema.Default;
import net.java.ao.schema.Unique;

/**
 * Defines a user of WPI Suite. Includes personal information and account-
 * related information such as administrator status, account locking, password
 * expiration. Also contains references to the projects a user is a member of.
 */
public interface User extends Entity {
	@Unique
	public String getUsername();

	public void setUsername(String username);

	public String getFirstName();

	public void setFirstName(String firstName);

	public String getLastName();

	public void setLastName(String lastName);

	public String getEmailAddress();

	public void setEmailAddress(String emailAddress);

	@Default("0")
	public boolean isAdmin();

	public void setAdmin(boolean isAdmin);

	public String getPasswordHash();

	public void setPasswordHash(String passwordHash);

	@ManyToMany(ProjectMember.class)
	public Project[] getProjects();

	public Project getLastOpenProject();

	public void setLastOpenProject(Project lastOpenedProject);

	@Default("1")
	public boolean isPasswordExpired();

	public void setPasswordExpired(boolean passwordExpired);

	@Default("0")
	public boolean isLocked();

	public void setLocked(boolean locked);
}