/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *    Sabrina Varanelli - Initial design
 *    Lucas Scotta - Initial design, additional implementation
 *    Ravi Singh - Additional design and implementation
 *    Ivo Dobrev  - Additional design and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data;

import net.java.ao.schema.Default;

/**
 * A type of requirement which represents a single, discrete, unambiguous,
 * testable task. A development team should be able to easily assign user
 * stories time estimates. User stories are typically associated with an
 * iteration and may be assigned an owner.
 */
public interface UserStory extends Requirement {
	public Integer getEstimatedUnits();

	public void setEstimatedUnits(Integer estimatedUnits);

	public Integer getActualUnits();

	public void setActualUnits(Integer actualUnits);

	public enum Status {
		OPEN("Open"), SELECTED("Selected"), IN_PROGRESS("In Progress"), PENDING(
				"Pending"), COMPLETED("Completed");

		private Status(String name) {
			this.name = name;
		}

		private final String name;

		@Override
		public String toString() {
			return name;
		}
	}

	@Default("0")
	// OPEN
	public Status getStatus();

	public void setStatus(Status status);

	public User getOwner();

	public void setOwner(User owner);

	public Project getProject();

	public void setProject(Project project);

	public User getAuthor();

	public void setAuthor(User author);

	public Iteration getIteration();

	public void setIteration(Iteration iteration);
}