/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data.errors;

import edu.wpi.wpisuite.components.editor.InvalidInputException;

/**
 * Represents an input failure specific to a User Story.
 */
@SuppressWarnings("serial")
public class InvalidUserStoryInputException extends InvalidInputException {
	private UserStoryErrorType errorType;

	public InvalidUserStoryInputException(UserStoryErrorType type) {
		super("");
		errorType = type;
	}

	public InvalidUserStoryInputException(UserStoryErrorType type,
			String extraMessage) {
		super(extraMessage);
		errorType = type;
	}

	public InvalidUserStoryInputException(String extraMessage) {
		super(extraMessage);
	}

	/**
	 * Returns the message associated with the errorType. Same as
	 * getErrorType().getMessage();
	 * 
	 * @return The message associated with this errorType, or null if there is
	 *         no errorType;
	 */
	@Override
	public String getLocalizedMessage() {
		if (errorType != null)
			return errorType.getMessage();
		else
			return null;
	}

	/**
	 * @return Both error Returns the error message associated with the error
	 *         type of this exception, combined with the custom error message.
	 */
	@Override
	public String getMessage() {
		String message = new String();
		if (errorType != null)
			message = getLocalizedMessage();
		if (super.getMessage() != null)
			message = message + super.getMessage();
		return message;
	}

	/**
	 * Gets the UserStoryErrorType that caused this exception
	 * 
	 * @return the error type that caused the exception
	 */
	public UserStoryErrorType getErrorType() {
		return errorType;
	}
}