/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data.errors;

/**
 * Enumerates all the possible ways in which iteration management could fail
 */
public enum IterationErrorType {
	DATABASE_ITERATION_CREATION(
			"A database error occured while creating this iteration."), DATABASE_ITERATION_DELETION(
			"A database error occured while attempting to delete this iteration."), DATABASE_USERSTORY_DELETION(
			"A database error occured while attempting to delete a user story in this iteration."), NO_PROJECT(
			"An iteration must be in a project."), NULL(
			"The given iteration to update is null."), NAME_NULL(
			"Name cannot be null."), NAME_EMPTY("Name cannot be empty"), NAME_ALL_SPACES(
			"Name cannot only contain spaces."), NAME_BEGIN_WITH_SPACE(
			"Name cannot begin with a space."), NAME_NOT_UNIQUE(
			"Another iteration with the same name already exists in this project."), NAME_TOO_LONG(
			"Name must be less than 255 characters"), DATE_INVALID(
			"Start date must be before end date."), DATE_OVERLAP(
			"This iteration overlaps with another iteration in this project."), RELEASE_INVALID(
			"Release is not a release of the same project as this iteration."), RELEASE_BEFORE_ITERATION_START(
			"Release ends before the iteration begins."), RELEASE_BEFORE_ITERATION_END(
			"Release ends before the iteration ends.");

	private final String message;

	private IterationErrorType(String message) {
		this.message = message;
	}

	public String getMessage() {
		return message;
	}
}
