/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data.errors;

/**
 * Enumerates all the possible ways in which User Story management could fail
 */
public enum UserStoryErrorType {
	DATABASE_USERSTORY_CREATION(
			"A database error occured while creating this user story."), DATABASE_USERSTORY_DELETION(
			"A database error occured while attempting to delete the user story."), NO_PROJECT(
			"A user story must be in a project."), NULL(
			"The given user story to update is null."), NAME_NULL(
			"Title cannot be null."), NAME_EMPTY("Title cannot be empty"), NAME_ALL_SPACES(
			"Title cannot only contain spaces."), NAME_BEGIN_WITH_SPACE(
			"Title cannot begin with a space."), NAME_NOT_UNIQUE(
			"Another user story with the same name already exists in this project."), NAME_TOO_LONG(
			"Name must be less than 255 characters."), DESCRIPTION_NULL(
			"Description cannot be null."), DESCRIPTION_EMPTY(
			"Description cannot be empty"), DESCRIPTION_ALL_SPACES(
			"Description cannot only contain spaces."), DESCRIPTION_TOO_LONG(
			"Description must be less than 255 characters."), ITERATION_INVALID(
			"Iteration is not an iteration of the same project as this user story."), ITERATION_OVER(
			"Iteration is already over; cannot add user story to it."), ESTIMATED_UNITS_NEGATIVE(
			"Estimated units cannot be negative."), ACTUAL_UNITS_NEGATIVE(
			"Acual units cannot be negative."), AUTHOR_INVALID(
			"The author must be part of the project."), STORY_COMPLETED(
			"A user story cannot be moved to a new iteration if it is completed."), OWNER_INVALID(
			"The owner must be part of the project.");
	private final String message;

	private UserStoryErrorType(String message) {
		this.message = message;
	}

	public String getMessage() {
		return message;
	}
}
