/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams - Initial design, API, and implementation
 *    Dan Bjorge - Implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data.funcs;

import java.sql.SQLException;

import edu.wpi.wpisuite.data.ActiveMatrixRelation;
import edu.wpi.wpisuite.data.TraceabilityMatrix;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriteria;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriterion;
import edu.wpi.wpisuite.data.errors.InvalidProjectInputException;
import edu.wpi.wpisuite.data.errors.ProjectErrorType;
import edu.wpi.wpisuite.database.Database;


/**
 * Incomplete class for functions dealing with Traceability Matrices.
 */
public class TraceabilityManager {

	/**
	 * Should be moved to a TracabilityMatrixManager class.  Deletes a <code> TraceabilityMatrix </code> and all its subcomponents.
	 * @param matrix
	 * @throws InvalidProejctInputException
	 */
	public static void deleteMatrix(TraceabilityMatrix matrix) throws InvalidProjectInputException {
		try {

			ActiveMatrixRelation[] activeAssocs = Database.manager().find(
					ActiveMatrixRelation.class, "traceabilityMatrixID = ?",
					matrix);
			Database.manager().delete(activeAssocs);

			if (matrix != null) {
				TraceabilityMatrixCriteria rowc = matrix.getRowCriteria();
				TraceabilityMatrixCriteria colc = matrix.getColumnCriteria();

				Database.manager().delete(matrix);

				if (rowc != null) {
					TraceabilityMatrixCriterion rown = rowc.getRootCriterion();

					Database.manager().delete(rowc);

					deleteCriterionTree(rown);
				}

				if (colc != null) {
					TraceabilityMatrixCriterion coln = colc.getRootCriterion();

					Database.manager().delete(colc);

					deleteCriterionTree(coln);
				}

			}

		} catch (SQLException e) {
			throw new InvalidProjectInputException(ProjectErrorType.DATABASE_PROJECT_DELETION ,"Error while deleting a matrix in this project.");
		}

	}
	
	/**
	 * Should be moved to a TracabilityMatrixManager class.  Deletes a tree of <code> TraceabilityMarixCriterion </code>.
	 * @param c
	 * @throws InvalidProjectInputException 
	 */
	public static void deleteCriterionTree(TraceabilityMatrixCriterion c) throws InvalidProjectInputException {
		if (c == null)
			return;

		if (c.getType() != null && c.getType().isTree())
			for (int id : c.getTypeData())
				deleteCriterionTree(Database.manager().get(
						TraceabilityMatrixCriterion.class, id));

		try {
			Database.manager().delete(c);
		} catch (SQLException e) {
			throw new InvalidProjectInputException(ProjectErrorType.DATABASE_PROJECT_DELETION ,"Error while deleting a matrix in this project.");
		}
	}
}
