/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.database;

import java.util.Scanner;

import org.jasypt.util.text.BasicTextEncryptor;

/**
 * This class provides a means of reversibly obsfucating a database password.
 * Right now it just encrypts it using a constant encryption password using the
 * jasypt encryption library. This function could concievably be modified in the
 * future for stronger encryption.
 */
public class DatabasePasswordCipher {
	/** The jasypt encryptor that will do the actual work */
	private BasicTextEncryptor textEncryptor;

	/** A constant password that will always be used for generation */
	private static final String defaultEncryptionPassword = "{dg,Ag\\-)Z%x~bXX;NpQz'%M_n{\"GUePz8|&sylHx(NM3tC8s3<auUjoS;t{w&;";

	/**
	 * Initializes the cipher with the default, constant password
	 */
	public DatabasePasswordCipher() {
		this(defaultEncryptionPassword);
	}

	/**
	 * Initializes the cipher with a password of choice.
	 * 
	 * @param encryptionPassword
	 *            The password to use FOR encryption. NOT the database password
	 *            to be encrypted.
	 */
	public DatabasePasswordCipher(String encryptionPassword) {
		textEncryptor = new BasicTextEncryptor();
		textEncryptor.setPassword(encryptionPassword);
	}

	/**
	 * Encrypts a password string using the preconfigured encryption system.
	 * 
	 * @param password
	 *            The cleartext database password to encrypt
	 * @return The ciphertext defining the password
	 */
	public String encrypt(String password) {
		return textEncryptor.encrypt(password);
	}

	/**
	 * Decrypts a password ciphertext using the preconfigured encryption system,
	 * which obviously must match whatever system was used to encrypt the string
	 * in the first place.
	 * 
	 * @param encryptedPassword
	 *            The ciphertext defining the password
	 * @return The cleartext of the password
	 */
	public String decrypt(String encryptedPassword) {
		return textEncryptor.decrypt(encryptedPassword);
	}

	public static void main(String[] args) {
		Scanner in = new Scanner(System.in);
		System.out.println(new DatabasePasswordCipher().encrypt(in.nextLine()));
	}
}
