/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module;

/**
 * 
 */
public class ActiveStateEvent {
	/** The different possible types of change event */
	public enum ActiveStateEventType {
		PROJECT, USER, STATUS
	}

	/** The state throwing this event */
	private ActiveState source;

	/** The type of change this event was triggered by */
	private ActiveStateEventType type;

	/**
	 * Constructs a new change event from the given source with the given type
	 * 
	 * @param source
	 *            The state whose change triggered this event
	 * @param type
	 *            The type of change triggering this event
	 */
	public ActiveStateEvent(ActiveState source, ActiveStateEventType type) {
		this.source = source;
		this.type = type;
	}

	/**
	 * @return The state whose change is causing this event
	 */
	public ActiveState getSource() {
		return source;
	}

	/**
	 * @return The type of change triggering this event
	 */
	public ActiveStateEventType getType() {
		return type;
	}
}
