/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module;

/**
 * Defines the current status of the program as a whole as a quick synopsis of
 * what the program is currently doing or has just finished doing. Interfaces
 * with the StatusBar to help give feedback to the user.
 */
public class Status {
	/** The possible types of message this Status might indicate */
	public enum StatusType {
		SUCCESS, FAILURE, ERROR, NONE
	}

	/** The message describing this status */
	private String message;

	/** The type of message this status describes */
	private StatusType type;

	/** Constructs a new status with type NONE and the given message */
	public Status(String message) {
		this(message, StatusType.NONE);
	}

	/** Constructs a new status with the given fields */
	public Status(String message, StatusType type) {
		this.message = message;
		this.type = type;
	}

	/** @return The message representing this status */
	public String getMessage() {
		return message;
	}

	/** @return The type of message this status represents */
	public StatusType getType() {
		return type;
	}

	/** @return A string representing this status (just the message) */
	@Override
	public String toString() {
		return getMessage();
	}
}
