/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module;

/**
 * An interface representing a container which references a Status that could
 * change. Provides methods to register StatusListeners to listen for such
 * changes. Implemented by ActiveState.
 */
public interface StatusState {
	/**
	 * Registers a new status listener to be notified of changes to the Status
	 * this object references
	 * 
	 * @param listener
	 *            The listener to register
	 */
	public void registerStatusListener(StatusListener listener);

	/**
	 * Unregisters a previously registered status listener
	 * 
	 * @param listener
	 *            The listener to unregister
	 */
	public void removeStatusListener(StatusListener listener);
}
