/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alex Kindle - GUI
 *    Bryan Crabtree - GUI
 *    Ian Williams - Implementation
 *    Lucas Scotta - Drag and drop
 *    Mihai Tiberiu Luca - Permissions
 *    Mihajlo Zeljkovic - Permissions
 *    
 *******************************************************************************/

package edu.wpi.wpisuite.module.projectadmin;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import javax.swing.DropMode;
import javax.swing.GroupLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.LayoutStyle;
import javax.swing.TransferHandler;


import org.apache.log4j.Logger;


import edu.wpi.wpisuite.components.EntityList;
import edu.wpi.wpisuite.components.EntityListCellRenderer;
import edu.wpi.wpisuite.components.EntityListModel;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.EditorPanelView;
import edu.wpi.wpisuite.components.transfer.EntityListTransferHandler;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.funcs.UserManager;
import edu.wpi.wpisuite.data.util.UserComparator;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.module.ModuleView;

/**
 * The view for the Project Admin module
 */
@SuppressWarnings("serial")
public class ProjectAdministrationView extends ModuleView {

	Logger logger = Logger.getLogger(this.getClass());
	private ProjectAdministration administration;
	private JLabel permUsernamelabel;
	private JCheckBox usCreateCheckBox;
	private JCheckBox usEditCheckBox;
	private JCheckBox usReadCheckBox;
	private JCheckBox fEditCheckBox;
	private JCheckBox fCreateCheckBox;
	private JCheckBox fReadCheckBox;
	private JCheckBox rCreateCheckBox;
	private JCheckBox rEditCheckBox;
	private JCheckBox rReadCheckBox;
	private JCheckBox iCreateCheckBox;
	private JCheckBox iEditCheckBox;
	private JCheckBox iReadCheckBox;
	private JCheckBox isProjectAdminCheckBox;

	/**
	 * Constructor
	 * 
	 * @param administration
	 */
	public ProjectAdministrationView(ProjectAdministration administration) {

		super(administration, new ImageIcon(
				"resources/OtherIcons/ProjectAdmin.PNG"));
		this.administration = administration;

		// Call Netbeans's init function
		this.initComponents();

		// Call our init functions
		this.initLists();

		// Register listeners
		administration.registerListeners();
	}

	public void setEditorView(EditorPanelView view) {
		this.topSplitPane.setLeftComponent(view);
	}

	/**
	 * Updates every component in the ProjectAdministrationView
	 */
	@Override
	public void updateAllComponents() {
		
		Project p = this.administration.getState().getProject();
		if (p != null) {
			// TODO put the list updates in a small update function
			updateUsersList(p);
			updateOtherUsersList(p);
		} else {
			this.usersList.getModel().clear();
			this.otherUsersList.getModel().clear();
		}

		this.administration.updateEditor();
	}

	private void updateOtherUsersList(Project p) {
		this.otherUsersList.clear();
		ArrayList<User> allNonProjectMembers = this.administration
				.getAllNonProjectMembers(p);
		for (User u : allNonProjectMembers)
			otherUsersList.getModel().addElement(u);
	}

	private void updateUsersList(Project p) {
		this.usersList.clear();
		User[] projectMembers = p.getMembers();
		Arrays.sort(projectMembers, new UserComparator());
		for (User u : projectMembers)
			usersList.getModel().addElement(u);
	}

	@Override
	public void initAddMenuItems() {

		JMenuItem projectAdder = new JMenuItem("Project", KeyEvent.VK_P);
		projectAdder.setIcon(icons.getProjectIcon(IconType.CREATE));

		ActionListener projectAdderListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ProjectAdministrationView.this.administration.createProject();
			}
		};
		projectAdder.addActionListener(projectAdderListener);

		addMenuItems.add(projectAdder);
	}

	/**
	 * Initializes the list listeners
	 */
	private void initLists() {
		// Project members list
		this.usersList.setDragEnabled(true);
		this.usersList.setDropMode(DropMode.INSERT);
		this.usersList.setCellRenderer(new EntityListCellRenderer());
		this.usersList.setTransferHandler(new EntityListTransferHandler<User>(
				User.class) {

			@SuppressWarnings("unchecked")
			@Override
			public boolean canImport(TransferHandler.TransferSupport support) {

				if (!super.canImport(support))
					return false;

				// users do not get duplicated within the same list
				ArrayList<User> transferUsers;

				// do not drop if there are errors
				try {
					transferUsers = (ArrayList<User>) support.getTransferable()
							.getTransferData(this.supportedDataFlavor);
				} catch (UnsupportedFlavorException e) {
					return false;
				} catch (IOException e) {
					return false;
				}

				EntityListModel<User> model = (EntityListModel<User>) ((JList) support
						.getComponent()).getModel();
				List<User> UsersOnList = model.getAllEntities();

				// see if any of the users already exist
				if (UsersOnList.containsAll(transferUsers))
					return false;

				return true;
			}

			@Override
			public void postImport(TransferHandler.TransferSupport support,
					List<User> data) {
				// toArray()
				User[] users = new User[data.size()];
				for (int i = 0; i < data.size(); i++)
					users[i] = data.get(i);

				administration.addUsersToProject(users, administration
						.getState().getProject());
				ProjectAdministrationView.this.updateAllComponents();
			}

		});

		// All users list
		this.otherUsersList.setDragEnabled(true);
		this.otherUsersList.setDropMode(DropMode.INSERT);
		this.otherUsersList.setCellRenderer(new EntityListCellRenderer());
		this.otherUsersList
				.setTransferHandler(new EntityListTransferHandler<User>(
						User.class) {

					@SuppressWarnings("unchecked")
					@Override
					public boolean canImport(
							TransferHandler.TransferSupport support) {

						if (!super.canImport(support))
							return false;

						// users do not get duplicated within the same list
						ArrayList<User> transferUsers;

						// do not drop if there are errors
						try {
							transferUsers = (ArrayList<User>) support
									.getTransferable().getTransferData(
											this.supportedDataFlavor);
						} catch (UnsupportedFlavorException e) {
							return false;
						} catch (IOException e) {
							return false;
						}

						EntityListModel<User> model = (EntityListModel<User>) ((JList) support
								.getComponent()).getModel();
						List<User> UsersOnList = model.getAllEntities();

						// see if any of the users already exist
						if (UsersOnList.containsAll(transferUsers))
							return false;

						return true;
					}

					@Override
					public void postImport(
							TransferHandler.TransferSupport support,
							List<User> data) {
						User[] users = new User[data.size()];
						for (int i = 0; i < data.size(); i++)
							users[i] = data.get(i);

						administration.removeUsersFromProjectAndUserStories(
								users, administration.getState().getProject());
						ProjectAdministrationView.this.updateAllComponents();
					}

				});

	}

	/*
	 * Opens the permission pane
	 */
	@SuppressWarnings("unused")
	private void selectUserForPermissions() {
		HashMap<String, Boolean> perms = null;
		User thisUser = usersList.getSelectedEntity();

		if (thisUser != null) {
			permUsernamelabel.setText(thisUser.getUsername());
			perms = UserManager.getPermissions(thisUser, administration
					.getState().getProject());
			updatePermssions(perms);
		}

		if (topSplitPane.getRightComponent() == null)
			topSplitPane.setRightComponent(permissionsPanel);
	}

	private void updatePermssions(HashMap<String, Boolean> perms) {
		usCreateCheckBox.setSelected(perms.get("canCreateUserStory"));
		usEditCheckBox.setSelected(perms.get("canEditUserStory"));
		usReadCheckBox.setSelected(perms.get("canReadUserStory"));
		fCreateCheckBox.setSelected(perms.get("canCreateFeature"));
		fEditCheckBox.setSelected(perms.get("canEditFeature"));
		fReadCheckBox.setSelected(perms.get("canReadFeature"));
		rCreateCheckBox.setSelected(perms.get("canCreateRelease"));
		rEditCheckBox.setSelected(perms.get("canEditRelease"));
		rReadCheckBox.setSelected(perms.get("canReadRelease"));
		iCreateCheckBox.setSelected(perms.get("canCreateIteration"));
		iEditCheckBox.setSelected(perms.get("canEditIteration"));
		iReadCheckBox.setSelected(perms.get("canReadIteration"));
		isProjectAdminCheckBox.setSelected(perms.get("isAdmin"));

	}

	/**
	 * Saves permissions from the pane check box selection
	 */
	private void savePermissions() {
		HashMap<String, Boolean> perms = new HashMap<String, Boolean>();
		User thisUser = null;
		try {
			thisUser = Database.manager().find(User.class, "username = ?",
					permUsernamelabel.getText())[0];
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		perms.put("canCreateUserStory", usCreateCheckBox.isSelected());
		perms.put("canEditUserStory", usEditCheckBox.isSelected());
		perms.put("canReadUserStory", usReadCheckBox.isSelected());
		perms.put("canCreateFeature", fCreateCheckBox.isSelected());
		perms.put("canEditFeature", fEditCheckBox.isSelected());
		perms.put("canReadFeature", fReadCheckBox.isSelected());
		perms.put("canCreateRelease", rCreateCheckBox.isSelected());
		perms.put("canEditRelease", rEditCheckBox.isSelected());
		perms.put("canReadRelease", rReadCheckBox.isSelected());
		perms.put("canCreateIteration", iCreateCheckBox.isSelected());
		perms.put("canEditIteration", iEditCheckBox.isSelected());
		perms.put("canReadIteration", iReadCheckBox.isSelected());
		perms.put("isAdmin", isProjectAdminCheckBox.isSelected());

		UserManager.setPermissions(thisUser, perms, administration.getState()
				.getProject());

		hidePermPane();
	}

	/**
	 * Hides permissions pane
	 */
	private void hidePermPane() {
		topSplitPane.setRightComponent(null);
	}

	private void removeUsersButtonActionPerformed(java.awt.event.ActionEvent evt) {
		this.administration.removeUsersButtonActionPerformed(evt);
	}

	private void addUsersButtonActionPerformed(java.awt.event.ActionEvent evt) {
		this.administration.addUsersButtonActionPerformed(evt);
	}

	/**
	 * @return the administrationAllUsersList
	 */
	public EntityList<User> getUsersList() {
		return this.usersList;
	}

	/**
	 * @return the administrationAllUsersList
	 */
	public EntityList<User> getOtherUsersList() {
		return this.otherUsersList;
	}

	// <editor-fold defaultstate="collapsed" desc="Generated Code">
	private void initComponents() {

		primarySplitPane = new javax.swing.JSplitPane();
		verticalSplitPane = new javax.swing.JSplitPane();
		usersPanel = new javax.swing.JPanel();
		usersScrollPane = new javax.swing.JScrollPane();
		usersList = new EntityList<User>();
		removeUsersButton = new javax.swing.JButton();
		otherUsersPanel = new javax.swing.JPanel();
		otherUsersScrollPane = new javax.swing.JScrollPane();
		otherUsersList = new EntityList<User>();
		addUsersButton = new javax.swing.JButton();
		topSplitPane = new javax.swing.JSplitPane();
		permissionsPanel = new javax.swing.JPanel();
		permissionsPanel.setPreferredSize(new Dimension(0, 0));

		setPreferredSize(new java.awt.Dimension(900, 600));

		primarySplitPane.setDividerLocation(300);
		primarySplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		primarySplitPane.setResizeWeight(0.5);
		primarySplitPane.setOneTouchExpandable(true);

		verticalSplitPane.setDividerLocation(450);
		verticalSplitPane.setResizeWeight(0.5);
		verticalSplitPane.setOneTouchExpandable(true);

		usersPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Project Members"));

		usersList.setVisibleRowCount(25);
		usersScrollPane.setViewportView(usersList);

		removeUsersButton.setText("Remove Selected Members from Project");
		removeUsersButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						removeUsersButtonActionPerformed(evt);
					}
				});

		javax.swing.GroupLayout usersPanelLayout = new javax.swing.GroupLayout(
				usersPanel);
		usersPanel.setLayout(usersPanelLayout);
		usersPanelLayout.setHorizontalGroup(usersPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						usersPanelLayout.createSequentialGroup().addComponent(
								removeUsersButton).addContainerGap())
				.addComponent(usersScrollPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 437,
						Short.MAX_VALUE));
		usersPanelLayout.setVerticalGroup(usersPanelLayout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				javax.swing.GroupLayout.Alignment.TRAILING,
				usersPanelLayout.createSequentialGroup().addComponent(
						usersScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE,
						236, Short.MAX_VALUE).addPreferredGap(
						javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(removeUsersButton)));

		verticalSplitPane.setLeftComponent(usersPanel);

		otherUsersPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Other Users"));

		otherUsersScrollPane.setViewportView(otherUsersList);

		addUsersButton.setText("Add Selected Users to Project");
		addUsersButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				addUsersButtonActionPerformed(evt);
			}
		});

		javax.swing.GroupLayout otherUsersPanelLayout = new javax.swing.GroupLayout(
				otherUsersPanel);
		otherUsersPanel.setLayout(otherUsersPanelLayout);
		otherUsersPanelLayout
				.setHorizontalGroup(otherUsersPanelLayout.createParallelGroup(
						javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								otherUsersPanelLayout.createSequentialGroup()
										.addComponent(addUsersButton)
										.addContainerGap()).addComponent(
								otherUsersScrollPane,
								javax.swing.GroupLayout.DEFAULT_SIZE, 430,
								Short.MAX_VALUE));
		otherUsersPanelLayout
				.setVerticalGroup(otherUsersPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								otherUsersPanelLayout
										.createSequentialGroup()
										.addComponent(
												otherUsersScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												236, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(addUsersButton)));

		verticalSplitPane.setRightComponent(otherUsersPanel);

		primarySplitPane.setRightComponent(verticalSplitPane);

		topSplitPane.setDividerLocation(600);

		permissionsPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Permissions"));

		// permissions pane
		JLabel usernameLabel;
		usernameLabel = new JLabel();
		usernameLabel.setFont(new Font("", Font.BOLD, 14));
		usernameLabel.setText("Username:");

		JLabel userStoryLabel;
		userStoryLabel = new JLabel();
		userStoryLabel.setFont(new Font("", Font.PLAIN, 14));
		userStoryLabel.setText("User Story");

		JLabel featureLabel;
		featureLabel = new JLabel();
		featureLabel.setFont(new Font("", Font.PLAIN, 14));
		featureLabel.setText("Feature");

		JLabel releaseLabel;
		releaseLabel = new JLabel();
		releaseLabel.setFont(new Font("", Font.PLAIN, 14));
		releaseLabel.setText("Release");

		JLabel iterationLabel;
		iterationLabel = new JLabel();
		iterationLabel.setFont(new Font("", Font.PLAIN, 14));
		iterationLabel.setText("Iteration");

		usCreateCheckBox = new JCheckBox();
		usCreateCheckBox.setText("Create");

		usEditCheckBox = new JCheckBox();
		usEditCheckBox.setText("Edit");

		usReadCheckBox = new JCheckBox();
		usReadCheckBox.setText("Read");

		fCreateCheckBox = new JCheckBox();
		fCreateCheckBox.setText("Create");

		fEditCheckBox = new JCheckBox();
		fEditCheckBox.setText("Edit");

		fReadCheckBox = new JCheckBox();
		fReadCheckBox.setText("Read");

		rCreateCheckBox = new JCheckBox();
		rCreateCheckBox.setText("Create");

		rEditCheckBox = new JCheckBox();
		rEditCheckBox.setText("Edit");

		rReadCheckBox = new JCheckBox();
		rReadCheckBox.setText("Read");

		iCreateCheckBox = new JCheckBox();
		iCreateCheckBox.setText("Create");

		iEditCheckBox = new JCheckBox();
		iEditCheckBox.setText("Edit");

		iReadCheckBox = new JCheckBox();
		iReadCheckBox.setText("Read");

		permUsernamelabel = new JLabel();
		permUsernamelabel.setFont(new Font("", Font.PLAIN, 14));

		JButton saveButton;
		saveButton = new JButton();
		saveButton.addActionListener(new ActionListener() {
			public void actionPerformed(final ActionEvent e) {
				savePermissions();
			}
		});
		saveButton.setText("Save");

		JButton cancelButton;
		cancelButton = new JButton();
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(final ActionEvent e) {
				hidePermPane();
			}
		});
		cancelButton.setText("Cancel");

		isProjectAdminCheckBox = new JCheckBox();
		isProjectAdminCheckBox.setText("Is Project Admin?");

		javax.swing.GroupLayout permissionsPanelLayout = new javax.swing.GroupLayout(
				permissionsPanel);
		permissionsPanelLayout
				.setHorizontalGroup(permissionsPanelLayout
						.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addGroup(
								permissionsPanelLayout
										.createSequentialGroup()
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.LEADING)
														.addComponent(
																isProjectAdminCheckBox)
														.addComponent(
																usReadCheckBox)
														.addComponent(
																usEditCheckBox)
														.addComponent(
																usCreateCheckBox)
														.addGroup(
																permissionsPanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				usernameLabel)
																		.addPreferredGap(
																				LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				permUsernamelabel))
														.addComponent(
																rCreateCheckBox)
														.addComponent(
																userStoryLabel)
														.addComponent(
																rReadCheckBox)
														.addComponent(
																releaseLabel)
														.addComponent(
																rEditCheckBox))
										.addGap(8, 8, 8)
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.LEADING)
														.addGroup(
																permissionsPanelLayout
																		.createSequentialGroup()
																		.addGap(
																				17,
																				17,
																				17)
																		.addGroup(
																				permissionsPanelLayout
																						.createParallelGroup(
																								GroupLayout.Alignment.LEADING)
																						.addComponent(
																								featureLabel)
																						.addComponent(
																								fReadCheckBox)
																						.addComponent(
																								fCreateCheckBox)
																						.addComponent(
																								fEditCheckBox)
																						.addComponent(
																								iCreateCheckBox)
																						.addComponent(
																								iEditCheckBox)
																						.addGroup(
																								permissionsPanelLayout
																										.createParallelGroup(
																												GroupLayout.Alignment.TRAILING)
																										.addComponent(
																												iterationLabel)
																										.addComponent(
																												iReadCheckBox))))
														.addGroup(
																permissionsPanelLayout
																		.createSequentialGroup()
																		.addGap(
																				54,
																				54,
																				54)
																		.addComponent(
																				saveButton)
																		.addPreferredGap(
																				LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				cancelButton)))
										.addContainerGap(
												GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));
		permissionsPanelLayout
				.setVerticalGroup(permissionsPanelLayout
						.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addGroup(
								permissionsPanelLayout
										.createSequentialGroup()
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(
																usernameLabel)
														.addComponent(
																permUsernamelabel))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(
																featureLabel)
														.addComponent(
																userStoryLabel))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(
																usReadCheckBox)
														.addComponent(
																fReadCheckBox))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(
																usCreateCheckBox)
														.addComponent(
																fCreateCheckBox))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(
																usEditCheckBox)
														.addComponent(
																fEditCheckBox))
										.addGap(9, 9, 9)
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(
																releaseLabel)
														.addComponent(
																iterationLabel))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(
																rReadCheckBox)
														.addComponent(
																iReadCheckBox))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(
																rCreateCheckBox)
														.addComponent(
																iCreateCheckBox))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(
																rEditCheckBox)
														.addComponent(
																iEditCheckBox))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED,
												47, Short.MAX_VALUE)
										.addGroup(
												permissionsPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(
																cancelButton)
														.addComponent(
																saveButton)
														.addComponent(
																isProjectAdminCheckBox))));
		permissionsPanel.setLayout(permissionsPanelLayout);

		primarySplitPane.setLeftComponent(topSplitPane);

		topSplitPane.setRightComponent(null);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				primarySplitPane, javax.swing.GroupLayout.DEFAULT_SIZE, 900,
				Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				primarySplitPane, javax.swing.GroupLayout.Alignment.TRAILING,
				javax.swing.GroupLayout.DEFAULT_SIZE, 600, Short.MAX_VALUE));
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JButton addUsersButton;
	private EntityList<User> otherUsersList;
	private javax.swing.JPanel otherUsersPanel;
	private javax.swing.JScrollPane otherUsersScrollPane;
	private javax.swing.JSplitPane primarySplitPane;
	private javax.swing.JButton removeUsersButton;
	private EntityList<User> usersList;
	private javax.swing.JPanel usersPanel;
	private javax.swing.JScrollPane usersScrollPane;
	private javax.swing.JSplitPane verticalSplitPane;
	private javax.swing.JPanel permissionsPanel;
	private javax.swing.JSplitPane topSplitPane;
	// End of variables declaration

}
