/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ravi Singh
 *    
 *******************************************************************************/

package edu.wpi.wpisuite.module.rm;

import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;

import javax.swing.DropMode;
import javax.swing.GroupLayout;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.LayoutStyle;
import javax.swing.TransferHandler;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.TableColumn;
import javax.swing.table.TableRowSorter;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import net.java.ao.Entity;

import edu.wpi.wpisuite.components.EntityList;
import edu.wpi.wpisuite.components.EntityListCellRenderer;
import edu.wpi.wpisuite.components.EntityListModel;
import edu.wpi.wpisuite.components.EntityTreeCellRenderer;
import edu.wpi.wpisuite.components.EntityTreeNode;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.EditorPanelView;
import edu.wpi.wpisuite.components.editor.Editor.ConfirmResult;
import edu.wpi.wpisuite.components.transfer.DataTransferable;
import edu.wpi.wpisuite.components.transfer.EntityListTransferHandler;
import edu.wpi.wpisuite.components.transfer.UserStoryTableTransferHandler;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.errors.InvalidUserStoryInputException;
import edu.wpi.wpisuite.data.funcs.IterationManager;
import edu.wpi.wpisuite.data.funcs.UserStoryManager;
import edu.wpi.wpisuite.data.util.IterationComparator;
import edu.wpi.wpisuite.data.util.ReleaseComparator;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.module.ActiveStateEvent;
import edu.wpi.wpisuite.module.ActiveStateListener;
import edu.wpi.wpisuite.module.ModuleView;
import edu.wpi.wpisuite.module.projectadmin.ProjectAdministration;


/**
 * The view for the RequirementsManager module
 */
@SuppressWarnings("serial")
public class RequirementsManagerView extends ModuleView {

	/** The width of a single item of the lists in the timeline */
	private final int timelineCellWidth = 120;

	private RequirementsManager requirementsManager;
	private UserStoryTableModel userStoriesTableModel;
	private EntityPopupMenu iterationsPopupMenu;
	private EntityPopupMenu releasePopupMenu;
	private EntityPopupMenu backlogPopupMenu;
	private EntityPopupMenu mainTablePopupMenu;
	private EntityPopupMenu otherRequirementsPopupMenu;

	/*
	 * private static HashMap<String, Boolean> permissionsHash = new
	 * HashMap<String, Boolean>();
	 */// permissions-related
	/**
	 * Initializes this view and its components. See init methods for details.
	 * 
	 * @param requirementManager
	 *            The Module this is a view of.
	 */
	public RequirementsManagerView(RequirementsManager requirementsManager) {
		super(requirementsManager, new ImageIcon(
				"resources/OtherIcons/Requirements.PNG"));
		this.requirementsManager = requirementsManager;
		this.initRequirementsManagerView();
	}

	/**
	 * Initializes requirements manager GUI components
	 */
	private void initRequirementsManagerView() {
		this.initComponents();
		this.initReleaseList();
		this.initIterationsList();
		this.initBacklogList();
		this.initOtherList();
		this.initProjectMembersTree();
		this.initUserStoriesTable();
		this.initPopupMenus(); // last

		// initListeners must be called last
		this.initListeners();
	}

	/**
	 * This method is called from the constructor to add listeners to elements
	 * of the form. This is hand-done - you MAY modify this code.
	 */
	private void initListeners() {
		this.iterationList
				.addListSelectionListener(new ListSelectionListener() {
					@Override
					public void valueChanged(ListSelectionEvent evt) {
						RequirementsManagerView.this
								.onIterationsListSelectionChange(evt);
					}
				});

		this.releaseList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent evt) {
				RequirementsManagerView.this.onReleaseListSelectionChange(evt);
			}
		});

		this.releaseList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent evt) {
				RequirementsManagerView.this.onReleaseListSelectionChange(evt);
			}
		});

		this.userTree.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
				onProjectMembersTreeSelectionChange(e);
			}

		});

		this.allIterationsButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {

				int[] indices = new int[RequirementsManagerView.this.iterationList
						.getModel().getSize()];
				for (int i = 0; i < indices.length; i++)
					indices[i] = i;
				RequirementsManagerView.this.iterationList
						.setSelectedIndices(indices);
			}
		});

		this.noIterationsButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {

				int[] indices = new int[RequirementsManagerView.this.iterationList
						.getModel().getSize()];
				for (int i = 0; i < indices.length; i++)
					indices[i] = i;
				RequirementsManagerView.this.iterationList
						.setSelectedIndices(new int[0]);
			}
		});

		this.otherList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent evt) {
				/*
				 * boolean canRead = permissionsHash.get("canReadFeature");
				 * boolean canCreate = permissionsHash.get("canCreateFeature");
				 * boolean canEdit = permissionsHash.get("canEditFeature");
				 * 
				 * if (!evt.getValueIsAdjusting()) {
				 * requirementsManager.entitySelectionChange
				 * (otherList.getSelectedEntity(), canRead, canCreate, canEdit);
				 * }
				 */// permissions-related
				if (!evt.getValueIsAdjusting())
					requirementsManager.entitySelectionChange(otherList
							.getSelectedEntity());
			}
		});

		this.otherList.addMouseListener(new MouseAdapter() {
			// De-select any selected requirements in other components
			// Only one component should have selected requirements at a time
			@Override
			public void mousePressed(MouseEvent evt) {
				RequirementsManagerView.this.backlogList.clearSelection();
				RequirementsManagerView.this.mainTable.clearSelection();
			}

			/**
			 * Mouse Listener for when a popup menu trigger is released(eg.
			 * right mouse click), displays popup menu.
			 */
			@Override
			public void mouseReleased(MouseEvent evt) {
				if (evt.isPopupTrigger())
					RequirementsManagerView.this.otherList
							.setSelectedIndex(RequirementsManagerView.this.otherList
									.locationToIndex(evt.getPoint()));

				if (evt.isPopupTrigger()
						&& RequirementsManagerView.this.otherList
								.isSelectionEmpty()) {
					otherRequirementsPopupMenu.allowCreate(true);
					otherRequirementsPopupMenu.allowDelete(false);
					otherRequirementsPopupMenu.show(otherList, evt.getX(), evt
							.getY());
				}

				if ((evt.isPopupTrigger())
						&& !RequirementsManagerView.this.otherList
								.isSelectionEmpty()) {
					otherRequirementsPopupMenu.allowCreate(true);
					otherRequirementsPopupMenu.allowDelete(true);
					otherRequirementsPopupMenu.show(otherList, evt.getX(), evt
							.getY());
				}

			}
		});

		this.backlogList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent evt) {
				if (!evt.getValueIsAdjusting())
					requirementsManager.entitySelectionChange(backlogList
							.getSelectedEntity());
			}
		});

		this.iterationList.addMouseListener(new MouseAdapter() {

			/**
			 * Mouse Listener for iterationsList when a popup menu trigger is
			 * released(eg. right mouse click), displays popup menu.
			 */
			@Override
			public void mouseReleased(MouseEvent evt) {
				if (evt.isPopupTrigger())
					RequirementsManagerView.this.iterationList
							.setSelectedIndex(RequirementsManagerView.this.iterationList
									.locationToIndex(evt.getPoint()));

				if (evt.isPopupTrigger()
						&& RequirementsManagerView.this.iterationList
								.isSelectionEmpty()) {
					iterationsPopupMenu.allowCreate(true);
					iterationsPopupMenu.allowDelete(false);
					iterationsPopupMenu.show(iterationList, evt.getX(), evt
							.getY());
				}

				if ((evt.isPopupTrigger())
						&& !RequirementsManagerView.this.iterationList
								.isSelectionEmpty()) {
					iterationsPopupMenu.allowCreate(true);
					iterationsPopupMenu.allowDelete(true);
					iterationsPopupMenu.show(iterationList, evt.getX(), evt
							.getY());
				}

				onMouseReleasedDisplayIterationEditor();
			}
		});

		this.releaseList.addMouseListener(new MouseAdapter() {

			/**
			 * Mouse Listener for releaseList when a popup menu trigger is
			 * released(eg. right mouse click), displays popup menu.
			 */
			@Override
			public void mouseReleased(MouseEvent evt) {
				if (evt.isPopupTrigger())
					RequirementsManagerView.this.releaseList
							.setSelectedIndex(RequirementsManagerView.this.releaseList
									.locationToIndex(evt.getPoint()));

				if (evt.isPopupTrigger()
						&& RequirementsManagerView.this.releaseList
								.isSelectionEmpty()) {
					releasePopupMenu.allowCreate(true);
					releasePopupMenu.allowDelete(false);
					releasePopupMenu.show(releaseList, evt.getX(), evt.getY());
				}

				if ((evt.isPopupTrigger())
						&& !RequirementsManagerView.this.releaseList
								.isSelectionEmpty()) {
					releasePopupMenu.allowCreate(true);
					releasePopupMenu.allowDelete(true);
					releasePopupMenu.show(releaseList, evt.getX(), evt.getY());
				}

				onMouseReleasedDisplayReleaseEditor();
			}
		});

		this.backlogList.addMouseListener(new MouseAdapter() {

			// De-select any selected requirements in other components
			// Only one component should have selected requirements at a time
			@Override
			public void mousePressed(MouseEvent evt) {
				RequirementsManagerView.this.mainTable.clearSelection();
				RequirementsManagerView.this.otherList.clearSelection();
			}

			/**
			 * Mouse Listener for iterationsList when a popup menu trigger is
			 * released(eg. right mouse click), displays popup menu.
			 */
			@Override
			public void mouseReleased(MouseEvent evt) {
				if (evt.isPopupTrigger())
					RequirementsManagerView.this.backlogList
							.setSelectedIndex(RequirementsManagerView.this.backlogList
									.locationToIndex(evt.getPoint()));

				if (evt.isPopupTrigger()
						&& RequirementsManagerView.this.backlogList
								.isSelectionEmpty()) {
					backlogPopupMenu.allowCreate(true);
					backlogPopupMenu.allowDelete(false);
					backlogPopupMenu.show(backlogList, evt.getX(), evt.getY());
				}

				if ((evt.isPopupTrigger())
						&& !RequirementsManagerView.this.backlogList
								.isSelectionEmpty()) {
					backlogPopupMenu.allowCreate(true);
					backlogPopupMenu.allowDelete(true);
					backlogPopupMenu.show(backlogList, evt.getX(), evt.getY());
				}

			}

		});

		this.mainTable.addMouseListener(new MouseAdapter() {

			// De-select any selected requirements in other components
			// Only one component should have selected requirements at a time
			@Override
			public void mousePressed(MouseEvent evt) {
				RequirementsManagerView.this.backlogList.clearSelection();
				RequirementsManagerView.this.otherList.clearSelection();
			}

			@Override
			public void mouseReleased(MouseEvent evt) {
				if (evt.isPopupTrigger()) {
					int index = RequirementsManagerView.this.mainTable
							.rowAtPoint(evt.getPoint());
					if (index >= 0)
						RequirementsManagerView.this.mainTable
								.setRowSelectionInterval(index, index);
				}

				if (evt.isPopupTrigger()
						&& (RequirementsManagerView.this.mainTable
								.getSelectedRow() == -1)) {
					mainTablePopupMenu.allowCreate(true);
					mainTablePopupMenu.allowDelete(false);
					mainTablePopupMenu.show(mainTable, evt.getX(), evt.getY());
				}

				if (evt.isPopupTrigger()
						&& (RequirementsManagerView.this.mainTable
								.getSelectedRow() != -1)) {
					mainTablePopupMenu.allowCreate(true);
					mainTablePopupMenu.allowDelete(true);
					mainTablePopupMenu.show(mainTable, evt.getX(), evt.getY());
				}

				if (RequirementsManagerView.this.mainTable.getSelectedRow() != -1)
					RequirementsManagerView.this.requirementsManager
							.entitySelectionChange(RequirementsManagerView.this.userStoriesTableModel
									.getUserStoryAt(RequirementsManagerView.this.mainTable
											.convertRowIndexToModel(RequirementsManagerView.this.mainTable
													.getSelectedRow())));
			}
		});

		// Listener for current row selection change in user story table
		this.mainTable.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
					@Override
					public void valueChanged(ListSelectionEvent evt) {
						if (!evt.getValueIsAdjusting())
							if (RequirementsManagerView.this.mainTable
									.getSelectedRow() != -1)
								RequirementsManagerView.this.requirementsManager
										.entitySelectionChange(RequirementsManagerView.this.userStoriesTableModel
												.getUserStoryAt(RequirementsManagerView.this.mainTable
														.convertRowIndexToModel(RequirementsManagerView.this.mainTable
																.getSelectedRow())));
					}
				});

		this.getModule().getState().registerListener(new ActiveStateListener() {
			@Override
			public void stateChange(ActiveStateEvent event) {
				if (requirementsManager == requirementsManager.getState().getModule())
					RequirementsManagerView.this.updateAllComponents();
			}
		});
	}

	@Override
	public void initAddMenuItems() {

		JMenuItem iterationAdder = new JMenuItem("Iteration", KeyEvent.VK_I);
		iterationAdder.setIcon(icons.getIterationIcon(IconType.CREATE));
		iterationAdder.setAccelerator(javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_I,
				java.awt.event.InputEvent.CTRL_MASK));

		ActionListener iterationAdderListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				RequirementsManagerView.this.requirementsManager
						.newEntityCreator(Iteration.class);
			}
		};
		iterationAdder.addActionListener(iterationAdderListener);

		// User Story
		JMenuItem userstoryAdder = new JMenuItem("User Story", KeyEvent.VK_U);
		userstoryAdder.setIcon(icons.getUserStoryIcon(IconType.CREATE));
		userstoryAdder.setAccelerator(javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_U,
				java.awt.event.InputEvent.CTRL_MASK));

		ActionListener userstoryAdderListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				RequirementsManagerView.this.requirementsManager
						.newEntityCreator(UserStory.class);
			}
		};
		userstoryAdder.addActionListener(userstoryAdderListener);

		// Feature
		JMenuItem featureAdder = new JMenuItem("Feature", KeyEvent.VK_F);
		featureAdder.setIcon(icons.getFeatureIcon(IconType.CREATE));
		featureAdder.setAccelerator(javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_F,
				java.awt.event.InputEvent.CTRL_MASK));

		ActionListener featureAdderListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				RequirementsManagerView.this.requirementsManager
						.newEntityCreator(Feature.class);
			}
		};
		featureAdder.addActionListener(featureAdderListener);

		// Release
		JMenuItem releaseAdder = new JMenuItem("Release", KeyEvent.VK_R);
		releaseAdder.setIcon(icons.getReleaseIcon(IconType.CREATE));
		releaseAdder.setAccelerator(javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_R,
				java.awt.event.InputEvent.CTRL_MASK));

		ActionListener releaseAdderListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				RequirementsManagerView.this.requirementsManager
						.newEntityCreator(Release.class);
			}
		};
		releaseAdder.addActionListener(releaseAdderListener);

		addMenuItems.add(releaseAdder);
		addMenuItems.add(iterationAdder);
		addMenuItems.add(userstoryAdder);
		addMenuItems.add(featureAdder);
	}

	public void changeEntityEditorView(EditorPanelView newEditorView) {
		if (newEditorView == null) {
			this.verticalSplitPane.setRightComponent(null);
			this.verticalSplitPane.setDividerLocation(1.0);
		} else {
			this.verticalSplitPane.setRightComponent(newEditorView);
			int editorHeight = newEditorView.getPreferredSize().height;
			int splitPaneHeight = this.verticalSplitPane.getSize().height;
			this.verticalSplitPane.setDividerLocation(splitPaneHeight
					- editorHeight);
		}
	}

	public void closeEditorView() {
		this.changeEntityEditorView(null);
	}

	/**
	 * Tells the project members tree and user stories table to be populated
	 * with data from the newly selected iteration
	 * 
	 * @param evt
	 *            The ListSelectionEvent causing this to be called
	 */
	private void onIterationsListSelectionChange(ListSelectionEvent evt) {
		if (!evt.getValueIsAdjusting()) { // don't change stuff unless user is
			// done
			this.updateUserStoriesTable();
			this.updateProjectMembersTree();
			if (this.getSelectedIterations().size() == 1)
				this.requirementsManager.entitySelectionChange(this
						.getSelectedIterations().get(0));
			else
				this.requirementsManager.setEntityEditor(null);
		}
	}

	/**
	 * When the selected release changes
	 * 
	 * @param evt
	 *            The ListSelectionEvent causing this to be called
	 */
	private void onReleaseListSelectionChange(ListSelectionEvent evt) {
		if (!evt.getValueIsAdjusting()) {
			// TODO: select iterations belonging to this release

			ArrayList<Iteration> iterations = new ArrayList<Iteration>();
			for (Release r : this.getSelectedReleases()) {
				iterations.addAll(Arrays.asList(r.getIterations()));
				setSelectedIterations(iterations);
			}

			if (this.getSelectedReleases().size() == 1)
				this.requirementsManager.entitySelectionChange(this
						.getSelectedReleases().get(0));
			else
				this.requirementsManager.setEntityEditor(null);
		}
	}

	public void setSelectedIterations(ArrayList<Iteration> iterations) {
		iterationList.clearSelection();

		for (Iteration it : iterations)
			for (int i = 0; i < iterationList.getModel().getSize(); i++)
				if (it.getID() == iterationList.getModel().getEntity(i).getID())
					iterationList.addSelectionInterval(i, i);
	}

	/**
	 * Displays the iteration editor pane for the selected iteration if clicked
	 * on. Added so that if the user selects an iteration, and then proceeds to
	 * work on user stories the editor pane changes to the user story editor. If
	 * the user clicks on the already selected iteration this method displays
	 * the iteration editor along with the details of the iteration.
	 */
	private void onMouseReleasedDisplayIterationEditor() {
		if (this.getSelectedIterations().size() == 1)
			this.requirementsManager.entitySelectionChange(this
					.getSelectedIterations().get(0));
	}

	private void onMouseReleasedDisplayReleaseEditor() {
		if (this.getSelectedReleases().size() == 1)
			this.requirementsManager.entitySelectionChange(this
					.getSelectedReleases().get(0));
	}

	// TODO All these things allow errors. Fix that. Use manager when their
	// ready.

	private void onBackLogListPopupMenuRemoveItemClick(MouseEvent evt) {

		UserStory userStoryToBeRemoved = backlogList.getSelectedEntity();
		try {
			Database.manager().delete(userStoryToBeRemoved);
			this.updateBacklogList();
		} catch (SQLException e) {
			this.popErrorMessage(e);
		}
	}

	private void onOtherRequirementsListPopupMenuRemoveItemClick(MouseEvent evt) {

		Feature featureToBeRemoved = otherList.getSelectedEntity();
		try {
			Database.manager().delete(featureToBeRemoved);
			this.updateOtherList();
		} catch (SQLException e) {
			this.popErrorMessage(e);
		}
	}

	private void onIterationsListPopupMenuCreateItemClick(MouseEvent evt) {
		this.requirementsManager.newEntityCreator(Iteration.class);
	}

	private void onIterationsListPopupMenuRemoveItemClick(MouseEvent evt) {
		List<Iteration> its = this.getSelectedIterations();
		if (its.size() != 1)
			return;
		try {
			this.requirementsManager.deleteIteration(its.get(0));
		} catch (IOException e) {
			this.popErrorMessage(e);
		}
	}

	private void onReleaseListPopupMenuCreateItemClick(MouseEvent evt) {
		this.requirementsManager.newEntityCreator(Release.class);
	}

	private void onReleaseListPopupMenuRemoveItemClick(MouseEvent evt) {
		// TODO
		JOptionPane.showMessageDialog(null, "TODO, delete release!");
	}

	private void onBackLogListPopupMenuCreateItemClick(MouseEvent evt) {
		this.requirementsManager.newEntityCreator(UserStory.class);
	}

	private void onOtherRequirementsListPopupMenuCreateItemClick(MouseEvent evt) {
		this.requirementsManager.newEntityCreator(Feature.class);
	}

	private void createMainTablePopupMenuItemReleased(MouseEvent evt) {
		// Iteration i =
		// iterationsListModel.getIteration(iterationsList.getSelectedIndex());
		// int index = iterationsList.getSelectedIndex();
		// TODO add a way to initialize the userStory creation panel with
		// default iteration
		this.requirementsManager.newEntityCreator(UserStory.class);
	}

	private void removeMainTableRemoveMenuItemReleased(MouseEvent evt) {
		UserStory userStoryToBeRemoved = this.userStoriesTableModel
				.getUserStoryAt(this.mainTable
						.convertRowIndexToModel(this.mainTable.getSelectedRow()));
		try {
			Database.manager().delete(userStoryToBeRemoved);
			int index = this.iterationList.getSelectedIndex();
			this.updateUserStoriesTable();
			this.iterationList.setSelectedIndex(index);
		} catch (SQLException e) {
			this.popErrorMessage(e);
		}
	}

	/**
	 * Sets the iterations list to be modeled by our custom handler object
	 * 
	 * @see IterationsListModel
	 */
	private void initIterationsList() {
		this.iterationList.setDropMode(DropMode.ON);
		// iterationsList.setDropMode(DropMode.INSERT_ROWS);
		this.iterationList
				.setTransferHandler(new EntityListTransferHandler<UserStory>(
						UserStory.class) {
					private static final long serialVersionUID = 829914674092219405L;

					@Override
					protected Transferable createTransferable(
							JComponent component) {

						int[] selectedIndices = ((JList) component)
								.getSelectedIndices();
						IterationsListModel listModel = (IterationsListModel) ((JList) component)
								.getModel();
						List<Iteration> selectedObjects = new ArrayList<Iteration>();

						for (int i = 0; i < selectedIndices.length; i++)
							selectedObjects.add(listModel
									.getEntity(selectedIndices[i]));

						Transferable transferable = new DataTransferable<Iteration>(
								selectedObjects, Iteration.class);
						return transferable;
					}

					@Override
					public boolean canImport(
							TransferHandler.TransferSupport support) {

						if (!super.canImport(support))
							return false;

						// must drop directly on an iteration
						if (((JList.DropLocation) support.getDropLocation())
								.getIndex() == -1)
							// TODO I THINK this is what doesn't allow paste
							// into an iteration (that is, while the iteraion is
							// the point of focus)
							return false;

						return true;
					}

					@Override
					public void postImport(
							TransferHandler.TransferSupport support,
							List<UserStory> data) {
						// The drop mode for the iterations list is DropMode.ON
						JList.DropLocation dropLocation = (JList.DropLocation) support
								.getDropLocation();
						JList list = (JList) support.getComponent();
						int targetIndex = dropLocation.getIndex();

						boolean changed = false;

						if (targetIndex != -1) {
							// user stories(s) were dragged to an iteration
							for (UserStory story : data) {
								Iteration targetIteration = ((IterationsListModel) list
										.getModel()).getEntity(targetIndex);

								// From other project; copy it
								if (!story.getProject().equals(
										requirementsManager.getState()
												.getProject())) {
									story = importUserStory(story,
											requirementsManager.getState()
													.getProject(),
											targetIteration,
											requirementsManager.getState()
													.getUser());
									if (story.getProject().equals(
											requirementsManager.getState()
													.getProject()))
										changed = true;
								} else
									try {
										UserStoryManager.updateIteration(story,
												targetIteration);
										changed = true;
									} catch (InvalidUserStoryInputException e) {
										JOptionPane.showMessageDialog(null, e
												.getMessage(), "Error",
												JOptionPane.ERROR_MESSAGE);
									}
							}
							if (changed) {
								RequirementsManagerView.this
										.updateBacklogList();
								RequirementsManagerView.this
										.updateUserStoriesTable();
							}
						} else {
							// user stories(s) were dragged to empty space
						}
					}
				});

		this.updateIterationsList();
	}

	/**
	 * Sets the release list to be modeled by our custom handler object
	 * 
	 * @see ReleaseListModel
	 */
	private void initReleaseList() {
		this.releaseList.setDragEnabled(false);
		this.releaseList.setDropMode(DropMode.ON);
		this.releaseList
				.setTransferHandler(new EntityListTransferHandler<Iteration>(
						Iteration.class) {

					@Override
					public boolean canImport(
							TransferHandler.TransferSupport support) {

						if (!super.canImport(support))
							return false;

						// must drop directly on a release
						if (((JList.DropLocation) support.getDropLocation())
								.getIndex() == -1)
							return false;

						return true;
					}

					@Override
					public void postImport(
							TransferHandler.TransferSupport support,
							List<Iteration> data) {

						// The drop mode for the release list is DropMode.ON
						JList.DropLocation dropLocation = (JList.DropLocation) support
								.getDropLocation();
						JList list = (JList) support.getComponent();
						int targetIndex = dropLocation.getIndex();

						if (targetIndex != -1) {
							// iteration(s) were dragged to a release
							Release targetRelease = ((ReleaseListModel) list
									.getModel()).getEntity(targetIndex);

							for (Iteration it : data) {
								it.setRelease(targetRelease);
								it.save();
							}
						} else {
							// iteration(s) were dragged to empty space
						}
					}
				});

		this.updateReleaseList();
	}

	/**
	 * Dialog popped when copying user stories from one project to another Asks
	 * if the owner should be kept or clear
	 * 
	 * @param story
	 * @return true if the current owner should be kept, or false if it should
	 *         not
	 */
	private boolean shouldKeepUserStoryOwnerDialog(UserStory story) {
		return JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(null,
				"Requirement '" + story.getTitle()
						+ "' is already owned by user '"
						+ story.getOwner().getUsername()
						+ "'. Keep this owner?", "Copy user story",
				JOptionPane.YES_NO_OPTION);
	}

	/**
	 * Import a user story to a project. Creates a copy of the user story with
	 * the project set to the given project. Asks if the owner should be kept or
	 * cleared.
	 * 
	 * @param story
	 *            the user story to be imported
	 * @param p
	 *            the project to import the user story to
	 * @return the new user story
	 */
	private UserStory importUserStory(UserStory story, Project p,
			Iteration iteration, User user) {
		boolean keepOwner = false;

		if (story.getOwner() != null
				&& Arrays.asList(p.getMembers()).contains(story.getOwner()))
			keepOwner = shouldKeepUserStoryOwnerDialog(story);
		try {
			story = UserStoryManager.copyStory(story, requirementsManager
					.getState().getProject(), iteration, user, keepOwner);
		} catch (InvalidUserStoryInputException e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Error",
					JOptionPane.ERROR_MESSAGE);
		}
		// story.setProject(requirementsManager.getState().getProject());

		return story;
	}

	/**
	 * Updates the list of iterations and keeps the old selected iterations too.
	 * 
	 * @param
	 */
	private void updateIterationsList() {

		Iteration[] iterations = this.requirementsManager.getIterationList();
		Arrays.sort(iterations, new IterationComparator());
		List<Iteration> oldSelected = this.getSelectedIterations();

		this.iterationList.clear();

		if (iterations != null)
			for (Iteration it : iterations)
				this.iterationList.getModel().addElement(it);
		for (Iteration it : oldSelected)
			for (int i = 0; i < this.iterationList.getModel().getSize(); i++) {
				int id = this.iterationList.getModel().getEntity(i).getID();
				if (id == it.getID()) {
					this.iterationList.addSelectionInterval(i, i);
					break;
				}
			}

		// color assignment to iterations.
		this.iterationList.clearFontStyles();
		Project p = requirementsManager.getState().getProject();
		if (p != null) {

			List<Iteration> iterationsInList = iterationList.getModel()
					.getAllEntities();

			for (Iteration i : iterationsInList)
				if (IterationManager.isActive(i))
					iterationList.setEntityBold(i);

			/*
			 * Iteration[] compIterations =
			 * ProjectManager.getCompletedIterations(p);
			 * 
			 * for (Iteration i : compIterations) {
			 * this.iterationsList.setEntityColor(i, Color.gray); }
			 */
		}
	}

	/**
	 * Updates the list of releases
	 * 
	 * @param
	 */
	private void updateReleaseList() {

		Release[] releases = this.requirementsManager.getReleaseList();
		Arrays.sort(releases, new ReleaseComparator());
		List<Release> oldSelected = this.getSelectedReleases();

		this.releaseList.clear();

		if (releases != null)
			for (Release r : releases)
				this.releaseList.getModel().addElement(r);
		for (Release r : oldSelected)
			for (int i = 0; i < this.releaseList.getModel().getSize(); i++) {
				int id = this.releaseList.getModel().getEntity(i).getID();
				if (id == r.getID()) {
					this.releaseList.addSelectionInterval(i, i);
					break;
				}
			}
	}

	/**
	 * Initializes the backlog list with a custom model and transfer handler
	 */
	private void initBacklogList() {
		this.backlogList.setDragEnabled(true);
		this.backlogList.setCellRenderer(new EntityListCellRenderer());
		this.backlogList
				.setTransferHandler(new EntityListTransferHandler<UserStory>(
						UserStory.class) {

					@SuppressWarnings("unchecked")
					@Override
					public boolean canImport(
							TransferHandler.TransferSupport support) {

						if (!super.canImport(support))
							return false;

						// no duplicate values
						ArrayList<UserStory> transferStories;
						try {
							transferStories = (ArrayList<UserStory>) support
									.getTransferable().getTransferData(
											this.supportedDataFlavor);
						} catch (UnsupportedFlavorException e) {
							return false;
						} catch (IOException e) {
							return false;
						}
						EntityListModel<UserStory> model = (EntityListModel<UserStory>) ((JList) support
								.getComponent()).getModel();
						List<UserStory> listStories = model.getAllEntities();
						if (listStories.containsAll(transferStories))
							return false;

						return true;
					}

					@Override
					public void postImport(
							TransferHandler.TransferSupport support,
							List<UserStory> data) {
						/*
						 * UserStory[] stories = new UserStory[data.size()]; for
						 * (int i = 0; i < data.size(); i++) { stories[i] =
						 * data.get(i); }(
						 */
						boolean changed = false;

						for (UserStory story : data)
							// From other project; copy it
							if (!story.getProject()
									.equals(
											requirementsManager.getState()
													.getProject())) {
								story = importUserStory(story,
										requirementsManager.getState()
												.getProject(),
										UserStoryManager.BACKLOG,
										requirementsManager.getState()
												.getUser());
								if (story.getProject().equals(
										requirementsManager.getState()
												.getProject()))
									changed = true;
							} else
								try {
									UserStoryManager.updateIteration(story,
											UserStoryManager.BACKLOG);
									changed = true;
								} catch (InvalidUserStoryInputException e) {
									JOptionPane.showMessageDialog(null, e
											.getMessage(), "Error",
											JOptionPane.ERROR_MESSAGE);
								}

						if (changed) {
							RequirementsManagerView.this
									.updateUserStoriesTable();
							RequirementsManagerView.this.updateBacklogList();
						}
					}
				});

		this.updateBacklogList();
	}

	/**
	 * Updates the contents of the backlog list from the database
	 */
	private void updateBacklogList() {
		UserStory[] backlog = this.requirementsManager.getBacklog();
		this.backlogList.clear();

		if (backlog != null)
			for (UserStory story : backlog)
				this.backlogList.getModel().addElement(story);
	}

	/**
	 * Initializes the 'other requirements' list with a custom model and
	 * transfer handler
	 */
	private void initOtherList() {

		// otherList.setDragEnabled(true);
		// otherList.setTransfeHandler(new UserStoryListTransferHandler());
		this.otherList.setCellRenderer(new EntityListCellRenderer());
		this.updateOtherList();
	}

	/**
	 * initialize project members tree
	 */
	private void initProjectMembersTree() {
		EntityTreeNode<Project> root = new EntityTreeNode<Project>(null);
		userTree.setModel(new DefaultTreeModel(root));
		userTree.setRootVisible(true);
		userTree.setCellRenderer(new EntityTreeCellRenderer());
		updateProjectMembersTree();
	}

	/**
	 * Display editor when selection changes in project members tree
	 */
	private void onProjectMembersTreeSelectionChange(TreeSelectionEvent e) {

		if (userTree.getSelectionCount() == 0)
			this.requirementsManager.setEntityEditor(null);
		else if (userTree.getSelectionCount() == 1) {

			DefaultMutableTreeNode node = (DefaultMutableTreeNode) userTree
					.getLastSelectedPathComponent();
			if (node == null)
				return;

			Object nodeInfo = node.getUserObject();
			if (nodeInfo instanceof Entity && !(nodeInfo instanceof User)
					&& !(nodeInfo instanceof Project))
				this.requirementsManager
						.entitySelectionChange((Entity) nodeInfo);
			else
				this.requirementsManager.setEntityEditor(null);
		}
	}

	/**
	 * Updates the contents of the 'other requirements' list from the database
	 */
	private void updateOtherList() {
		// this should be Requirement[], but requirement is not supported fully
		// yet
		// Requirement[] requirements =
		// requirementManager.getOtherRequirements();
		Project curProj = this.requirementsManager.getState().getProject();
		this.otherList.clear();

		/*
		 * if ((curProj != null) &&
		 * Boolean.TRUE.equals((permissionsHash.get("canReadFeature")))) { for
		 * (Feature requirement : curProj.getFeatures()) {
		 * this.otherList.getModel().addElement(requirement); } }
		 */// permissions-related
		if (curProj != null)
			for (Feature requirement : curProj.getFeatures())
				this.otherList.getModel().addElement(requirement);
	}

	/**
     * 
     */
	private void initUserStoriesTable() {
		this.userStoriesTableModel = new UserStoryTableModel();
		TableRowSorter<UserStoryTableModel> sorter = new TableRowSorter<UserStoryTableModel>(
				this.userStoriesTableModel);
		this.mainTable.setRowSorter(sorter);
		this.mainTable.setModel(this.userStoriesTableModel);
		this.mainTable.setDragEnabled(true);
		this.mainTable.setDropMode(DropMode.INSERT_ROWS);

		Enumeration<TableColumn> cols = mainTable.getColumnModel().getColumns();
		while (cols.hasMoreElements()) {
			TableColumn tc = cols.nextElement();
			tc.setCellRenderer(new UserStoryTableCellRenderer());
		}

		this.mainTable
				.setTransferHandler(new UserStoryTableTransferHandler<UserStory>(
						UserStory.class) {

					@SuppressWarnings("unchecked")
					@Override
					public boolean canImport(
							TransferHandler.TransferSupport support) {

						if (!super.canImport(support))
							return false;

						List<Iteration> selectedIterations = RequirementsManagerView.this
								.getSelectedIterations();
						if (selectedIterations.size() != 1)
							return false;

						// no duplicate values
						ArrayList<UserStory> transferStories;
						try {
							transferStories = (ArrayList<UserStory>) support
									.getTransferable().getTransferData(
											this.supportedDataFlavor);
						} catch (UnsupportedFlavorException e) {
							return false;
						} catch (IOException e) {
							return false;
						}
						UserStoryTableModel model = (UserStoryTableModel) ((JTable) support
								.getComponent()).getModel();
						ArrayList<UserStory> tableStories = model.getAll();
						if (tableStories.containsAll(transferStories))
							return false;

						return true;
					}

					@Override
					public void postImport(JComponent component,
							List<UserStory> data) {
						List<Iteration> selectedIterations = RequirementsManagerView.this
								.getSelectedIterations();

						boolean changed = false;

						for (UserStory story : data)
							// From other project; copy it
							if (!story.getProject()
									.equals(
											requirementsManager.getState()
													.getProject())) {
								story = importUserStory(story,
										requirementsManager.getState()
												.getProject(),
										selectedIterations.get(0),
										requirementsManager.getState()
												.getUser());
								if (story.getProject().equals(
										requirementsManager.getState()
												.getProject()))
									changed = true;
							} else
								try {
									UserStoryManager.updateIteration(story,
											selectedIterations.get(0));
									changed = true;
								} catch (InvalidUserStoryInputException e) {
									JOptionPane.showMessageDialog(null, e
											.getMessage(), "Error",
											JOptionPane.ERROR_MESSAGE);
								}
						if (changed) {
							RequirementsManagerView.this.updateBacklogList();
							RequirementsManagerView.this
									.updateUserStoriesTable();
						}
					}
				});

		this.updateUserStoriesTable();

	}

	/**
	 * Updates the table containing all the user stories.
	 */
	private void updateUserStoriesTable() {

		List<Iteration> iterations = this.getSelectedIterations();
		this.userStoriesTableModel.clear();

		if (iterations.isEmpty())
			return;

		// Terrible, but it makes it work.
		// TODO figure out why we need this
		// non-thread-safe caching or something??
		Database.manager().flushAll();

		for (Iteration it : iterations) {
			UserStory[] stories = it.getUserStories();
			if (stories != null)
				for (UserStory story : stories)
					this.userStoriesTableModel.addUserStory(story);
		}
	}

	/**
	 * Updates the permission hash for the current user for the current selected
	 * project
	 */
	private void updatePermissions() {
		/*
		 * User currUser =
		 * requirementsManager.getWpiSuite().getState().getUser(); Project
		 * currProj = requirementsManager.getState().getProject();
		 * 
		 * 
		 * if (currProj != null){ permissionsHash =
		 * data.funcs.UserFunctions.getPermissions(currUser, currProj); }
		 */
	}

	/**
	 * Looks at iterationsList and gets all the Iterations that are selected.
	 * 
	 * @return All selected Iterations in iterationsList
	 */
	public List<Iteration> getSelectedIterations() {
		return iterationList.getSelectedEntities();
	}

	/**
	 * Looks at releaseList and gets all the Releases that are selected.
	 * 
	 * @return All selected Releases in releaseList
	 */
	public List<Release> getSelectedReleases() {
		return releaseList.getSelectedEntities();
	}

	/**
	 * Updates the tree containing all Users that are associated with the
	 * current iterations of the current project.
	 */
	@SuppressWarnings("unchecked")
	private void updateProjectMembersTree() {
		EntityTreeNode<Project> usersRoot = (EntityTreeNode<Project>) userTree
				.getModel().getRoot();
		usersRoot.removeAllChildren();

		Project curProj = this.requirementsManager.getState().getProject();
		List<Iteration> iterations = this.getSelectedIterations();
		// Feature[] features = curProj.getFeatures();

		if (curProj != null) {

			usersRoot
					.setUserObject(requirementsManager.getState().getProject());

			User[] users = curProj.getMembers();
			HashMap<Integer, EntityTreeNode<User>> mapOfNodes = new HashMap<Integer, EntityTreeNode<User>>();
			for (User u : users)
				mapOfNodes.put(u.getID(), new EntityTreeNode<User>(u));
			// Add user stories
			if(users.length>0) {
				if (!iterations.isEmpty())
					try {
						String query = "projectID = ? AND ownerID IN ("
								+ Database.questionMarks(users.length)
								+ ") AND iterationID IN ("
								+ Database.questionMarks(iterations.size()) + ")";
						ArrayList<Object> queryArgs = new ArrayList<Object>();
						queryArgs.add(curProj);
						queryArgs.addAll(Arrays.asList(users));
						queryArgs.addAll(iterations);
						for (UserStory userStory : Database.manager().find(
								UserStory.class, query, queryArgs.toArray()))
							mapOfNodes.get(userStory.getOwner().getID()).add(
									new EntityTreeNode<UserStory>(userStory));
					} catch (SQLException e) {
						this.popErrorMessage(e);
					}
				// Add features
				
				try {
					String query = "projectID = ? AND ownerID in ("
							+ Database.questionMarks(users.length) + ")";
					ArrayList<Object> queryArgs = new ArrayList<Object>();
					queryArgs.add(curProj);
					queryArgs.addAll(Arrays.asList(users));
					for (Feature feature : Database.manager().find(Feature.class,
							query, queryArgs.toArray()))
						mapOfNodes.get(feature.getOwner().getID()).add(
								new EntityTreeNode<Feature>(feature));
				} catch (SQLException e) {
					this.popErrorMessage(e);
				}
			
				// Add users
				for (User u : users)
					usersRoot.add(mapOfNodes.get(u.getID()));
			}
		}

		userTree.updateUI();
	}

	private void initPopupMenus() {
		iterationsPopupMenu = new EntityPopupMenu("Iteration",
				new MouseAdapter() {
					@Override
					public void mouseReleased(MouseEvent evt) {
						RequirementsManagerView.this
								.onIterationsListPopupMenuCreateItemClick(evt);
					}
				}, new MouseAdapter() {
					@Override
					public void mouseReleased(MouseEvent evt) {
						RequirementsManagerView.this
								.onIterationsListPopupMenuRemoveItemClick(evt);
					}
				});

		releasePopupMenu = new EntityPopupMenu("Release", new MouseAdapter() {
			@Override
			public void mouseReleased(MouseEvent evt) {
				RequirementsManagerView.this
						.onReleaseListPopupMenuCreateItemClick(evt);
			}
		}, new MouseAdapter() {
			@Override
			public void mouseReleased(MouseEvent evt) {
				RequirementsManagerView.this
						.onReleaseListPopupMenuRemoveItemClick(evt);
			}
		});
		backlogPopupMenu = new EntityPopupMenu("User Story",
				new MouseAdapter() {
					@Override
					public void mouseReleased(MouseEvent evt) {
						RequirementsManagerView.this
								.onBackLogListPopupMenuCreateItemClick(evt);
					}
				}, new MouseAdapter() {
					@Override
					public void mouseReleased(MouseEvent evt) {
						RequirementsManagerView.this
								.onBackLogListPopupMenuRemoveItemClick(evt);
					}
				});
		mainTablePopupMenu = new EntityPopupMenu("User Story",
				new MouseAdapter() {

					@Override
					public void mouseReleased(MouseEvent evt) {
						RequirementsManagerView.this
								.createMainTablePopupMenuItemReleased(evt);
					}
				}, new MouseAdapter() {
					@Override
					public void mouseReleased(MouseEvent evt) {
						RequirementsManagerView.this
								.removeMainTableRemoveMenuItemReleased(evt);
					}
				});
		otherRequirementsPopupMenu = new EntityPopupMenu("Feature",
				new MouseAdapter() {
					@Override
					public void mouseReleased(MouseEvent evt) {
						RequirementsManagerView.this
								.onOtherRequirementsListPopupMenuCreateItemClick(evt);
					}
				}, new MouseAdapter() {
					@Override
					public void mouseReleased(MouseEvent evt) {
						RequirementsManagerView.this
								.onOtherRequirementsListPopupMenuRemoveItemClick(evt);
					}
				});
	}

	@Override
	public void updateAllComponents() {
		this.updatePermissions();
		this.updateBacklogList();
		this.updateReleaseList();
		this.updateIterationsList();
		this.updateOtherList();
		this.updateProjectMembersTree();
		this.updateUserStoriesTable();
	}

	public ConfirmResult askYesNoCancelQuestion(String title, String message) {
		int result = JOptionPane.showConfirmDialog(null, message, title,
				JOptionPane.YES_NO_CANCEL_OPTION);
		switch (result) {
		case JOptionPane.NO_OPTION:
			return ConfirmResult.NO;
		case JOptionPane.YES_OPTION:
			return ConfirmResult.YES;
		default:
			return ConfirmResult.CANCEL;
		}
	}

	// ////////////////////////////////////////////
	// //
	// Avoid editing below this point! //
	// //
	// ////////////////////////////////////////////

	@SuppressWarnings( { "unchecked" })
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		mainSplitPane = new javax.swing.JSplitPane();
		sideSplitPaneTop = new javax.swing.JSplitPane();
		sideSplitPaneBottom = new javax.swing.JSplitPane();
		backlogPanel = new javax.swing.JPanel();
		backlogScrollPane = new javax.swing.JScrollPane();
		backlogList = new EntityList<UserStory>();
		otherPanel = new javax.swing.JPanel();
		otherScrollPane = new javax.swing.JScrollPane();
		otherList = new EntityList<Feature>();
		userPanel = new javax.swing.JPanel();
		userScrollPane = new javax.swing.JScrollPane();
		userTree = new javax.swing.JTree();
		verticalSplitPane = new javax.swing.JSplitPane();
		mainScrollPane = new javax.swing.JScrollPane();
		mainTable = new javax.swing.JTable();
		editorPanel = new javax.swing.JPanel();
		allIterationsButton = new javax.swing.JButton();
		noIterationsButton = new javax.swing.JButton();
		timelineScrollPane = new javax.swing.JScrollPane();
		timelinePanel = new javax.swing.JPanel();
		iterationScrollPane = new javax.swing.JScrollPane();
		releaseList = new EntityList<Release>();
		iterationScrollPane1 = new javax.swing.JScrollPane();
		iterationList = new EntityList<Iteration>();

		setPreferredSize(new java.awt.Dimension(900, 600));
		setRequestFocusEnabled(false);

		mainSplitPane.setDividerLocation(650);
		mainSplitPane.setResizeWeight(0.8);
		mainSplitPane.setOneTouchExpandable(true);

		sideSplitPaneTop.setDividerLocation(170);
		sideSplitPaneTop.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		sideSplitPaneTop.setResizeWeight(0.33);
		sideSplitPaneTop.setOneTouchExpandable(true);

		sideSplitPaneBottom.setDividerLocation(170);
		sideSplitPaneBottom
				.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		sideSplitPaneBottom.setResizeWeight(0.5);
		sideSplitPaneBottom.setOneTouchExpandable(true);

		backlogPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Backlog"));

		backlogScrollPane.setViewportView(backlogList);

		sideSplitPaneBottom.setTopComponent(backlogPanel);

		javax.swing.GroupLayout backlogPanelLayout = new javax.swing.GroupLayout(
				backlogPanel);
		backlogPanelLayout.setHorizontalGroup(backlogPanelLayout
				.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(backlogScrollPane, GroupLayout.DEFAULT_SIZE, 22,
						Short.MAX_VALUE));
		backlogPanelLayout.setVerticalGroup(backlogPanelLayout
				.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(backlogScrollPane, GroupLayout.DEFAULT_SIZE, 22,
						Short.MAX_VALUE));
		backlogPanel.setLayout(backlogPanelLayout);

		otherPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Other Requirements"));

		otherScrollPane.setViewportView(otherList);

		sideSplitPaneBottom.setRightComponent(otherPanel);

		javax.swing.GroupLayout otherPanelLayout = new javax.swing.GroupLayout(
				otherPanel);
		otherPanelLayout.setHorizontalGroup(otherPanelLayout
				.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(otherScrollPane, GroupLayout.DEFAULT_SIZE, 22,
						Short.MAX_VALUE));
		otherPanelLayout.setVerticalGroup(otherPanelLayout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addComponent(otherScrollPane,
				GroupLayout.DEFAULT_SIZE, 22, Short.MAX_VALUE));
		otherPanel.setLayout(otherPanelLayout);

		sideSplitPaneTop.setBottomComponent(sideSplitPaneBottom);

		userPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Users"));

		javax.swing.tree.DefaultMutableTreeNode treeNode1 = new javax.swing.tree.DefaultMutableTreeNode(
				"root");
		javax.swing.tree.DefaultMutableTreeNode treeNode2 = new javax.swing.tree.DefaultMutableTreeNode(
				"User 1");
		javax.swing.tree.DefaultMutableTreeNode treeNode3 = new javax.swing.tree.DefaultMutableTreeNode(
				"Task 1");
		treeNode2.add(treeNode3);
		treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Task 2");
		treeNode2.add(treeNode3);
		treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Task 3");
		treeNode2.add(treeNode3);
		treeNode1.add(treeNode2);
		treeNode2 = new javax.swing.tree.DefaultMutableTreeNode("User 2");
		treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Task 1");
		treeNode2.add(treeNode3);
		treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Task 2");
		treeNode2.add(treeNode3);
		treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Task 3");
		treeNode2.add(treeNode3);
		treeNode1.add(treeNode2);
		treeNode2 = new javax.swing.tree.DefaultMutableTreeNode("User 3");
		treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Task 1");
		treeNode2.add(treeNode3);
		treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Task 2");
		treeNode2.add(treeNode3);
		treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Task 3");
		treeNode2.add(treeNode3);
		treeNode1.add(treeNode2);
		userTree.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
		userTree.setRootVisible(false);
		userScrollPane.setViewportView(userTree);

		sideSplitPaneTop.setLeftComponent(userPanel);

		javax.swing.GroupLayout userPanelLayout = new javax.swing.GroupLayout(
				userPanel);
		userPanelLayout.setHorizontalGroup(userPanelLayout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addComponent(userScrollPane,
				GroupLayout.Alignment.TRAILING, GroupLayout.DEFAULT_SIZE, 22,
				Short.MAX_VALUE));
		userPanelLayout.setVerticalGroup(userPanelLayout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addComponent(userScrollPane,
				GroupLayout.Alignment.TRAILING, GroupLayout.DEFAULT_SIZE, 22,
				Short.MAX_VALUE));
		userPanel.setLayout(userPanelLayout);

		mainSplitPane.setRightComponent(sideSplitPaneTop);

		verticalSplitPane.setDividerLocation(650);
		verticalSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		verticalSplitPane.setResizeWeight(1.0);
		verticalSplitPane.setOneTouchExpandable(true);

		mainTable.setAutoCreateRowSorter(true);
		mainTable.setModel(new javax.swing.table.DefaultTableModel(
				new Object[][] {
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null },
						{ null, null, null, null, null, null, null, null } },
				new String[] { "Iteration", "Title", "Status", "Owner",
						"Creator", "Category", "Priority", "Units" }) {
			Class[] types = new Class[] { java.lang.Integer.class,
					java.lang.String.class, java.lang.String.class,
					java.lang.String.class, java.lang.String.class,
					java.lang.String.class, java.lang.Integer.class,
					java.lang.Integer.class };

			@Override
			public Class getColumnClass(int columnIndex) {
				return types[columnIndex];
			}
		});
		mainTable.setFillsViewportHeight(true);
		mainTable.setShowHorizontalLines(false);
		mainScrollPane.setViewportView(mainTable);
		mainTable.getColumnModel().getColumn(0).setPreferredWidth(60);
		mainTable.getColumnModel().getColumn(1).setPreferredWidth(180);
		mainTable.getColumnModel().getColumn(2).setPreferredWidth(90);
		mainTable.getColumnModel().getColumn(3).setPreferredWidth(120);
		mainTable.getColumnModel().getColumn(4).setPreferredWidth(120);
		mainTable.getColumnModel().getColumn(5).setPreferredWidth(90);
		mainTable.getColumnModel().getColumn(6).setPreferredWidth(60);
		mainTable.getColumnModel().getColumn(7).setPreferredWidth(60);

		verticalSplitPane.setLeftComponent(mainScrollPane);

		verticalSplitPane.setRightComponent(editorPanel);

		javax.swing.GroupLayout editorPanelLayout = new javax.swing.GroupLayout(
				editorPanel);
		editorPanelLayout.setHorizontalGroup(editorPanelLayout
				.createParallelGroup(GroupLayout.Alignment.LEADING).addGap(0,
						0, Short.MAX_VALUE));
		editorPanelLayout.setVerticalGroup(editorPanelLayout
				.createParallelGroup(GroupLayout.Alignment.LEADING).addGap(0,
						0, Short.MAX_VALUE));
		editorPanel.setLayout(editorPanelLayout);

		mainSplitPane.setLeftComponent(verticalSplitPane);

		allIterationsButton.setText("All Iterations");

		noIterationsButton.setText("No Iterations");

		timelineScrollPane.setBorder(null);
		timelineScrollPane
				.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		timelineScrollPane
				.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

		iterationScrollPane
				.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

		releaseList.setModel(new ReleaseListModel());
		releaseList.setDragEnabled(true);
		releaseList.setLayoutOrientation(javax.swing.JList.HORIZONTAL_WRAP);
		releaseList.setVisibleRowCount(1);
		releaseList.setFixedCellWidth(timelineCellWidth);
		iterationScrollPane.setViewportView(releaseList);

		iterationScrollPane1
				.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

		iterationList.setModel(new IterationsListModel());
		iterationList.setDragEnabled(true);
		iterationList.setLayoutOrientation(javax.swing.JList.HORIZONTAL_WRAP);
		iterationList.setVisibleRowCount(1);
		iterationList.setFixedCellWidth(timelineCellWidth);
		iterationScrollPane1.setViewportView(iterationList);

		timelineScrollPane
				.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);

		javax.swing.GroupLayout timelinePanelLayout = new javax.swing.GroupLayout(
				timelinePanel);
		timelinePanel.setLayout(timelinePanelLayout);
		timelinePanelLayout.setHorizontalGroup(timelinePanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(releaseList,
						javax.swing.GroupLayout.Alignment.TRAILING,
						javax.swing.GroupLayout.DEFAULT_SIZE, 720,
						Short.MAX_VALUE).addComponent(iterationList,
						javax.swing.GroupLayout.Alignment.TRAILING,
						javax.swing.GroupLayout.DEFAULT_SIZE,
						javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE));
		timelinePanelLayout
				.setVerticalGroup(timelinePanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								timelinePanelLayout
										.createSequentialGroup()
										.addComponent(releaseList, 16,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.DEFAULT_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(iterationList, 16,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.DEFAULT_SIZE)
										.addContainerGap()));

		timelineScrollPane.setViewportView(timelinePanel);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		layout.setHorizontalGroup(layout.createParallelGroup(
				GroupLayout.Alignment.TRAILING)
				.addGroup(
						layout.createSequentialGroup().addComponent(
								timelineScrollPane, GroupLayout.DEFAULT_SIZE,
								676, Short.MAX_VALUE).addPreferredGap(
								LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(allIterationsButton,
										GroupLayout.PREFERRED_SIZE, 102,
										GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(noIterationsButton,
										GroupLayout.PREFERRED_SIZE, 98,
										GroupLayout.PREFERRED_SIZE)
								.addContainerGap()).addComponent(mainSplitPane,
						GroupLayout.DEFAULT_SIZE, 900, Short.MAX_VALUE));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addGroup(
												layout
														.createParallelGroup(
																GroupLayout.Alignment.LEADING)
														.addGroup(
																layout
																		.createSequentialGroup()
																		.addGap(
																				33,
																				33,
																				33)
																		.addGroup(
																				layout
																						.createParallelGroup(
																								GroupLayout.Alignment.BASELINE)
																						.addComponent(
																								allIterationsButton)
																						.addComponent(
																								noIterationsButton)))
														.addGroup(
																layout
																		.createSequentialGroup()
																		.addContainerGap()
																		.addComponent(
																				timelineScrollPane,
																				GroupLayout.PREFERRED_SIZE,
																				63,
																				GroupLayout.PREFERRED_SIZE)))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(mainSplitPane,
												GroupLayout.DEFAULT_SIZE, 519,
												Short.MAX_VALUE)));
		this.setLayout(layout);
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton allIterationsButton;
	private EntityList<UserStory> backlogList;
	private javax.swing.JPanel backlogPanel;
	private javax.swing.JScrollPane backlogScrollPane;
	private javax.swing.JPanel editorPanel;
	private javax.swing.JScrollPane iterationScrollPane;
	private javax.swing.JScrollPane iterationScrollPane1;
	private EntityList<Iteration> iterationList;
	private javax.swing.JScrollPane mainScrollPane;
	private javax.swing.JSplitPane mainSplitPane;
	private javax.swing.JTable mainTable;
	private javax.swing.JButton noIterationsButton;
	private EntityList<Feature> otherList;
	private javax.swing.JPanel otherPanel;
	private javax.swing.JScrollPane otherScrollPane;
	private EntityList<Release> releaseList;
	private javax.swing.JSplitPane sideSplitPaneBottom;
	private javax.swing.JSplitPane sideSplitPaneTop;
	private javax.swing.JPanel timelinePanel;
	private javax.swing.JScrollPane timelineScrollPane;
	private javax.swing.JPanel userPanel;
	private javax.swing.JScrollPane userScrollPane;
	private javax.swing.JTree userTree;
	private javax.swing.JSplitPane verticalSplitPane;
	// End of variables declaration//GEN-END:variables
}
