/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design, API, and implementation
 *    
 *******************************************************************************/

package edu.wpi.wpisuite.module.rm;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.DefaultTableModel;

import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;

/**
 * Models a table which is represented by and stores UserStories, but actually
 * displays the UserStory names and some information.
 */
public class UserStoryTableModel extends DefaultTableModel {

	private static final long serialVersionUID = 8757495639558777914L;
	private String[] columnNames = { "Iteration", "Title", "Status", "Owner",
			"Creator", "Est. Units", "Actual Units" };
	// private Object[][] data;
	public ArrayList<UserStory> stories = new ArrayList<UserStory>();

	public UserStoryTableModel() {
	}

	public UserStoryTableModel(List<UserStory> l) {
		this.stories.addAll(l);
	}

	@Override
	public int getColumnCount() {
		return this.columnNames.length;
	}

	@Override
	public int getRowCount() {
		if (this.stories == null)
			return 0;
		return this.stories.size();
	}

	@Override
	public String getColumnName(int col) {
		return this.columnNames[col];
	}

	/**
	 * Disable editing of cell values
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int mColIndex) {
		return false;
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		switch (columnIndex) {
		// Hacked solution for generic drag and drop
		case -1:
			return UserStory.class;
		case 0:
			return String.class;
		case 1:
			return String.class;
		case 2:
			return String.class;
		case 3:
			return String.class;
		case 4:
			return String.class;
		case 5:
			return Integer.class;
		case 6:
			return Integer.class;
		default:
			return String.class;
		}
	}

	/**
	 * Returns the value at the given index.
	 */
	@Override
	public Object getValueAt(int row, int col) {

		UserStory story = this.stories.get(row);
		String tempString;
		User tempUser;
		Iteration tempIteration;

		switch (col) {
		// Hacked solution for generic drag and drop
		case -1:
			return story;
		case 0:
			tempIteration = story.getIteration();
			if (tempIteration != null)
				tempString = tempIteration.getName();
			else
				return "-";
			if (tempString != null)
				return tempString;
			else
				return "-";
		case 1:
			tempString = story.getTitle();
			if (tempString != null)
				return tempString;
			else
				return "-";
		case 2:
			tempString = (story.getStatus() == null ? null : story.getStatus()
					.toString());
			if (tempString != null)
				return tempString;
			else
				return "-";
		case 3:
			tempUser = story.getOwner();
			if (tempUser != null)
				tempString = tempUser.getUsername();
			else
				return "-";
			if (tempString != null)
				return tempString;
			else
				return "-";
		case 4:
			tempUser = story.getAuthor();
			if (tempUser != null)
				tempString = tempUser.getUsername();
			else
				return "-";
			if (tempString != null)
				return tempString;
			else
				return "-";
		case 5:
			return story.getEstimatedUnits();
		case 6:
			return story.getActualUnits();
		default:
			return "";
		}
	}

	/**
	 * Returns the value at the given index.
	 */
	public Object getObjectAt(int row, int col) {

		UserStory story = this.stories.get(row);

		switch (col) {
		case 0:
			return story.getIteration();
		case 1:
			return story.getTitle();
		case 2:
			return story.getStatus();
		case 3:
			return story.getOwner();
		case 4:
			return story.getAuthor();
		case 5:
			return story.getEstimatedUnits();
		case 6:
			return story.getActualUnits();
		default:
			return null;
		}
	}

	// story.getEstimatedUnits();
	public void addUserStory(UserStory story) {
		if (story == null)
			return;

		this.stories.add(story);
		this.fireTableDataChanged();
	}

	public void addUserStoryList(List<UserStory> stories) {
		this.stories.addAll(stories);
		this.fireTableDataChanged();
	}

	public void clear() {
		this.stories.clear();
		this.fireTableDataChanged();
	}

	public UserStory getUserStoryAt(int row) {
		try {
			return this.stories.get(row);
		} catch (IndexOutOfBoundsException e) {
			return null;
		}
	}

	public ArrayList<UserStory> getAll() {
		return this.stories;
	}

	public UserStory removeUserStoryAt(int row) {
		UserStory story = this.stories.remove(row);
		this.fireTableDataChanged();
		return story;
	}
}
