/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Unknown
 *    
 *******************************************************************************/

package edu.wpi.wpisuite.module.systemadmin;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


import org.apache.log4j.Logger;


import edu.wpi.wpisuite.components.EntityList;
import edu.wpi.wpisuite.components.EntityListCellRenderer;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.EditorPanelView;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.funcs.LoginFunctions;
import edu.wpi.wpisuite.module.ModuleView;

/**
 * View for the System Administration class
 */
@SuppressWarnings("serial")
public class SystemAdminView extends ModuleView {

	Logger logger = Logger.getLogger(this.getClass());

	SystemAdmin systemAdmin;

	public SystemAdminView(SystemAdmin systemAdmin) {
		super(systemAdmin,
				new ImageIcon("resources/OtherIcons/SystemAdmin.PNG"));
		this.systemAdmin = systemAdmin;
		this.initComponents();

		this.usersList.setCellRenderer(new EntityListCellRenderer());

		this.initListeners();
	}

	public void initListeners() {
		this.usersList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent arg0) {
				if (!arg0.getValueIsAdjusting())
					usersListSelectionChanged();
			}
		});

		this.maximumLoginsSpinner.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent arg0) {
				setMaxLogins();
			}
		});

		this.newUserButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				addNewUser();
			}
		});

		emailCancelButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				resetEmailFields();
			}
		});

		emailApplyButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				saveEmailFields();
			}
		});

		emailProtocolTextField.getDocument().addDocumentListener(
				emailButtonEnablerListener);
		emailServerTextField.getDocument().addDocumentListener(
				emailButtonEnablerListener);
		emailFromTextField.getDocument().addDocumentListener(
				emailButtonEnablerListener);
		emailUsernameTextField.getDocument().addDocumentListener(
				emailButtonEnablerListener);
		emailPasswordField.getDocument().addDocumentListener(
				emailButtonEnablerListener);
	}

	@Override
	public void initAddMenuItems() {
		JMenuItem userAdder = new JMenuItem("User", KeyEvent.VK_U);
		userAdder.setIcon(icons.getUserIcon(IconType.CREATE));

		ActionListener userAdderListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				addNewUser();
			}
		};
		userAdder.addActionListener(userAdderListener);

		addMenuItems.add(userAdder);
	}

	public void setMaxLogins() {
		LoginFunctions.setMaxFailedLogins((Integer) this.maximumLoginsSpinner
				.getModel().getValue());
	}

	public void addNewUser() {
		systemAdmin.addNewUser();
	}

	public void updateMaxLogins() {
		this.maximumLoginsSpinner.setValue(LoginFunctions.getMaxFailedLogins());
	}

	public void saveEmailFields() {
		systemAdmin.saveEmailSettings(emailProtocolTextField.getText(),
				emailServerTextField.getText(), emailFromTextField.getText(),
				emailUsernameTextField.getText(), new String(emailPasswordField
						.getPassword()));
		resetEmailFields();
	}

	public void setEmailFields(String protocol, String server, String from,
			String username, String password) {
		emailProtocolTextField.setText(protocol);
		emailServerTextField.setText(server);
		emailFromTextField.setText(from);
		emailUsernameTextField.setText(username);
		emailPasswordField.setText(password);
		emailCancelButton.setEnabled(false);
		emailApplyButton.setEnabled(false);
	}

	private DocumentListener emailButtonEnablerListener = new DocumentListener() {
		private void go() {
			emailCancelButton.setEnabled(true);
			emailApplyButton.setEnabled(true);
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			go();
		}

		@Override
		public void insertUpdate(DocumentEvent e) {
			go();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			go();
		}
	};

	public void resetEmailFields() {
		systemAdmin.resetEmailFields();
	}

	public void usersListSelectionChanged() {
		systemAdmin.selectUser(usersList.getSelectedEntity());
	}

	@Override
	public void updateAllComponents() {
		updateUsersList();
		updateMaxLogins();
	}

	public void updateUsersList() {
		usersList.getModel().clear();
		for (User u : systemAdmin.getUsers())
			usersList.getModel().addElement(u);
	}

	public void setEditorView(EditorPanelView v) {
		this.verticalSplitPane.setBottomComponent(v);
	}

	private void initComponents() {

		primarySplitPane = new javax.swing.JSplitPane();
		usersPanel = new javax.swing.JPanel();
		usersScrollPane = new javax.swing.JScrollPane();
		usersList = new EntityList<User>();
		newUserButton = new javax.swing.JButton();
		verticalSplitPane = new javax.swing.JSplitPane();
		toolsPanel = new javax.swing.JPanel();

		setPreferredSize(new java.awt.Dimension(900, 600));

		primarySplitPane.setDividerLocation(250);
		primarySplitPane.setOneTouchExpandable(true);

		usersPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("All Users"));

		usersList.setVisibleRowCount(25);
		usersScrollPane.setViewportView(usersList);

		newUserButton.setText("New User");

		javax.swing.GroupLayout usersPanelLayout = new javax.swing.GroupLayout(
				usersPanel);
		usersPanel.setLayout(usersPanelLayout);
		usersPanelLayout.setHorizontalGroup(usersPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(usersScrollPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 234,
						Short.MAX_VALUE).addGroup(
						usersPanelLayout.createSequentialGroup().addComponent(
								newUserButton).addContainerGap(155,
								Short.MAX_VALUE)));
		usersPanelLayout.setVerticalGroup(usersPanelLayout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				javax.swing.GroupLayout.Alignment.TRAILING,
				usersPanelLayout.createSequentialGroup().addComponent(
						usersScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE,
						565, Short.MAX_VALUE).addPreferredGap(
						javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(newUserButton)));

		primarySplitPane.setLeftComponent(usersPanel);

		verticalSplitPane.setDividerLocation(600);
		verticalSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		verticalSplitPane.setResizeWeight(1.0);
		verticalSplitPane.setOneTouchExpandable(true);

		// /////////////////

		loginSettingsLabel = new javax.swing.JLabel();
		loginSettingsSeparator = new javax.swing.JSeparator();
		maximumLoginsLabel = new javax.swing.JLabel();
		maximumLoginsSpinner = new javax.swing.JSpinner();
		systemEmailSettingsLabel = new javax.swing.JLabel();
		systemEmailSettingsSeparator = new javax.swing.JSeparator();
		emailProtocolLabel = new javax.swing.JLabel();
		emailServerLabel = new javax.swing.JLabel();
		emailFromLabel = new javax.swing.JLabel();
		emailUsernameLabel = new javax.swing.JLabel();
		emailPasswordLabel = new javax.swing.JLabel();
		emailProtocolTextField = new javax.swing.JTextField();
		emailServerTextField = new javax.swing.JTextField();
		emailFromTextField = new javax.swing.JTextField();
		emailUsernameTextField = new javax.swing.JTextField();
		emailPasswordField = new javax.swing.JPasswordField();
		emailApplyButton = new javax.swing.JButton();
		emailCancelButton = new javax.swing.JButton();

		loginSettingsLabel.setFont(new java.awt.Font("Tahoma", 0, 16)); // NOI18N
		loginSettingsLabel.setText("Login Settings");

		maximumLoginsLabel.setText("Maximum Login Attempts (0 for unlimited)");

		systemEmailSettingsLabel.setFont(new java.awt.Font("Tahoma", 0, 16)); // NOI18N
		systemEmailSettingsLabel.setText("Outgoing Email Settings");

		emailProtocolLabel.setText("Protocol");
		emailProtocolLabel.setToolTipText("For example, smtp");
		emailProtocolTextField.setToolTipText("For example, smtp");

		emailServerLabel.setText("Server");
		emailServerLabel.setToolTipText("For example, smtp.yourdomain.com");
		emailServerTextField.setToolTipText("For example, smtp.yourdomain.com");

		emailFromLabel.setText("Sender name");
		emailFromLabel.setToolTipText("For example, WPI Suite Administrator");
		emailFromTextField
				.setToolTipText("For example, WPI Suite Administrator");

		emailUsernameLabel.setText("Username");
		emailUsernameLabel
				.setToolTipText("For example, wpisuiteadmin@yourdomain.com");
		emailUsernameTextField
				.setToolTipText("For example, wpisuiteadmin@yourdomain.com");

		emailPasswordLabel.setText("Password");
		emailPasswordLabel.setToolTipText("For example, swordfish");
		emailPasswordField.setToolTipText("For example, swordfish");

		emailApplyButton.setText("Apply");

		emailCancelButton.setText("Cancel");

		javax.swing.GroupLayout toolsPanelLayout = new javax.swing.GroupLayout(
				toolsPanel);
		toolsPanel.setLayout(toolsPanelLayout);
		toolsPanelLayout
				.setHorizontalGroup(toolsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								toolsPanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												toolsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																systemEmailSettingsSeparator,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																541,
																Short.MAX_VALUE)
														.addGroup(
																toolsPanelLayout
																		.createSequentialGroup()
																		.addGap(
																				10,
																				10,
																				10)
																		.addComponent(
																				maximumLoginsLabel)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
																		.addComponent(
																				maximumLoginsSpinner,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				323,
																				Short.MAX_VALUE))
														.addComponent(
																loginSettingsSeparator,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																541,
																Short.MAX_VALUE)
														.addComponent(
																loginSettingsLabel,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																541,
																Short.MAX_VALUE)
														.addComponent(
																systemEmailSettingsLabel)
														.addGroup(
																toolsPanelLayout
																		.createSequentialGroup()
																		.addGap(
																				10,
																				10,
																				10)
																		.addGroup(
																				toolsPanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addGroup(
																								toolsPanelLayout
																										.createSequentialGroup()
																										.addGroup(
																												toolsPanelLayout
																														.createParallelGroup(
																																javax.swing.GroupLayout.Alignment.LEADING,
																																false)
																														.addComponent(
																																emailServerLabel,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																Short.MAX_VALUE)
																														.addComponent(
																																emailFromLabel,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																Short.MAX_VALUE)
																														.addComponent(
																																emailPasswordLabel,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																Short.MAX_VALUE)
																														.addComponent(
																																emailUsernameLabel,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																Short.MAX_VALUE)
																														.addComponent(
																																emailProtocolLabel,
																																javax.swing.GroupLayout.Alignment.TRAILING,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																Short.MAX_VALUE))
																										.addPreferredGap(
																												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																										.addGroup(
																												toolsPanelLayout
																														.createParallelGroup(
																																javax.swing.GroupLayout.Alignment.LEADING)
																														.addComponent(
																																emailServerTextField,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																479,
																																Short.MAX_VALUE)
																														.addComponent(
																																emailFromTextField,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																479,
																																Short.MAX_VALUE)
																														.addComponent(
																																emailUsernameTextField,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																479,
																																Short.MAX_VALUE)
																														.addComponent(
																																emailPasswordField,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																479,
																																Short.MAX_VALUE)
																														.addComponent(
																																emailProtocolTextField,
																																javax.swing.GroupLayout.DEFAULT_SIZE,
																																479,
																																Short.MAX_VALUE)))
																						.addGroup(
																								toolsPanelLayout
																										.createSequentialGroup()
																										.addComponent(
																												emailApplyButton)
																										.addPreferredGap(
																												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																										.addComponent(
																												emailCancelButton)))))
										.addContainerGap()));
		toolsPanelLayout
				.setVerticalGroup(toolsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								toolsPanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(loginSettingsLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												loginSettingsSeparator,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												toolsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																maximumLoginsLabel)
														.addComponent(
																maximumLoginsSpinner,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addGap(18, 18, 18)
										.addComponent(systemEmailSettingsLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												systemEmailSettingsSeparator,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												toolsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																emailProtocolLabel)
														.addComponent(
																emailProtocolTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												toolsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																emailServerLabel)
														.addComponent(
																emailServerTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												toolsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																emailFromLabel)
														.addComponent(
																emailFromTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												toolsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																emailUsernameLabel)
														.addComponent(
																emailUsernameTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												toolsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																emailPasswordLabel)
														.addComponent(
																emailPasswordField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												toolsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																emailApplyButton)
														.addComponent(
																emailCancelButton))
										.addContainerGap(
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));

		// /////////////////

		verticalSplitPane.setTopComponent(toolsPanel);

		primarySplitPane.setRightComponent(verticalSplitPane);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				primarySplitPane, javax.swing.GroupLayout.DEFAULT_SIZE, 900,
				Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				primarySplitPane, javax.swing.GroupLayout.Alignment.TRAILING,
				javax.swing.GroupLayout.DEFAULT_SIZE, 623, Short.MAX_VALUE));
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JButton emailApplyButton;
	private javax.swing.JButton emailCancelButton;
	private javax.swing.JPasswordField emailPasswordField;
	private javax.swing.JLabel emailPasswordLabel;
	private javax.swing.JLabel emailFromLabel;
	private javax.swing.JTextField emailFromTextField;
	private javax.swing.JLabel emailProtocolLabel;
	private javax.swing.JTextField emailProtocolTextField;
	private javax.swing.JLabel emailServerLabel;
	private javax.swing.JTextField emailServerTextField;
	private javax.swing.JLabel emailUsernameLabel;
	private javax.swing.JTextField emailUsernameTextField;
	private javax.swing.JLabel loginSettingsLabel;
	private javax.swing.JSeparator loginSettingsSeparator;
	private javax.swing.JLabel maximumLoginsLabel;
	private javax.swing.JSpinner maximumLoginsSpinner;
	private javax.swing.JLabel systemEmailSettingsLabel;
	private javax.swing.JSeparator systemEmailSettingsSeparator;

	private javax.swing.JButton newUserButton;
	private javax.swing.JSplitPane primarySplitPane;
	private javax.swing.JPanel toolsPanel;
	private EntityList<User> usersList;
	private javax.swing.JPanel usersPanel;
	private javax.swing.JScrollPane usersScrollPane;
	private javax.swing.JSplitPane verticalSplitPane;
	// End of variables declaration

}
