/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability;

import edu.wpi.wpisuite.data.TraceabilityMatrix;

/**
 * Interfaces between the traceability controller and the matrix chooser view -
 * allows for users to choose a matrix and possibly tell the application that
 * they'd like to modify it, delete it, or make a new one
 */
public class MatrixChooserController implements TraceabilityStepController {
	private TraceabilityController controller;
	private MatrixChooserView view;

	public MatrixChooserController(TraceabilityController controller,
			MatrixDisplayController displayController) {
		this.controller = controller;
		this.view = new MatrixChooserView(this, displayController.getView());
		update();
	}

	public void matrixSelectionChange(TraceabilityMatrix selectedEntity) {
		controller.setActiveTraceabilityMatrix(selectedEntity);
	}

	public void editSelected() {
		controller.editActiveMatrix();
	}

	public void deleteSelected() {
		controller.deleteActiveMatrix();
	}

	public void newSelected() {
		controller.newActiveMatrix();
	}

	public void selectMatrix(TraceabilityMatrix m) {
		view.setSelectedMatrix(m);
	}

	@Override
	public void update() {
		view.setMatrixOptions(controller.getMatrixOptions());
		selectMatrix(controller.getActiveTraceabilityMatrix());
	}

	@Override
	public MatrixChooserView getView() {
		return view;
	}
}
