/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.ScrollPaneConstants;


import edu.wpi.wpisuite.components.EntityComboBox;
import edu.wpi.wpisuite.components.EntityIconSet;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.data.TraceabilityMatrix;

/**
 * Defines the panel that allows the user to choose between and view matrices
 * 
 * The viewing is outsourced to MatrixDisplay stuff.
 */
@SuppressWarnings("serial")
public class MatrixChooserView extends JPanel {

	private MatrixChooserController controller;
	private EntityIconSet icons = new EntityIconSet();

	public MatrixChooserView(MatrixChooserController controller,
			MatrixDisplayView content) {
		this.controller = controller;

		initComponents();
		setContentPane(content);

		headerLabel.setIcon(icons.getTraceabilityMatrixIcon(IconType.DEFAULT));

		matrixComboBox.addActionListener(matrixListener);

		editButton.addActionListener(editListener);

		deleteButton.addActionListener(deleteListener);

		newButton.addActionListener(newListener);
	}

	private ActionListener matrixListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			controller
					.matrixSelectionChange(matrixComboBox.getSelectedEntity());
		}
	};

	private ActionListener editListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			controller.editSelected();
		}
	};

	private ActionListener deleteListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			controller.deleteSelected();
		}
	};

	private ActionListener newListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			controller.newSelected();
		}
	};

	public void setMatrixOptions(TraceabilityMatrix[] options) {
		matrixComboBox.removeActionListener(matrixListener);

		matrixComboBox.getModel().reset(options);

		matrixComboBox.addActionListener(matrixListener);
	}

	public TraceabilityMatrix getSelectedMatrix() {
		return matrixComboBox.getSelectedEntity();
	}

	/**
	 * Does not activate listeners
	 */
	public void setSelectedMatrix(TraceabilityMatrix matrix) {
		matrixComboBox.removeActionListener(matrixListener);

		matrixComboBox.setSelectedEntity(matrix);

		matrixComboBox.addActionListener(matrixListener);
	}

	public void setContentPane(Component component) {
		contentScrollPane.setViewportView(component);
	}

	private void initComponents() {

		headerLabel = new javax.swing.JLabel();
		matrixComboBox = new EntityComboBox<TraceabilityMatrix>();
		deleteButton = new javax.swing.JButton();
		newButton = new javax.swing.JButton();
		editButton = new javax.swing.JButton();
		contentScrollPane = new javax.swing.JScrollPane();

		headerLabel.setFont(new java.awt.Font("Tahoma", 0, 16)); // NOI18N
		headerLabel.setText("Traceability Matrix:");

		matrixComboBox.setMinimumSize(new java.awt.Dimension(50, 18));

		deleteButton.setText("Delete");

		newButton.setText("New...");

		editButton.setText("Edit...");

		contentScrollPane.setBorder(null);
		contentScrollPane
				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		contentScrollPane
				.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout
				.setHorizontalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(headerLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(matrixComboBox, 0, 315,
												Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(editButton)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(deleteButton)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(newButton)
										.addContainerGap()).addComponent(
								contentScrollPane,
								javax.swing.GroupLayout.Alignment.TRAILING,
								javax.swing.GroupLayout.DEFAULT_SIZE, 689,
								Short.MAX_VALUE));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																headerLabel)
														.addComponent(
																matrixComboBox,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(newButton)
														.addComponent(
																deleteButton)
														.addComponent(
																editButton))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												contentScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												442, Short.MAX_VALUE)));
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JScrollPane contentScrollPane;
	private javax.swing.JButton deleteButton;
	private javax.swing.JButton editButton;
	private javax.swing.JLabel headerLabel;
	private EntityComboBox<TraceabilityMatrix> matrixComboBox;
	private javax.swing.JButton newButton;
	// End of variables declaration

}
