/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design, API, and implementation
 *    Sabrina Varanelli - Initial design and implementation
 *    
 *******************************************************************************/
package edu.wpi.wpisuite.module.traceability;

import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JTable;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;



import edu.wpi.wpisuite.components.editor.EditorPanelView;
import edu.wpi.wpisuite.data.Requirement;
import edu.wpi.wpisuite.data.TraceabilityAssociation;
import edu.wpi.wpisuite.module.traceability.components.TraceMatrixTableModel;
import edu.wpi.wpisuite.module.traceability.components.TraceTableCellRenderer;

/**
 * View for the matrix displayer
 */
@SuppressWarnings("serial")
public class MatrixDisplayView extends javax.swing.JPanel {

	// Non-generative variables - may modify
	private ImageIcon arrowIcon;
	private MatrixDisplayController controller;
	private TraceabilityController traceability;
	private TraceMatrixTableModel matrixModel;

	// End non-generative variables

	public void updateAllComponents() {
		this.updateMatrix();
	}

	/** Creates new form TM_Main */
	public MatrixDisplayView(MatrixDisplayController controller,
			TraceabilityController traceability) {
		this.traceability = traceability;
		this.controller = controller;

		this.arrowIcon = new ImageIcon("resources/OtherIcons/Traceability.PNG");

		this.initComponents();
		this.initMatrix();
		this.initListeners();

		// TODO: change in netbeans
		this.jSplitPane1.setResizeWeight(1);
		this.jSplitPane2.setResizeWeight(0.5);
		closeEditorView();
	}

	/**
	 * Sets the traceMatrix table to be modeled by our custom handler object
	 * 
	 * @see TraceMatrixTableModel
	 */
	private void initMatrix() {
		this.matrixModel = new TraceMatrixTableModel();
		this.matrix.setModel(this.matrixModel);
		this.matrix.setFocusable(false);
		this.matrix.setCellSelectionEnabled(false);
		this.matrix.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		this.matrix.getTableHeader().setDefaultRenderer(
				new TraceTableCellRenderer());

		this.matrix.setRowHeight(arrowIcon.getIconHeight());
	}

	private void initListeners() {
		this.matrix.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent evnt) {
				int row = MatrixDisplayView.this.matrix
						.convertRowIndexToModel(MatrixDisplayView.this.matrix
								.rowAtPoint(new Point(evnt.getX(), evnt.getY())));
				int col = MatrixDisplayView.this.matrix
						.convertColumnIndexToModel(MatrixDisplayView.this.matrix
								.columnAtPoint(new Point(evnt.getX(), evnt
										.getY())));

				if (col > 0) {
					Requirement rowRequirement = MatrixDisplayView.this.matrixModel
							.getRow(row);
					Requirement colRequirement = MatrixDisplayView.this.matrixModel
							.getColumn(col);
					MatrixDisplayView.this.controller.getRowEditorController()
							.entitySelectionChange(rowRequirement);
					MatrixDisplayView.this.controller.getColEditorController()
							.entitySelectionChange(colRequirement);

					if (MatrixDisplayView.this.matrixModel.getValueAt(row, col) != "x") {
						MatrixDisplayView.this.matrixModel.setValueAt("x", row,
								col);
						MatrixDisplayView.this.controller
								.createTraceabilityAssociation(
										MatrixDisplayView.this.traceability
												.getActiveTraceabilityMatrix(),
										rowRequirement, colRequirement);
					} else {
						MatrixDisplayView.this.matrixModel.setValueAt("", row,
								col);
						MatrixDisplayView.this.controller
								.deleteTraceabilityAssociation(
										MatrixDisplayView.this.traceability
												.getActiveTraceabilityMatrix(),
										rowRequirement, colRequirement);
					}
				}
			}
		});
	}

	/**
	 * 
	 */
	private void updateMatrix() {
		List<TraceabilityAssociation> associations = controller
				.getTraceabilityAssociations();
		List<Requirement> cols = this.controller.getCols();
		List<Requirement> rows = this.controller.getRows();

		this.matrixModel.setCols(cols);
		this.matrixModel.setRows(rows);

		TableColumnModel columnModel = this.matrix.getColumnModel();
		int columnCount = columnModel.getColumnCount();

		if (columnCount > 0) {

			TableColumn headerColumn = columnModel.getColumn(0);
			headerColumn.setCellRenderer(this.matrix.getTableHeader()
					.getDefaultRenderer());

			for (int i = 1; i < columnCount; i++) {
				TableColumn column = columnModel.getColumn(i);
				column.setCellRenderer(this.matrix.getTableHeader()
						.getDefaultRenderer());
			}
		}

		// updateMatrixAssociations();
		for (TraceabilityAssociation ta : associations) {
			int colIndex = matrix.convertRowIndexToView(matrixModel
					.getColIndexOf(ta.getFrom()));
			int rowIndex = matrix.convertRowIndexToView(matrixModel
					.getRowIndexOf(ta.getTo()));

			if (colIndex != -1 && rowIndex != -1)
				MatrixDisplayView.this.matrixModel.setValueAt("x", rowIndex,
						colIndex);
		}
	}

	public void changeRowEntityEditorView(EditorPanelView newEditorView) {
		if (newEditorView == null)
			this.jSplitPane2.setLeftComponent(null);
		// this.jSplitPane2.setDividerLocation(0.0);
		else {
			this.jSplitPane1.setRightComponent(jSplitPane2);
			this.jSplitPane2.setLeftComponent(newEditorView);
			int editorHeight = newEditorView.getPreferredSize().height;
			int splitPaneHeight = this.jSplitPane1.getSize().height;
			this.jSplitPane1.setDividerLocation(splitPaneHeight - editorHeight);
		}
	}

	public void changeColEntityEditorView(EditorPanelView newEditorView) {
		if (newEditorView == null)
			this.jSplitPane2.setRightComponent(null);
		// this.splitPane.setDividerLocation(1.0);
		else {
			this.jSplitPane1.setRightComponent(jSplitPane2);
			this.jSplitPane2.setRightComponent(newEditorView);
			int editorHeight = newEditorView.getPreferredSize().height;
			int splitPaneHeight = this.jSplitPane1.getSize().height;
			this.jSplitPane1.setDividerLocation(splitPaneHeight - editorHeight);
		}
	}

	public void closeEditorView() {
		this.changeRowEntityEditorView(null);
		this.changeColEntityEditorView(null);
		this.jSplitPane1.setRightComponent(null);
		this.jSplitPane1.setDividerLocation(1.0);
	}

	// ///////////////////////////////////////////////////////////////
	// //
	// Modifications below this point should be avoided //
	// //
	// ///////////////////////////////////////////////////////////////

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		jSplitPane1 = new javax.swing.JSplitPane();
		matrixPane = new javax.swing.JPanel();
		jScrollPane1 = new javax.swing.JScrollPane();
		matrix = new javax.swing.JTable();
		jPanel3 = new javax.swing.JPanel();
		jSplitPane2 = new javax.swing.JSplitPane();
		jPanel1 = new javax.swing.JPanel();
		jPanel2 = new javax.swing.JPanel();

		setMaximumSize(new java.awt.Dimension(750, 600));
		setMinimumSize(new java.awt.Dimension(750, 600));
		setPreferredSize(new java.awt.Dimension(750, 600));

		jSplitPane1.setDividerLocation(500);
		jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		jSplitPane1.setOneTouchExpandable(true);

		matrix.setModel(new javax.swing.table.DefaultTableModel(new Object[][] {
				{ null, null, null, null }, { null, null, null, null },
				{ null, null, null, null }, { null, null, null, null } },
				new String[] { "Title 1", "Title 2", "Title 3", "Title 4" }));
		jScrollPane1.setViewportView(matrix);

		javax.swing.GroupLayout matrixPaneLayout = new javax.swing.GroupLayout(
				matrixPane);
		matrixPane.setLayout(matrixPaneLayout);
		matrixPaneLayout
				.setHorizontalGroup(matrixPaneLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								matrixPaneLayout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												jScrollPane1,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												728, Short.MAX_VALUE)
										.addContainerGap()));
		matrixPaneLayout.setVerticalGroup(matrixPaneLayout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				matrixPaneLayout.createSequentialGroup().addContainerGap()
						.addComponent(jScrollPane1,
								javax.swing.GroupLayout.DEFAULT_SIZE, 477,
								Short.MAX_VALUE).addContainerGap()));

		jSplitPane1.setTopComponent(matrixPane);

		jSplitPane2.setDividerLocation(374);
		jSplitPane2.setResizeWeight(0.5);

		javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(
				jPanel1);
		jPanel1.setLayout(jPanel1Layout);
		jPanel1Layout.setHorizontalGroup(jPanel1Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 508,
				Short.MAX_VALUE));
		jPanel1Layout.setVerticalGroup(jPanel1Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 92,
				Short.MAX_VALUE));

		jSplitPane2.setLeftComponent(jPanel1);

		javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(
				jPanel2);
		jPanel2.setLayout(jPanel2Layout);
		jPanel2Layout.setHorizontalGroup(jPanel2Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 233,
				Short.MAX_VALUE));
		jPanel2Layout.setVerticalGroup(jPanel2Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 92,
				Short.MAX_VALUE));

		jSplitPane2.setRightComponent(jPanel2);

		javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(
				jPanel3);
		jPanel3.setLayout(jPanel3Layout);
		jPanel3Layout.setHorizontalGroup(jPanel3Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				jSplitPane2, javax.swing.GroupLayout.Alignment.TRAILING,
				javax.swing.GroupLayout.DEFAULT_SIZE, 748, Short.MAX_VALUE));
		jPanel3Layout.setVerticalGroup(jPanel3Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				jSplitPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 94,
				Short.MAX_VALUE));

		jSplitPane1.setRightComponent(jPanel3);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 750,
				Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 600,
				Short.MAX_VALUE));
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JSplitPane jSplitPane1;
	private javax.swing.JSplitPane jSplitPane2;
	private javax.swing.JTable matrix;
	private javax.swing.JPanel matrixPane;
	// End of variables declaration//GEN-END:variables

}
