/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability;

import edu.wpi.wpisuite.data.TraceabilityMatrix;
import edu.wpi.wpisuite.module.traceability.criteria.CriteriaController;

/**
 * Allows for the editing of a matrix - in name and criteria, specifically.
 * 
 * Does the name itself, but delegates criteria to a pair of CriteriaControllers
 */
public class MatrixEditController implements TraceabilityStepController {

	private TraceabilityController controller;
	private MatrixEditView view;

	public MatrixEditController(TraceabilityController controller,
			CriteriaController rowController, CriteriaController colController) {
		this.controller = controller;
		view = new MatrixEditView(this);

		view.setFromCriteriaView(colController.getView());
		view.setToCriteriaView(rowController.getView());

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.traceability.TraceabilityStepController#getView()
	 */
	@Override
	public MatrixEditView getView() {
		return view;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.traceability.TraceabilityStepController#update()
	 */
	@Override
	public void update() {
		TraceabilityMatrix activeMatrix = controller
				.getActiveTraceabilityMatrix();
		String name = "";
		if (activeMatrix != null)
			name = activeMatrix.getName();
		view.setMatrixName(name);
	}

	public void selectReturn() {
		controller.returnToActiveMatrix();

	}

	public void matrixNameChanged(String matrixName) {
		controller.changeActiveMatrixName(matrixName);
	}

	public String getMatrixName() {
		return view.getMatrixName();
	}

}
