/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import edu.wpi.wpisuite.components.DelayedDocumentListener;
import edu.wpi.wpisuite.components.EntityIconSet;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;

/**
 * 
 */
@SuppressWarnings("serial")
public class MatrixEditView extends JPanel {
	private MatrixEditController controller;

	private EntityIconSet icons = new EntityIconSet();

	public MatrixEditView(MatrixEditController controller) {
		this.controller = controller;
		initComponents();

		headerLabel.setIcon(icons.getTraceabilityMatrixIcon(IconType.DEFAULT));
		returnButton.addActionListener(returnListener);
		matrixNameTextField.getDocument().addDocumentListener(
				matrixNameListener);
	}

	public void setFromCriteriaView(Component fromComponent) {
		contentSplitPane.setLeftComponent(fromComponent);
	}

	public void setToCriteriaView(Component toComponent) {
		contentSplitPane.setRightComponent(toComponent);
	}

	private ActionListener returnListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			controller.selectReturn();
		}
	};

	private DocumentListener matrixNameListener = new DelayedDocumentListener(
			new DocumentListener() {
				@Override
				public void changedUpdate(DocumentEvent e) {
					controller.matrixNameChanged(getMatrixName());
				}

				@Override
				public void insertUpdate(DocumentEvent e) {
					controller.matrixNameChanged(getMatrixName());
				}

				@Override
				public void removeUpdate(DocumentEvent e) {
					controller.matrixNameChanged(getMatrixName());
				}
			}, 3000 // time in ms - will save changes after this long since
					// change
	);

	public String getMatrixName() {
		return matrixNameTextField.getText();
	}

	/**
	 * Does not invoke listeners
	 */
	public void setMatrixName(String newName) {
		matrixNameTextField.getDocument().removeDocumentListener(
				matrixNameListener);

		matrixNameTextField.setText(newName);

		matrixNameTextField.getDocument().addDocumentListener(
				matrixNameListener);
	}

	private void initComponents() {

		headerLabel = new javax.swing.JLabel();
		returnButton = new javax.swing.JButton();
		matrixNameTextField = new javax.swing.JTextField();
		contentSplitPane = new javax.swing.JSplitPane();

		headerLabel.setFont(new java.awt.Font("Tahoma", 0, 16)); // NOI18N
		headerLabel.setText("Traceability Matrix:");

		returnButton.setText("Display...");

		contentSplitPane.setDividerLocation(0.5);
		contentSplitPane.setResizeWeight(0.5);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout
				.setHorizontalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(headerLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(
												matrixNameTextField,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												436, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(returnButton)
										.addContainerGap()).addComponent(
								contentSplitPane,
								javax.swing.GroupLayout.DEFAULT_SIZE, 684,
								Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				layout.createSequentialGroup().addContainerGap().addGroup(
						layout.createParallelGroup(
								javax.swing.GroupLayout.Alignment.BASELINE)
								.addComponent(headerLabel).addComponent(
										matrixNameTextField,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addComponent(returnButton)).addPreferredGap(
						javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(contentSplitPane,
								javax.swing.GroupLayout.DEFAULT_SIZE, 424,
								Short.MAX_VALUE)));
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JSplitPane contentSplitPane;
	private javax.swing.JLabel headerLabel;
	private javax.swing.JTextField matrixNameTextField;
	private javax.swing.JButton returnButton;
	// End of variables declaration
}
