/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sabrina Varanelli - Initial design
 *    Lucas Scotta - Additional design, initial API and implementation
 *    Dan Bjorge - comments and cleanup of controller-view separation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability;

import java.awt.CardLayout;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import edu.wpi.wpisuite.module.ModuleView;
import edu.wpi.wpisuite.module.traceability.TraceabilityController.TraceabilityStep;


/**
 * The view for the traceability matrix module. Defines the overall hierarchy of
 * view classes for the components of the module - the actual parts are defined
 * in their own classes.
 */
@SuppressWarnings("serial")
public class TraceabilityView extends ModuleView {
	/** The module controller for this view */
	private TraceabilityController controller;

	private CardLayout layout;

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.ModuleView#initAddMenuItems()
	 */
	@Override
	public void initAddMenuItems() {
		// Blank by design
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.ModuleView#updateAllComponents()
	 */
	@Override
	public void updateAllComponents() {
		controller.updateSelected();
	}

	/**
	 * Initializes this view and its components.
	 * 
	 * @param dashboard
	 *            The Module this is a view of.
	 */
	public TraceabilityView(TraceabilityController traceability) {
		super(traceability, new ImageIcon(
				"resources/OtherIcons/Traceability.PNG"));
		this.controller = traceability;

		this.initLayout();
	}

	/**
	 * Initializes the main panel which will contain sub-views to actually
	 * contain the sub-views for easy switching
	 */
	private void initLayout() {
		layout = new CardLayout();
		this.setLayout(layout);

		for (TraceabilityStep step : TraceabilityStep.values())
			this.add(step.toString(), controller.getStepView(step));
	}

	/**
	 * Updates the main viewing panel to display the appropriate view for the
	 * current view selection
	 */
	public void selectStep(TraceabilityStep step) {
		layout.show(this, step.toString());
	}

	public void error(String string) {
		JOptionPane.showMessageDialog(null, string, "Error",
				JOptionPane.ERROR_MESSAGE);
	}
}
