package edu.wpi.wpisuite.module.traceability.components;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;


import edu.wpi.wpisuite.components.EntityTreeNode;
import edu.wpi.wpisuite.components.checktree.CheckTree;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.Requirement;
import edu.wpi.wpisuite.data.UserStory;

/**
 * A CheckTree which filters over requirements
 */
@SuppressWarnings("serial")
public class RequirementFilterTree extends CheckTree {
	// private Logger logger = Logger.getLogger(this.getClass());

	public RequirementFilterTree() {
		super();
	}

	public void displayUserStories(Project p) {
		setRootChildren(getUserStoryBaseNodes(p));
	}

	public void displayFeatures(Project p) {
		setRootChildren(getFeatureBaseNodes(p));
	}

	public Set<Requirement> getRequirements() {
		return getChecked(Requirement.class);
	}

	public List<Requirement> getSubRequirements(TreeNode node) {
		return getDescendants(node, Requirement.class);
	}

	private List<MutableTreeNode> getUserStoryBaseNodes(Project p) {
		List<MutableTreeNode> baseNodes = new ArrayList<MutableTreeNode>();

		Map<Integer, EntityTreeNode<Iteration>> itNodeMap = new HashMap<Integer, EntityTreeNode<Iteration>>();

		Map<Integer, EntityTreeNode<Release>> relNodeMap = new HashMap<Integer, EntityTreeNode<Release>>();

		EntityTreeNode<Iteration> backlogNode = new EntityTreeNode<Iteration>(
				null) {
			@Override
			protected String getNullDisplayObject() {
				return "Backlog";
			}
		};
		baseNodes.add(backlogNode);
		itNodeMap.put(null, backlogNode);

		for (Release rel : p.getReleases()) {
			EntityTreeNode<Release> relNode = new EntityTreeNode<Release>(rel);
			baseNodes.add(relNode);
			relNodeMap.put(rel.getID(), relNode);
		}

		for (Iteration it : p.getIterations()) {
			EntityTreeNode<Iteration> itNode = new EntityTreeNode<Iteration>(it);
			itNodeMap.put(it.getID(), itNode);
			Release itRel = it.getRelease();
			if (itRel != null
					&& relNodeMap.containsKey(it.getRelease().getID()))
				relNodeMap.get(it.getRelease().getID()).add(itNode);
			else
				baseNodes.add(itNode);
		}

		for (UserStory us : p.getUserStories()) {
			EntityTreeNode<UserStory> usNode = new EntityTreeNode<UserStory>(us);
			Iteration usIt = us.getIteration();
			if (usIt == null)
				backlogNode.add(usNode);
			else
				itNodeMap.get(us.getIteration().getID()).add(usNode);
		}

		return baseNodes;
	}

	private List<MutableTreeNode> getFeatureBaseNodes(Project p) {
		List<MutableTreeNode> baseNodes = new ArrayList<MutableTreeNode>();

		Map<Integer, EntityTreeNode<Release>> relNodeMap = new HashMap<Integer, EntityTreeNode<Release>>();

		for (Release rel : p.getReleases()) {
			EntityTreeNode<Release> relNode = new EntityTreeNode<Release>(rel);
			baseNodes.add(relNode);
			relNodeMap.put(rel.getID(), relNode);
		}

		for (Feature f : p.getFeatures()) {
			EntityTreeNode<Feature> fNode = new EntityTreeNode<Feature>(f);
			if (f.getRelease() != null)
				relNodeMap.get(f.getRelease().getID()).add(fNode);
			else
				baseNodes.add(fNode);
		}

		return baseNodes;
	}
}
