/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design, API, and implementation
 *    
 *******************************************************************************/
package edu.wpi.wpisuite.module.traceability.components;

import java.awt.Component;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;
import javax.swing.ListModel;


import edu.wpi.wpisuite.components.EntityIconSet;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriteria.RequirementType;

/**
 * List cell renderer to set the appropriate icon for any type of requirement
 * 
 * @author Lucas Scotta
 * @see http://www.java2s.com/Tutorial/Java/0240__Swing/
 *      CreatingaComplexListCellRendererfonticonandcolor.htm
 * @see http://www.famfamfam.com/lab/icons/silk/ for icons
 */
@SuppressWarnings("serial")
public class RequirementTypeListCellRenderer extends DefaultListCellRenderer {
	EntityIconSet icons;

	public RequirementTypeListCellRenderer() {
		icons = new EntityIconSet();
	}

	@Override
	public Component getListCellRendererComponent(JList list, Object value,
			int index, boolean isSelected, boolean cellHasFocus) {

		super.getListCellRendererComponent(list, value, index, isSelected,
				cellHasFocus);

		ListModel m = list.getModel();

		Object obj = m.getElementAt(index);

		if (obj != null)
			if (obj instanceof RequirementType)
				if (((RequirementType) obj).getRequirementClass().equals(
						Feature.class))
					setIcon(icons.getFeatureIcon(IconType.DEFAULT));
				else if (((RequirementType) obj).getRequirementClass().equals(
						UserStory.class))
					setIcon(icons.getUserStoryIcon(IconType.DEFAULT));

		return this;
	}

}
