/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - initial API and implementation
 *    Dan Bjorge - additional implementation (for consistency across rows, cols)
 *******************************************************************************/

//This file was based heavily upon the code at
//  http://forums-beta.sun.com/thread.jspa?messageID=2970549
//which, as of this writing 5/1/2009, was unlicensed
package edu.wpi.wpisuite.module.traceability.components;

import java.awt.Component;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.plaf.BorderUIResource;
import javax.swing.table.DefaultTableCellRenderer;


import edu.wpi.wpisuite.components.EntityIconSet;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.UserStory;

/**
 * The renderer for the row and column headers of the traceability matrix table,
 * customized to diplay consistently for both types of header
 */
@SuppressWarnings("serial")
public class TraceTableCellRenderer extends DefaultTableCellRenderer {
	protected Border noFocusBorder, focusBorder;

	EntityIconSet icons = new EntityIconSet();

	/**
	 * Constructs a new renderer and sets some default custom values
	 */
	public TraceTableCellRenderer() {
		setOpaque(true);
		setBorder(noFocusBorder);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.DefaultTableCellRenderer#updateUI()
	 */
	@Override
	public void updateUI() {
		super.updateUI();
		Border cell = UIManager.getBorder("TableHeader.cellBorder");
		Border focus = UIManager.getBorder("Table.focusCellHighlightBorder");

		focusBorder = new BorderUIResource.CompoundBorderUIResource(cell, focus);

		Insets i = focus.getBorderInsets(this);

		noFocusBorder = new BorderUIResource.CompoundBorderUIResource(cell,
				BorderFactory.createEmptyBorder(i.top, i.left, i.bottom,
						i.right));

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.swing.table.DefaultTableCellRenderer#getTableCellRendererComponent
	 * (javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
	 */
	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean selected, boolean focused, int row, int column) {
		setHorizontalAlignment(javax.swing.SwingConstants.CENTER);

		if (table != null && table.getModel() instanceof TraceMatrixTableModel) {
			TraceMatrixTableModel model = (TraceMatrixTableModel) table
					.getModel();

			if (selected) {
				setBackground(table.getSelectionBackground());
				setForeground(table.getSelectionForeground());
			} else {
				setBackground(table.getBackground());
				setForeground(table.getForeground());
			}
			setFont(table.getFont());
			setEnabled(table.isEnabled());

			int mcolumn = table.convertColumnIndexToModel(column);
			int mrow = table.convertRowIndexToModel(row);

			if (mcolumn == 0 && mrow >= 0) {
				if (model.getRow(mrow) instanceof UserStory) {
					setIcon(icons.getUserStoryIcon(IconType.DEFAULT));
					setValue(value);
				} else if (model.getRow(mrow) instanceof Feature) {
					setIcon(icons.getFeatureIcon(IconType.DEFAULT));
					setValue(value);
				}
			} else if (mcolumn != 0 && mrow == -1) {
				if (model.getColumn(mcolumn) instanceof UserStory) {
					setIcon(icons.getUserStoryIcon(IconType.DEFAULT));
					setValue(value);
				} else if (model.getColumn(mcolumn) instanceof Feature) {
					setIcon(icons.getFeatureIcon(IconType.DEFAULT));
					setValue(value);
				}
			} else if (mrow >= 0 && mcolumn > 0
					&& model.getValueAt(mrow, mcolumn) != null) {
				if (model.getValueAt(mrow, mcolumn).equals("x")) {
					setIcon(icons.getTraceIcon());
					setValue("");
				} else {
					setIcon(null);
					setValue(null);
				}
			} else {
				setIcon(null);
				setValue(null);
			}

			// TODO
			/*
			 * if (column != -1 && row != -1) { if
			 * (table.convertColumnIndexToModel(column) == 0 &&
			 * model.getRow(table.convertRowIndexToModel(row)) instanceof
			 * UserStory) { setIcon(icons.getUserStoryIcon(IconType.DEFAULT)); }
			 * else if (table.convertColumnIndexToModel(column) == 0 &&
			 * model.getRow(table.convertRowIndexToModel(row)) instanceof
			 * Feature) { setIcon(icons.getUserStoryIcon(IconType.DEFAULT)); } }
			 */

			/*
			 * if (table.convertColumnIndexToModel(column) == 0 &&
			 * table.convertRowIndexToModel(row) >= 0) { // header column
			 * Requirement r = model.getRow(table.convertRowIndexToModel(row));
			 * if (r instanceof UserStory) {
			 * setIcon(icons.getUserStoryIcon(IconType.DEFAULT)); } else if (r
			 * instanceof Feature) {
			 * setIcon(icons.getFeatureIcon(IconType.DEFAULT)); } } else if
			 * (table.convertColumnIndexToModel(column) > 0 &&
			 * table.convertRowIndexToModel(row) == 0) { // header column
			 * Requirement r =
			 * model.getColumn(table.convertColumnIndexToModel(column)); if (r
			 * instanceof UserStory) {
			 * setIcon(icons.getUserStoryIcon(IconType.DEFAULT)); } else if (r
			 * instanceof Feature) {
			 * setIcon(icons.getFeatureIcon(IconType.DEFAULT)); } } else { //
			 * traceability column if (row > 0 && column > 0) { Object val =
			 * model.getValueAt(row, column);//getValueAt(row, column); if (val
			 * instanceof String && ((String)val).equals("x")) {
			 * setIcon(icons.getTraceIcon()); } else { setIcon(null);
			 * setText(""); } } }
			 */

		}

		if (focused)
			setBorder(focusBorder);
		else
			setBorder(noFocusBorder);

		return this;
	}
}