/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability.criteria;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.GroupLayout.Alignment;


import org.apache.log4j.Logger;

import edu.wpi.wpisuite.data.TraceabilityMatrixCriteria.RequirementType;
import edu.wpi.wpisuite.module.traceability.components.RequirementTypeListCellRenderer;

/**
 * 
 */
@SuppressWarnings("serial")
public class CriteriaPanelView extends JPanel {
	private CriteriaController controller;

	private Logger logger = Logger.getLogger(this.getClass());

	public CriteriaPanelView(CriteriaController criteriaController) {
		this.controller = criteriaController;
		initComponents();
		axisTypeLabel.setText(controller.getAxisDescription());
		initRequirementTypeComboBox();
	}

	private void initRequirementTypeComboBox() {
		requirementTypeComboBox.setModel(new DefaultComboBoxModel(
				RequirementType.values()));
		requirementTypeComboBox
				.setRenderer(new RequirementTypeListCellRenderer());

		requirementTypeComboBox.addActionListener(requirementTypeListener);

		updateRequirementType();
	}

	public void updateRequirementType() {
		requirementTypeComboBox.removeActionListener(requirementTypeListener);
		requirementTypeComboBox
				.setSelectedItem(controller.getRequirementType());
		requirementTypeComboBox.addActionListener(requirementTypeListener);
	}

	public void error(String message) {
		logger.error(message);
		JOptionPane.showMessageDialog(null, message, "Error",
				JOptionPane.ERROR_MESSAGE);
	}

	public void setRootCriterionView(CriterionView v) {
		criterionScrollPane.setViewportView(v);
	}

	private ActionListener requirementTypeListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			requirementTypeSelected(e);
		}
	};

	private void requirementTypeSelected(ActionEvent e) {
		controller.requirementTypeSelected(getSelectedRequirementType());
	}

	public RequirementType getSelectedRequirementType() {
		return (RequirementType) requirementTypeComboBox.getSelectedItem();
	}

	private void initComponents() {
		requirementTypeComboBox = new javax.swing.JComboBox();
		axisTypeLabel = new javax.swing.JLabel();
		criterionScrollPane = new javax.swing.JScrollPane();

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				layout.createSequentialGroup().addContainerGap().addComponent(
						axisTypeLabel).addGap(5, 5, Short.MAX_VALUE)
						.addComponent(requirementTypeComboBox,
								javax.swing.GroupLayout.PREFERRED_SIZE, 100,
								javax.swing.GroupLayout.PREFERRED_SIZE)
						.addContainerGap()).addGroup(
				layout.createSequentialGroup().addContainerGap().addComponent(
						criterionScrollPane).addContainerGap()));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(
																axisTypeLabel)
														.addComponent(
																requirementTypeComboBox,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												criterionScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												476, Short.MAX_VALUE)
										.addContainerGap()));
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JComboBox requirementTypeComboBox;
	private javax.swing.JLabel axisTypeLabel;
	private javax.swing.JScrollPane criterionScrollPane;
	// End of variables declaration//GEN-END:variables

}
