/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability.criteria;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import edu.wpi.wpisuite.data.TraceabilityMatrixCriterion;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriterion.CriterionType;

/**
 * 
 */
public class CriterionControllerFactory {
	private CriterionControllerContainer container;

	private Map<CriterionType, Class<? extends CriterionController<?>>> controllerClassMap;

	private Logger logger = Logger.getLogger(this.getClass());

	public CriterionControllerFactory(CriterionControllerContainer container) {
		this.container = container;

		initControllerClassMap();
	}

	private void initControllerClassMap() {
		controllerClassMap = new HashMap<CriterionType, Class<? extends CriterionController<?>>>();

		controllerClassMap.put(CriterionType.REQUIREMENT,
				RequirementController.class);
	}

	public CriterionController<?> constructFor(
			TraceabilityMatrixCriterion rootCriterion) {
		if (rootCriterion == null)
			return null;

		if (controllerClassMap.containsKey(rootCriterion.getType())) {
			Class<? extends CriterionController<?>> clz = controllerClassMap
					.get(rootCriterion.getType());
			try {
				return clz.getConstructor(CriterionControllerContainer.class,
						TraceabilityMatrixCriterion.class).newInstance(
						container, rootCriterion);
			} catch (Exception e) {
				logger.error("Failed to instantiate criterion constructor "
						+ clz.getSimpleName());
			}
		}

		return null;
	}

	public CriterionController<?> constructNew(CriterionType type) {
		if (type == null)
			return null;

		if (controllerClassMap.containsKey(type)) {
			Class<? extends CriterionController<?>> clz = controllerClassMap
					.get(type);
			try {
				return clz.getConstructor(CriterionControllerContainer.class,
						TraceabilityMatrixCriterion.class).newInstance(
						container, null);
			} catch (Exception e) {
				logger.error("Failed to instantiate criterion constructor "
						+ clz.getSimpleName());
			}
		}

		return null;
	}

	public CriterionController<?> constructDefaultRoot() {
		return new RequirementController(container, null);
	}

}
