/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability.criteria;

import java.util.List;
import java.util.Set;

import edu.wpi.wpisuite.data.Requirement;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriterion;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriteria.RequirementType;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriterion.CriterionType;

/**
 * 
 */
public class RequirementController extends CriterionController<RequirementView> {

	public RequirementController(CriterionControllerContainer container,
			TraceabilityMatrixCriterion criterion) {
		super(container, criterion, RequirementView.class);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * module.traceability.criteria.CriterionController#getSelectedRequirements
	 * (data.Project)
	 */
	@Override
	public Set<Requirement> getSelectedRequirements() {
		return getView().getSelectedRequirements();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.traceability.criteria.CriterionController#initCriterion()
	 */
	@Override
	public void initCriterion() {
		RequirementType type = container.getRequirementType();
		switch (type) {
		case USER_STORY:
			getView().displayUserStories(container.getState().getProject());
			break;
		case FEATURE:
			getView().displayFeatures(container.getState().getProject());
			break;
		}

		view.setSelectedRequirements(this.getCriterionEntities(container
				.getRequirementType().getRequirementClass()));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.traceability.criteria.CriterionController#initNewCriterion()
	 */
	@Override
	public void initNewCriterion() {
		this.criterion.setType(CriterionType.REQUIREMENT);
		this.criterion.setTypeData(new int[0]);

		initCriterion();
	}

	public void requirementsSelected(List<Requirement> changedItems) {
		addEntitiesToCriterion(changedItems);
	}

	public void requirementsUnselected(List<Requirement> changedItems) {
		removeEntitiesFromCriterion(changedItems);
	}

}
