/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability.criteria;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.JScrollPane;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;

import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Requirement;
import edu.wpi.wpisuite.module.traceability.components.RequirementFilterTree;

/**
 * 
 */
@SuppressWarnings("serial")
public class RequirementView extends CriterionView {
	RequirementFilterTree filterTree;
	RequirementController controller;

	public RequirementView(RequirementController controller) {
		super(controller);
		this.controller = controller;

		initTree();

		initLayout();

	}

	private void initTree() {
		filterTree = new RequirementFilterTree();
		filterTree.getCheckingModel().addTreeSelectionListener(checkListener);
	}

	private TreeSelectionListener checkListener = new TreeSelectionListener() {
		@Override
		public void valueChanged(TreeSelectionEvent e) {
			DefaultMutableTreeNode changedNode = (DefaultMutableTreeNode) e
					.getPath().getLastPathComponent();
			List<Requirement> changedItems = filterTree
					.getSubRequirements(changedNode);
			if (e.isAddedPath())
				controller.requirementsSelected(changedItems);
			else
				controller.requirementsUnselected(changedItems);
		}
	};

	public void setSelectedRequirements(List<? extends Requirement> list) {
		filterTree.checkPathsTo(new ArrayList<Requirement>(list));
	}

	public Set<Requirement> getSelectedRequirements() {
		return filterTree.getRequirements();
	}

	public void displayUserStories(Project p) {
		filterTree.displayUserStories(p);
	}

	public void displayFeatures(Project p) {
		filterTree.displayFeatures(p);
	}

	private void initLayout() {
		// JViewPort pane = new javax.swing.;
		JScrollPane scroller = new JScrollPane();
		scroller.setViewportView(filterTree);
		this.add(scroller);
		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				javax.swing.GroupLayout.Alignment.TRAILING,
				layout.createSequentialGroup().addComponent(scroller)));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				javax.swing.GroupLayout.Alignment.TRAILING,
				layout.createSequentialGroup().addComponent(scroller)));

	}

}
