/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - initial API and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.velocity;

import java.awt.event.KeyEvent;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.funcs.IterationManager;
import edu.wpi.wpisuite.module.ActiveState;
import edu.wpi.wpisuite.module.ActiveStateEvent;
import edu.wpi.wpisuite.module.ActiveStateListener;
import edu.wpi.wpisuite.module.Module;
import edu.wpi.wpisuite.module.ActiveStateEvent.ActiveStateEventType;

/**
 * Defines a module which allows the user to create a Velocity Report
 */
public class Velocity extends Module implements ActiveStateListener {

	/** The view for this module controller */
	private VelocityView view;

	/** The display name of this module */
	public final static String moduleName = "Velocity Report";

	/**
	 * Just calls generic module constructor and generates a new view
	 * 
	 * @param wpiSuite
	 *            The main application object
	 */
	public Velocity(ActiveState state) {
		super(state);
		state.registerListener(this);
		this.view = new VelocityView(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.Module#getView()
	 */
	@Override
	public VelocityView getView() {
		return this.view;
	}

	@Override
	public boolean shouldBeEnabled() {
		return getState().getProject() != null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.Module#getModuleName()
	 */
	@Override
	public String getModuleName() {
		return moduleName;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.Module#getModuleKeyShortcut()
	 */
	@Override
	public int getModuleKeyShortcut() {
		return KeyEvent.VK_V;
	}

	/**
	 * Generates a list of the iterations which are currently selected in the
	 * view and orders them appropriately for displaying
	 * 
	 * @return The iterations to display
	 */
	public List<Iteration> getOrderedIterations() {
		if (view == null)
			return Collections.emptyList();

		return IterationManager.sort(view.getSelectedIterations());
	}

	public List<Iteration> getSelectableIterations() {
		return IterationManager.getOrderedIterations(getState().getProject());
	}

	@Override
	public void stateChange(ActiveStateEvent event) {
		if (this == this.getState().getModule() && event.getType().equals(ActiveStateEventType.PROJECT))
			view.updateAllComponents();
	}

	public void updateSelectedIterations(Collection<Iteration> iterations) {
		view.updateSelectedIterations();
	}
}
