/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - initial API and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.velocity;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;


import edu.wpi.wpisuite.components.EntityTreeNode;
import edu.wpi.wpisuite.components.checktree.CheckTree;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Release;

/**
 * A CheckTree which filters over a group of iterations
 */
@SuppressWarnings("serial")
public class VelocityFilterTree extends CheckTree {
	private Logger logger = Logger.getLogger(this.getClass());

	public VelocityFilterTree() {
		super();
	}

	public void setIterations(List<Iteration> iterations) {
		setRootChildren(getBaseNodes(iterations));
	}

	public Set<Iteration> getIterations() {
		return getChecked(Iteration.class);
	}

	public List<EntityTreeNode<?>> getBaseNodes(List<Iteration> iterations) {
		List<EntityTreeNode<?>> nodes = new ArrayList<EntityTreeNode<?>>();
		Map<Integer, EntityTreeNode<Release>> releaseNodes = new HashMap<Integer, EntityTreeNode<Release>>();

		for (Iteration it : iterations) {
			if (it == null) {
				logger.error("Attempted to display null iteration");
				continue;
			}

			Release rel = it.getRelease();
			EntityTreeNode<Iteration> itNode = new EntityTreeNode<Iteration>(it);

			if (rel == null)
				nodes.add(itNode);
			else if (releaseNodes.containsKey(rel.getID()))
				releaseNodes.get(rel.getID()).add(itNode);
			else {
				EntityTreeNode<Release> relNode = new EntityTreeNode<Release>(
						rel);
				relNode.add(itNode);
				releaseNodes.put(rel.getID(), relNode);
				nodes.add(relNode);
			}
		}

		return nodes;
	}
}
